
#ifdef _XBOX
	#include <xtl.h>
#endif

#ifdef _WIN32
	#ifndef _XBOX
		#define WIN32_LEAN_AND_MEAN
		#include "windows.h"
	#endif
#else
    #include <dirent.h>
#endif

#include "global.h"
#include <ctype.h>
#define stricmp(str1,str2) memcmp(str1,str2,strlen(str1))

extern char *convertPath(char *source);

char * lowercase(char * name)
{
	for(unsigned int k = 0; k < strlen(name); k++)
	{
		name[k] = tolower(name[k]);
	}

	return name;
}

List::List()
{
	head = NULL;
	current = NULL;
	count = 0;
}


List::~List()
{
	ListEntry *temp, *del;

	temp = head;
	for(int k = 0; k < count; k++)
	{
		del = temp;
		temp = temp->next;
		delete del;
	}
}

ListEntry * List::GetIndex(int index)
{
	ListEntry * temp = head;
	for(int k = 0; k < index; k++)
		temp = temp->next;

	return temp;
}

void List::SetCurrent(int index)
{
	current = head;
	for(int k = 0; k < index; k++)
		current = current->next;
}

int List::GetCurrentIndex()
{
	ListEntry * temp = head;
	int index = 0;

	while(temp != current)
	{
		index++;
		temp = temp->next;
	}

	return index;
}

void List::BubbleSort()
{
	if(count < 2)
		return;

	bool fDone = false;

	while(!fDone)
	{
		fDone = true;
		ListEntry * leftnode = head;
		ListEntry * rightnode = head->next;

		for(int k = 0; k < count - 1; k++)
		{
			if(stricmp(leftnode->key, rightnode->key) > 0)
			{
				fDone = false;

				if(leftnode == head)  //if node is head
					head = rightnode;

				leftnode->next = rightnode->next;
				rightnode->prev = leftnode->prev;
				leftnode->prev->next = rightnode;
				rightnode->next->prev = leftnode;
				leftnode->prev = rightnode;
				rightnode->next = leftnode;

				ListEntry * temp = leftnode;
				leftnode = rightnode;
				rightnode = temp;
			}

			leftnode = rightnode;
			rightnode = rightnode->next;
		}
	}
}

void List::random()
{
	if(count <= 1)
		return;

	if(head)
	{
		int r = rand() % (count - 1) + 1;
		
		for(int i = 0; i < r; i++)
			current = current->next;
	}
}



MapList::MapList()
{
	bool error = true;

#ifdef _WIN32
	WIN32_FIND_DATA	finddata;
	HANDLE			findhandle;

#ifdef _XBOX
	char * szPath = "d:\\maps\\";
#else
	char * szPath = "maps/";
#endif

	char szScanPath[32];
	sprintf(szScanPath, "%s*.map", szPath);

	if((findhandle = FindFirstFile(szScanPath, &finddata)) == INVALID_HANDLE_VALUE)
	{
		printf("Can't find map files.");
		exit(0);
	}

	char szFilePath[64];
	sprintf(szFilePath, "%s%s", szPath, lowercase(finddata.cFileName));

	char * pszMapKey = GetMapKey(szFilePath, 0);
	head = new NameEntry(szFilePath, pszMapKey);
	delete pszMapKey;

	head->next = head;
	head->prev = head;
	current = head;
	error = false;
	count++;

	while(FindNextFile(findhandle, &finddata))
	{
		sprintf(szFilePath, "%s%s", szPath, lowercase(finddata.cFileName));

		char * pszMapKey = GetMapKey(szFilePath, 0);
		current->next = new NameEntry(szFilePath, pszMapKey);
		delete pszMapKey;

		current->next->prev = current;
		current = current->next;
		current->next = head;
		head->prev = current;
		count++;
	} 
	
	FindClose(findhandle);

#ifdef _DEBUG

#ifdef _XBOX
	char * szTestPath = "d:\\maps\\test\\";
#else
	char * szTestPath = "maps/test/";
#endif

	sprintf(szScanPath, "%s*.map", szTestPath);

	if((findhandle = FindFirstFile(szScanPath, &finddata)) != INVALID_HANDLE_VALUE)
	{
		do
		{
			sprintf(szFilePath, "%s%s", szTestPath, lowercase(finddata.cFileName));

			char * pszMapKey = GetMapKey(szFilePath, 5);
			current->next = new NameEntry(szFilePath, pszMapKey);
			delete pszMapKey;

			current->next->prev = current;
			current = current->next;
			current->next = head;
			head->prev = current;
			count++;

		} while(FindNextFile(findhandle, &finddata));
	}

	FindClose(findhandle);

#endif

#else

    DIR * dhandle;
    dhandle=opendir("maps/");
    if (dhandle==NULL) 
    {
	    printf("Can't find map files.");
	    exit(0);
    }

    current=NULL;
    struct dirent * curent;
    char fname[512];
    while ((curent=readdir(dhandle))!=NULL)
    {
        //check to see if it ends in .map
        int p=strlen(curent->d_name);
        if (p>4)
        {
            if (strncmp((curent->d_name)+p-4,".map",4)==0)
            {
                //its a map
                sprintf(fname,"%s%s","maps/",curent->d_name);
    			char * pszMapKey = GetMapKey(fname, 0);
                if (current!=NULL)
                {
                    current->next=new NameEntry(fname,pszMapKey);
                    current->next->prev=current;
                    current=current->next;
                    current->next=head;
                    head->prev=current;
                }
                else
                {
                    current=new NameEntry(fname,pszMapKey);
                    current->next=current;
                    current->prev=current;
                    head=current;
                }
                delete pszMapKey;
                count++;
            }
        }
    }
    closedir(dhandle);

	error = false;
#endif



	BubbleSort();

	current = head;
}

MapList::~MapList()
{}

void MapList::add(char * name)
{
	//first scan to see if we already have that name
	ListEntry * find = head;
	char filename[256];

#ifdef _XBOX
	char * szPath = "d:\\maps\\";
#else
	char * szPath = "maps/";
#endif

	sprintf(filename, "%s%s", szPath, name);

	while(true)
	{
		if(!strcmp(find->name, filename))
		{
			current = find;
			return;
		}

		find = find->next;
		if(find == head)
			break;
	}

	//if not, add a new entry for this map name
	ListEntry * tail = head->prev;
	tail->next = new NameEntry(filename, "");
	tail->next->prev = tail;
	current = tail->next;
	current->next = head;
	head->prev = current;
	count++;
}

bool MapList::find(char * name)
{
	ListEntry * temp = current;
	
	do
	{
		current = current->next;

		if(strstr(current->name, name))
			return true;

	} while(current != temp);

	return false;
}

bool MapList::startswith(char letter)
{
	ListEntry * temp = current;
	
	do
	{
		current = current->next;

		if(current->key[0] == letter)
			return true;

	} while(current != temp);

	return false;
}


char * MapList::GetMapKey(char * name, int offset)
{
	char * mapkey = new char[strlen(name)];
	char mapname[64];

#ifdef _XBOX
	sprintf(mapname, name + 8 + offset);
#else
	sprintf(mapname, name + 5 + offset);
#endif

	for(int i = (int)strlen(mapname); i >= 0; i--)
	{
		if(mapname[i] == '.')
		{
			mapname[i] = '\0';
			break;
		}
	}

	char * pszFindUnderscore = mapname;
	bool fFoundUnderscore = false;

	while(*pszFindUnderscore != '\0')
	{
		if(*pszFindUnderscore == '_')
		{
			fFoundUnderscore = true;
			pszFindUnderscore++;
			break;
		}

		pszFindUnderscore++;
	}

	if(fFoundUnderscore)
		strcpy(mapkey, pszFindUnderscore);
	else
		strcpy(mapkey, mapname);

	return mapkey;
}

void MapList::printlist()
{
	ListEntry * temp = current;
	
	printf("%s\n", current->name);

	do
	{
		current = current->next;
		printf("%s\n", current->name);

	} while(current != temp);
}

SkinList::SkinList()
{

#ifdef _WIN32
	WIN32_FIND_DATA	finddata;
	HANDLE			findhandle;

#ifdef _XBOX
	char * szPath = "d:\\gfx\\skins\\";
#else
	char * szPath = "gfx/skins/";
#endif

	char szScanPath[32];
	sprintf(szScanPath, "%s*.bmp", szPath);

	if((findhandle = FindFirstFile(szScanPath, &finddata)) == INVALID_HANDLE_VALUE)
	{
		printf("Can't find skin files.");
		exit(0);
	}
	
	char szFilePath[64];

	do
	{
		sprintf(szFilePath, "%s%s", szPath, finddata.cFileName);

		if(head == NULL)
		{
			head = new NameEntry(szFilePath, szFilePath);
			head->next = head;
			head->prev = head;
			current = head;
		}
		else
		{
			current->next = new NameEntry(szFilePath, szFilePath);
			current->next->prev = current;
			current = current->next;
			current->next = head;
			head->prev = current;
		}
		count++;
	} 
	while(FindNextFile(findhandle, &finddata));
	
	FindClose(findhandle);
#else

    DIR * dhandle;
    dhandle=opendir("gfx/skins/");
    if (dhandle==NULL) 
    {
        puts("can't find skins.");
        exit(1);
    }

    current=NULL;
    count=0;
    struct dirent * curent;
    char fname[512];
    while ((curent=readdir(dhandle))!=NULL)
    {
        //check to see if it ends in .map
        int p=strlen(curent->d_name);
        if (p>4)
        {
            if (strncmp((curent->d_name)+p-4,".bmp",4)==0)
            {
                //its a map
                sprintf(fname,"%s%s","gfx/skins/",curent->d_name);
                if (current!=NULL)
                {
                    current->next=new NameEntry(fname,fname);
                    current->next->prev=current;
                    current=current->next;
                    current->next=head;
                    head->prev=current;
                }
                else
                {
                    current=new NameEntry(fname,"");
                    current->next=current;
                    current->prev=current;
                    head=current;
                }
                count++;
            }
        }
    }
    closedir(dhandle);
	
#endif

	BubbleSort();
	current = head;
}

SkinList::~SkinList()
{}

AnnouncerList::AnnouncerList()
{

#ifdef _WIN32
	WIN32_FIND_DATA	finddata;
	HANDLE			findhandle;

#ifdef _XBOX
	char * szPath = "d:\\sfx\\announcer\\";
#else
	char * szPath = "sfx/announcer/";
#endif

	char szScanPath[32];
	sprintf(szScanPath, "%s*.txt", szPath);

	if((findhandle = FindFirstFile(szScanPath, &finddata)) == INVALID_HANDLE_VALUE)
	{
		printf("Can't find announcer files.");
		exit(0);
	}
	
	char szFilePath[64];

	do
	{
		sprintf(szFilePath, "%s%s", szPath, finddata.cFileName);

		if(head == NULL)
		{
			head = new NameEntry(szFilePath, "");
			head->next = head;
			head->prev = head;
			current = head;
		}
		else
		{
			current->next = new NameEntry(szFilePath, "");
			current->next->prev = current;
			current = current->next;
			current->next = head;
			head->prev = current;
		}
		count++;
	} 
	while(FindNextFile(findhandle, &finddata));
	
	FindClose(findhandle);
#else
    DIR * dhandle;
    dhandle=opendir("sfx/announcer/");
    if (dhandle==NULL) 
    {
        puts("can't find announcer files.");
        exit(1);
    }

    current=NULL;
    count=0;
    struct dirent * curent;
    char fname[512];
    while ((curent=readdir(dhandle))!=NULL)
    {
        //check to see if it ends in .map
        int p=strlen(curent->d_name);
        if (p>4)
        {
            if (strncmp((curent->d_name)+p-4,".txt",4)==0)
            {
                //its a map
                sprintf(fname,"%s%s","sfx/announcer/",curent->d_name);
                if (current!=NULL)
                {
                    current->next=new NameEntry(fname,"");
                    current->next->prev=current;
                    current=current->next;
                    current->next=head;
                    head->prev=current;
                }
                else
                {
                    current=new NameEntry(fname,fname);
                    current->next=current;
                    current->prev=current;
                    head=current;
                }
                count++;
            }
        }
    }
    closedir(dhandle);
#endif

	current = head;
}

AnnouncerList::~AnnouncerList()
{}
/*
MusicPlayList::MusicPlayList()
{

#ifdef _WIN32
	WIN32_FIND_DATA	finddata;
	HANDLE			findhandle;

#ifdef _XBOX
	char * szPath = "d:\\music\\";
#else
	char * szPath = "music/";
#endif

	char szScanPath[32];
	sprintf(szScanPath, "%s*.ogg", szPath);

	if((findhandle = FindFirstFile(szScanPath, &finddata)) == INVALID_HANDLE_VALUE)
	{
		printf("Can't find music files.");
		exit(0);
	}
	
	char szFilePath[64];

	do
	{
		sprintf(szFilePath, "%s%s", szPath, finddata.cFileName);

		if(head == NULL)
		{
			head = new NameEntry(szFilePath, "");
			head->next = head;
			head->prev = head;
			current = head;
		}
		else
		{
			current->next = new NameEntry(szFilePath, "");
			current->next->prev = current;
			current = current->next;
			current->next = head;
			head->prev = current;
		}
		count++;
	} 
	while(FindNextFile(findhandle, &finddata));
	
	FindClose(findhandle);
#else
    DIR * dhandle;
    dhandle=opendir("music/");
    if (dhandle==NULL) 
    {
        puts("can't find music files.");
        exit(1);
    }

    current=NULL;
    count=0;
    struct dirent * curent;
    char fname[512];
    while ((curent=readdir(dhandle))!=NULL)
    {
        //check to see if it ends in .map
        int p=strlen(curent->d_name);
        if (p>4)
        {
            if (strncmp((curent->d_name)+p-4,".ogg",4)==0)
            {
                //its a map
                sprintf(fname,"%s%s","music/",curent->d_name);
                if (current!=NULL)
                {
                    current->next=new NameEntry(fname,"");
                    current->next->prev=current;
                    current=current->next;
                    current->next=head;
                    head->prev=current;
                }
                else
                {
                    current=new NameEntry(fname,"");
                    current->next=current;
                    current->prev=current;
                    head=current;
                }
                count++;
            }
        }
    }
    closedir(dhandle);
#endif

	current = head;
}

MusicPlayList::~MusicPlayList()
{}
*/

MusicList::MusicList()
{
	CurrentMusic = NULL;

#ifdef _WIN32
	WIN32_FIND_DATA	finddata;
	HANDLE			findhandle;

#ifdef _XBOX
	char * szPath = "d:\\music\\";
#else
	char * szPath = "music/";
#endif

	char szScanPath[32];
	sprintf(szScanPath, "%s*.txt", szPath);

	if((findhandle = FindFirstFile(szScanPath, &finddata)) == INVALID_HANDLE_VALUE)
	{
		printf("Can't find music configuration files.");
		exit(0);
	}
	
	char szFilePath[64];

	do
	{
		sprintf(szFilePath, "%s%s", szPath, finddata.cFileName);

		if(head == NULL)
		{
			head = new MusicEntry(szFilePath);
			head->next = head;
			head->prev = head;
			current = head;
		}
		else
		{
			current->next = new MusicEntry(szFilePath);
			current->next->prev = current;
			current = current->next;
			current->next = head;
			head->prev = current;
		}
		count++;
	} 
	while(FindNextFile(findhandle, &finddata));
	
	FindClose(findhandle);
#else
    DIR * dhandle;
    dhandle=opendir("music/");
    if (dhandle==NULL) 
    {
        puts("Can't find music configuration files.");
        exit(1);
    }

    current=NULL;
    count=0;
    struct dirent * curent;
    char fname[512];
    while ((curent=readdir(dhandle))!=NULL)
    {
        //check to see if it ends in .txt
        int p=strlen(curent->d_name);
        if (p>4)
        {
            if (strncmp((curent->d_name)+p-4,".txt",4)==0)
            {
                //its a music config
                sprintf(fname,"%s%s","music/",curent->d_name);
                if (current!=NULL)
                {
                    current->next=new MusicEntry(fname);
                    current->next->prev=current;
                    current=current->next;
                    current->next=head;
                    head->prev=current;
                }
                else
                {
                    current=new MusicEntry(fname);
                    current->next=current;
                    current->prev=current;
                    head=current;
                }
                count++;
            }
        }
    }
    closedir(dhandle);
#endif

	current = head;
}

MusicList::~MusicList()
{}

char * MusicList::GetMusic(int musicID)
{
	return ((MusicEntry*)current)->GetMusic(musicID);
}

void MusicList::SetRandomMusic(int backgroundID)
{
	CurrentMusic = ((MusicEntry*)current)->GetRandomMusic(backgroundID);
}

char * MusicList::GetCurrentMusic()
{
	return CurrentMusic;
}

MusicEntry::MusicEntry(char *szFile)
{
	int i, k;

	for(k = 0; k < MAXBACKGROUNDID; k++)
		numsongsforbackground[k] = 0;

	char musiclistname[64];
#ifdef _XBOX
	sprintf(musiclistname, szFile + 9);
#else
	sprintf(musiclistname, szFile + 6);
#endif

	for(i = (int)strlen(musiclistname); i >= 0; i--)
	{
		if(musiclistname[i] == '.')
		{
			musiclistname[i] = '\0';
			break;
		}
	}

	name = new char[strlen(musiclistname)+1];
	sprintf(name, "%s", musiclistname);

	key = new char[1];
	key[0] = '\0';

	FILE * in = fopen(szFile, "r");

	char szBuffer[256];

	nummusicfiles = 0;
	while(fgets(szBuffer, 256, in))
	{
		//Ignore comment lines
		if(szBuffer[0] == '#' || szBuffer[0] == ' ' || szBuffer[0] == '\t' || szBuffer[0] == '\n' || szBuffer[0] == '\r')
			continue;

		nummusicfiles++;
	}

	rewind(in);

	songfilenames = new char*[nummusicfiles];

	for(k = 0; k < MAXBACKGROUNDID; k++)
		songsforbackground[k] = new int[nummusicfiles];

	int iNumFile = 0;
	while(fgets(szBuffer, 256, in))
	{
		//Ignore comment lines
		if(szBuffer[0] == '#' || szBuffer[0] == ' ' || szBuffer[0] == '\t' || szBuffer[0] == '\n' || szBuffer[0] == '\r')
			continue;

		//Find first comma
		int startIndex = 0;
		int stringLength = strlen(szBuffer);
		for(k = 0; k < stringLength; k++)
		{
			if(szBuffer[k] == ',' || szBuffer[k] == '\r' || szBuffer[k] == '\n')
			{
				szBuffer[k] = '\0';
				startIndex = k + 1;
				break;
			}
		}

		char * convertedPath = convertPath(szBuffer);
		int pathlength = strlen(convertedPath);
		
		songfilenames[iNumFile] = new char[pathlength + 1];
		sprintf(songfilenames[iNumFile], "%s", convertedPath);

		//Don't parse for allowed backgrounds if this is one of the 4 special music files
		if(iNumFile < 4)
		{
			iNumFile++;
			continue;
		}

		//Now loop through the number of backgrounds and setup what music is allowed for which background
		for(i = 0; i < MAXBACKGROUNDID; i++)
		{
			char * startString = szBuffer + startIndex;

			for(k = startIndex; k < stringLength; k++)
			{
				if(szBuffer[k] == ',' || szBuffer[k] == '\r' || szBuffer[k] == '\n')
				{
					szBuffer[k] = '\0';
					startIndex = k + 1;
					break;
				}
			}

			if(!strcmp(startString, "1"))
			{
				songsforbackground[i][numsongsforbackground[i]] = iNumFile;
				numsongsforbackground[i]++;
			}
		}

		iNumFile++;
	}

	fclose(in);

	next = NULL;
	prev = NULL;

	/*
	for(i = 0; i < MAXBACKGROUNDID; i++)
	{
		printf("---- %d ----\n", i);
		printf("Count: %d\n", numsongsforbackground[i]);
		for(k = 0; k < numsongsforbackground[i]; k++)
			printf("%d\n", songsforbackground[i][k]);
	}
	*/
};

MusicEntry::~MusicEntry()
{
	delete name;
	delete key;

	int k; 
	for(k = 0; k < MAXBACKGROUNDID; k++)
	{
		delete [] songsforbackground[k];
	}

	for(k = 0; k < nummusicfiles; k++)
	{
		delete [] songfilenames[k];
	}

	delete [] songfilenames;
};

char * MusicEntry::GetMusic(int musicID)
{
	if(musicID >= 0 && musicID < 4)
		return songfilenames[musicID];

	return songfilenames[4];
}

char * MusicEntry::GetRandomMusic(int backgroundID)
{
	if(backgroundID >= 0 && backgroundID < MAXBACKGROUNDID && numsongsforbackground[backgroundID] > 0)
		return songfilenames[songsforbackground[backgroundID][rand() % numsongsforbackground[backgroundID]]];

	return songfilenames[4];
}

