
#include <string.h>
#include <math.h>

#include "global.h"

//this system is pretty cool ;)
//------------------------------------------------------------------------------
// class cloud
//------------------------------------------------------------------------------
EC_Cloud::EC_Cloud(gfxSprite *nspr, float nx, float ny, float nvelx)
{
	spr = nspr;
	x = nx;
	y = ny;
	velx = nvelx;
	w = spr->getWidth();
}


void EC_Cloud::draw()
{
	spr->draw((int)x, (int)y);

	if(x + w > 639)
		spr->draw((int)x - 639,   (int)y);
	else if(x < 0)
		spr->draw((int)x + 640,   (int)y);
}


void EC_Cloud::update()
{
	x += velx;
	if( x > 639.0f)
		x -= 639.0f;
	else if( x < 0.0f)
		x += 640.0f;
}


//------------------------------------------------------------------------------
// class ghost
//------------------------------------------------------------------------------
EC_Ghost::EC_Ghost(gfxSprite *nspr, float nx, float ny, float nvelx, int ianimationspeed, int inumframes)
{
	spr = nspr;
	x = nx;
	y = ny;
	velx = nvelx;
	w = spr->getWidth() >> 2;
	h = spr->getHeight();

	animationoffset = (nvelx < 0.0f ? 2 : 0);
	animationspeed = ianimationspeed;
	animationtimer = 0;
	animationframe = 0;
	numframes = inumframes;
}


void EC_Ghost::draw()
{
	spr->draw((int)x, (int)y, (animationframe + animationoffset) * w, 0, h, w);

	if(x + w > 639)
		spr->draw((int)x - 639, (int)y, (animationframe + animationoffset) * w, 0, h, w);
	else if(x < 0)
		spr->draw((int)x + 640, (int)y, (animationframe + animationoffset) * w, 0, h, w);
}


void EC_Ghost::update()
{
	x += velx;

	if( x > 639.0f)
		x -= 639.0f;
	else if( x < 0.0f)
		x += 640.0f;

	if(++animationtimer >= animationspeed)
	{
		animationtimer = 0;

		if(++animationframe >= numframes)
		{
			animationframe = 0;
		}
	}
}

//------------------------------------------------------------------------------
// class corpse
//------------------------------------------------------------------------------
EC_Corpse::EC_Corpse(gfxSprite *nspr, float nx, float ny)
{
	spr = nspr;
	x = nx;
	y = ny;
	vely = 1.0f;
	timeleft = CORPSESTAY;

	if(x + PWOFFSET < 0.0f)
		tx = (int) ( (x + 640.0f + PWOFFSET) / TILESIZE );
	else
		tx = (int) ( (x + PWOFFSET) / TILESIZE );

	if(x + PWOFFSET + PW >= 640.0f)
		tx2 = (int) ( (x + PWOFFSET + PW - 640.0f) / TILESIZE );
	else
		tx2 = (int) ( (x + PWOFFSET + PW) / TILESIZE );
}


void EC_Corpse::draw()
{
	spr->draw((int)x, (int)y);

	if((x + 32) > 639)
		spr->draw((int)x - 639, (int)y);
	else if(x < 0)
		spr->draw((int)x + 640, (int)y);
}


void EC_Corpse::update()
{
	if(vely != 0.0f)
	{
		int nexty = (int)(y + 32.0f + vely);

		if(nexty >= 480)
		{
			dead = true;
			return;
		}

		if(nexty >= 0)
		{
			int ty = nexty / TILESIZE;
			
			if(map.map(tx, ty) == tile_solid_on_top || map.map(tx2, ty) == tile_solid_on_top)
			{	//on ground on tile solid_on_top
				if((y + 32.0f - vely) / TILESIZE < ty)
				{	//only if we were above the tile in the previous frame
					y		= (float) (ty * TILESIZE - 32);
					vely	= 0.0f;
					return;
				}
			}

			if((map.map(tx, ty) & 0x05) > 0 || (map.map(tx2, ty) & 0x05) > 0 ||
				map.block(tx, ty) || map.block(tx2, ty))
			{	//on ground
				y		= (float) (ty * TILESIZE - 32);
				vely	= 0.0f;
				return;
			}
		}
		
		//falling (in air)
		y		+= vely;
		vely	= CapVelocity(GRAVITATION + vely);
	}
	else
	{
		if(timeleft > 0)
			timeleft--;
		else
			dead = true;
	}
}



//------------------------------------------------------------------------------
// class EC_GravText
//------------------------------------------------------------------------------

EC_GravText::EC_GravText(gfxFont *nfont, int nx, int ny, const char *ntext, float nvely)
{
	font = nfont;
	x = (float)(nx - (font->getWidth(ntext) / 2));
	y = (float)ny;
	
	text = new char[strlen(ntext)+1];	//someone should test if we got the memory...
	strcpy(text, ntext);
	
	vely = nvely;
}


void EC_GravText::update()
{
	y += vely;
	vely = CapVelocity(GRAVITATION + vely);

	if(y > 480)
		dead = true;
}


void EC_GravText::draw()
{
	font->draw((int)x, (int)y, text);

	int w = font->getWidth(text);

	if(x < 0)
		font->draw(640 + (int)x, (int)y, text);
	else if(x + w > 639)
		font->draw(-(w-(int)(x + w)%640), (int)y, text);
	
}


EC_GravText::~EC_GravText()
{
	delete [] text;
	text = NULL;
}


//------------------------------------------------------------------------------
// class EC_FallingObject
//------------------------------------------------------------------------------

EC_FallingObject::EC_FallingObject(gfxSprite *nspr, int nx, int ny, float nvely)
{
	spr = nspr;
	x = (float)nx;
	y = (float)ny;
	vely = nvely;

	w = spr->getWidth();
}


void EC_FallingObject::update()
{
	y += vely;
	vely = CapVelocity(GRAVITATION + vely);

	if(y > 480)
		dead = true;
}


void EC_FallingObject::draw()
{
	spr->draw((int)x, (int)y);	

	if((x+w) > 639)
		spr->draw(-(w-(int)(x+w)%640),   (int)y);
	else if(x < 0)
		spr->draw((int)x + 640,   (int)y);
}


//------------------------------------------------------------------------------
// class EC_BrokenBlock
//------------------------------------------------------------------------------

EC_BrokenBlock::EC_BrokenBlock(gfxSprite *nspr, int nx, int ny, float nvelx, float nvely)
{
	spr = nspr;
	x = (float)nx;
	y = (float)ny;
	vely = nvely;
	velx = nvelx;

	w = spr->getWidth();
}


void EC_BrokenBlock::update()
{
	y += vely;
	x += velx;
	vely = CapVelocity(GRAVITATION + vely);
	
	if(y > 480)
		dead = true;
}


void EC_BrokenBlock::draw()
{
	spr->draw((int)x, (int)y);

	if(x + w > 639)
		spr->draw((int)x - 639, (int)y);
	else if(x < 0)
		spr->draw((int)x + 640, (int)y);
}

//------------------------------------------------------------------------------
// class EC_SingleAnimation
//------------------------------------------------------------------------------

EC_SingleAnimation::EC_SingleAnimation(gfxSprite *nspr, int nx, int ny, int iframes, int irate)
{
	spr = nspr;
	x = (float)nx;
	y = (float)ny;
	frames = iframes;
	frame = 0;
	counter = 0;
	rate = irate;

	iw = spr->getWidth()/iframes;
	ih = spr->getHeight();

}


void EC_SingleAnimation::update()
{
	if(++counter >= rate)
	{
		counter = 0;
		if(++frame >= frames)
		{
			dead = true;
		}
	}
}


void EC_SingleAnimation::draw()
{
	spr->draw((int)x, (int)y, frame * iw, 0, iw, ih);

	if(x + iw > 639)
		spr->draw((int)x - 639, (int)y, frame * iw, 0, iw, ih);
	else if(x < 0)
		spr->draw((int)x + 640, (int)y, frame * iw, 0, iw, ih);
}

//------------------------------------------------------------------------------
// class EC_LoopingAnimation
//------------------------------------------------------------------------------

EC_LoopingAnimation::EC_LoopingAnimation(gfxSprite *nspr, int nx, int ny, int iframes, int irate, int iloops) :
	EC_SingleAnimation(nspr, nx, ny, iframes, irate)
{
	countloops = 0;
	loops = iloops;
}

void EC_LoopingAnimation::update()
{
	if(++counter >= rate)
	{
		counter = 0;
		if(++frame >= frames)
		{
			if(loops > 0 && ++countloops >= loops)
				dead = true;

			frame = 0;
		}
	}
}


//------------------------------------------------------------------------------
// class award
//------------------------------------------------------------------------------
EC_Award::EC_Award(gfxSprite *nspr, int ix, int iy, int destx, int desty, int iframes, int irate) :
	EC_LoopingAnimation(nspr, ix, iy, iframes, irate, 0)
{
	dx = (float)destx;
	dy = (float)desty;

	velx = (dx - x) / 100.0f;
	vely = (dy - y) / 100.0f;
}


void EC_Award::update()
{
	EC_LoopingAnimation::update();

	x += velx;
	y += vely;

	if( fabsf(dx - x) < fabsf(velx) && fabsf(dy - y) < fabsf(vely))
	{
		dead = true;
	}
}



//------------------------------------------------------------------------------
// class EC_ExplodingAward
//------------------------------------------------------------------------------

EC_ExplodingAward::EC_ExplodingAward(gfxSprite *nspr, int nx, int ny, float nvelx, float nvely, int timetolive, int awardID)
{
	spr = nspr;
	x = (float)nx;
	y = (float)ny;
	vely = nvely;
	velx = nvelx;
	timer = 0;
	ttl = timetolive;
	id = awardID;

	w = spr->getWidth()/9;
	h = spr->getHeight();
}


void EC_ExplodingAward::update()
{
	y += vely;
	x += velx;
	
	if(++timer > ttl)
	{
		dead = true;
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, (int)x + (w >> 1) - 16, (int)y + (h >> 1) - 16, 3, 8));
	}

	if(y > 480 || y + h < 0 || x < 0 || x + w > 640)
		dead = true;
}


void EC_ExplodingAward::draw()
{
	spr->draw((int)x, (int)y, id * w, 0, w, h);

	if(x + w > 639)
		spr->draw((int)x - 639, (int)y, id * w, 0, w, h);
	else if(x < 0)
		spr->draw((int)x + 640, (int)y, id * w, 0, w, h);
}

//------------------------------------------------------------------------------
// class EC_SwirlingAward
//------------------------------------------------------------------------------

EC_SwirlingAward::EC_SwirlingAward(gfxSprite *nspr, int nx, int ny, float nangle, float nradius, float nvel, int timetolive, int awardID)
{
	spr = nspr;
	x = nx;
	y = ny;
	vel = nvel;
	angle = nangle;
	radius = nradius;
	timer = 0;
	ttl = timetolive;
	id = awardID;

	w = spr->getWidth()/10;
	h = spr->getHeight();
}


void EC_SwirlingAward::update()
{
	angle += vel;
	if(angle > TWO_PI)
		angle -= TWO_PI;

	radius += 3.0f;
	
	if(++timer > ttl)
	{
		int awardx = x + (int)(radius * cos(angle));
		int awardy = y + (int)(radius * sin(angle));
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, (int)awardx + w/2 - 16, (int)awardy + h/2 - 16, 3, 8));
		
		dead = true;
	}
}


void EC_SwirlingAward::draw()
{
	int awardx = x + (int)(radius * cos(angle));
	int awardy = y + (int)(radius * sin(angle));
		
	spr->draw(awardx, awardy, id * w, 0, w, h);

	if((awardx+w) > 639)
		spr->draw(awardx - 639, awardy, id * w, 0, w, h);
	else if(awardx < 0)
		spr->draw(awardx + 640, awardy, id * w, 0, w, h);
}


//------------------------------------------------------------------------------
// class EC_RocketAward
//------------------------------------------------------------------------------

EC_RocketAward::EC_RocketAward(gfxSprite *nspr, int nx, int ny, float nvelx, float nvely, int timetolive, int awardID, int numAwards)
{
	spr = nspr;
	x = (float)nx;
	y = (float)ny;
	velx = nvelx;
	vely = nvely;

	timer = 0;
	ttl = timetolive;
	id = awardID;

	w = spr->getWidth()/numAwards;
	h = spr->getHeight();
}


void EC_RocketAward::update()
{
	vely += 0.2f;

	y += vely;
	x += velx;

	if(++timer > ttl)
	{
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, (int)x + w/2 - 16, (int)y + h/2 - 16, 3, 8));
		dead = true;
	}
}


void EC_RocketAward::draw()
{
	spr->draw((int)x, (int)y, id * w, 0, w, h);

	if((x+w) > 639)
		spr->draw((int)x - 639, (int)y, id * w, 0, w, h);
	else if(x < 0)
		spr->draw((int)x + 640, (int)y, id * w, 0, w, h);
}

//------------------------------------------------------------------------------
// class EC_SoulsAward
//------------------------------------------------------------------------------

EC_SoulsAward::EC_SoulsAward(gfxSprite *nspr, gfxSprite *nspr2, int nx, int ny, int timetolive, float nSpeed, int nSouls, int * nSoulArray)
{
	spr = nspr;
	spawnspr = nspr2;

	x = nx;
	y = ny;
	
	ttl = timetolive;
	numSouls = nSouls;

	id = new int[numSouls];

	for(int k = 0; k < numSouls; k++)
		id[k] = nSoulArray[k];

	timer = 0;
	speed = nSpeed;
	count = 0;

	if(numSouls < 1)
		dead = true;

	animationCount = 0;
	frame = 0;
	endmode = false;

	w = nspr2->getWidth() / 7;
	h = nspr2->getHeight();

}

EC_SoulsAward::~EC_SoulsAward()
{
	delete [] id;
}

void EC_SoulsAward::update()
{
	if(!endmode && ++timer > speed)
	{
		timer = 0;

		float addangle = QUARTER_PI / 20.0f;
		float startangle = -HALF_PI;
	
		float angle = (float)(rand()%21 - 10) * addangle + startangle;
		float velx = speed * cos(angle);
		float vely = speed * sin(angle);

		eyecandyfront.add(new EC_RocketAward(&spr_awardsouls, x - 8, y - 8, velx, vely, ttl, id[count], 5));
		
		if(++count >= numSouls)
		{
			endmode = true;
			frame = 4;
		}
	}

	if(++animationCount > 6)
	{
		animationCount = 0;

		frame++;

		if(!endmode && frame >= 4)
			frame = 0;
		else if(endmode && frame >= 7)
			dead = true; 
	}
}


void EC_SoulsAward::draw()
{
	spawnspr->draw(x - 16, y - 16, frame * w, 0, w, h);
	
	if(x + w > 655)
		spawnspr->draw(x - 654, y - 16, frame * w, 0, w, h);
	else if(x < 16)
		spawnspr->draw(x + 624, y - 16, frame * w, 0, w, h);
}

//------------------------------------------------------------------------------
// class EC_DoorFront
//------------------------------------------------------------------------------

EC_Door::EC_Door(gfxSprite *nspr, gfxSprite *nmariospr, int nx, int ny, int irate)
{
	spr = nspr;
	mariospr = nmariospr;

	x = nx;
	y = ny;
	timer = 0;
	rate = irate;

	iw = spr->getWidth() / 10;
	ih = spr->getHeight();

	offsety = ih;
	state = 0;
	frame = 0;
}


void EC_Door::update()
{
	if(++timer > rate)
	{
		timer = 0;

		if(state == 0)
		{
			offsety -= 3;

			if(offsety <= 0)
			{
				state = 1;
				offsety = 0;
			}

			frame = 1;	
		}
		else if(state == 1)
		{
			if(++frame >= 9)
			{
				state = 2;
				frame = 9;
			}
		}
		else if(state == 2)
		{
			if(--frame <= 1)
			{
				state = 3;
				frame = 1;
			}
		}
		else if(state == 3)
		{
			offsety += 3;

			if(offsety >= 32)
			{
				dead = true;
				offsety = 32;
			}

			frame = 1;	
		}

	}
}

void EC_Door::draw()
{
	spr->draw(x, y + offsety, 0, 0, iw, ih - offsety);

	if(state == 1)
		mariospr->draw(x + 16 - HALFPW - PWOFFSET, y + 16 - HALFPH - PHOFFSET + offsety, 0, 0, iw, ih - offsety);

	spr->draw(x, y + offsety, frame * iw, 0, iw, ih - offsety);
}
//------------------------------------------------------------------------------
// class eyecandy_container
//------------------------------------------------------------------------------
CEyecandyContainer::CEyecandyContainer()
{
	for(int i = 0; i < MAXEYECANDY; i++)
		list[i] = NULL;

	list_end = 0;
}


CEyecandyContainer::~CEyecandyContainer()
{
	clean();
}


void CEyecandyContainer::clean()
{
	for(int i = 0; i < list_end; i++)
	{
		delete list[i];
		list[i] = NULL;
	}
	list_end = 0;
}


int CEyecandyContainer::add(CEyecandy *ec)
{
	if(list_end < MAXEYECANDY)
	{
		list[list_end] = ec;
		ec->dead = false;
		list_end++;

		return list_end - 1;
	}
	else
	{
		delete ec;	//otherwise memory leak!
		printf("eyecandy list full!\n");
	}

	return -1;
}


void CEyecandyContainer::remove(int i)
{	
	delete list[i];
	list_end--;

	if(i != list_end)
	{		//if we didn't remove the last element we move the last elemnt in the new free place in the eyecandy list
		list[i] = list[list_end];
	}
}

void CEyecandyContainer::cleandeadobjects()
{
	for(int i = 0; i < list_end; i++)
	{
		if(list[i]->dead)
		{
			delete list[i];
			list_end--;

			if(i != list_end)
			{
				list[i] = list[list_end];
			}

			i--;
		}
	}
}

