#ifndef _EYECANDY_H
#define _EYECANDY_H


//this system is pretty cool ;)
//it can be easily changed to handle every object in the game


//eyecandy base class
class CEyecandy
{
	public:
		virtual void draw(){printf("CEyecandy::draw() - NO!\n");};
		virtual void update(){printf("CEyecandy::update() - NO!\n");};
		virtual ~CEyecandy(){;};
	
	protected:
		bool dead;
	
	friend class CEyecandyContainer;
};





//actual eyecandy implementations
class EC_Corpse : public CEyecandy
{
	public:
		EC_Corpse(gfxSprite *nspr, float nx, float ny);
		~EC_Corpse(){;};
		void draw();
		void update();

	private:
		gfxSprite *spr;
		float x, y;
		int tx, tx2;
		float vely;
		int timeleft;
};


class EC_Cloud : public CEyecandy
{
	public:
		EC_Cloud(gfxSprite *nspr, float nx, float ny, float nvelx);
		~EC_Cloud(){;};
		void draw();
		void update();

	private:
		gfxSprite *spr;
		float x, y;
		float velx;
		int w;
};


class EC_Ghost : public CEyecandy
{
	public:
		EC_Ghost(gfxSprite *nspr, float nx, float ny, float nvelx, int ianimationspeed, int inumframes);
		~EC_Ghost(){;};
		void draw();
		void update();

	private:
		gfxSprite *spr;
		float x, y;
		float velx;
		int w, h;

		int animationoffset;
		int animationspeed;
		int animationtimer;
		int animationframe;
		int numframes;
};



class EC_GravText : public CEyecandy
{
	public:
		EC_GravText(gfxFont *font, int nx, int ny, const char *text, float nvely);
		~EC_GravText();
		void draw();
		void update();

	private:
		gfxFont *font;
		float x, y;
		float vely;
		char *text;
};


class EC_FallingObject : public CEyecandy
{
	public:
		EC_FallingObject(gfxSprite *nspr, int nx, int ny, float nvely);
		void draw();
		void update();

	private:
		gfxSprite *spr;
		float x, y;
		int w;
		float vely;
};

class EC_BrokenBlock : public CEyecandy
{
	public:
		EC_BrokenBlock(gfxSprite *nspr, int nx, int ny, float nvelx, float nvely);
		void draw();
		void update();

	private:
		gfxSprite *spr;
		float x, y;
		int w;
		float velx, vely;
};

class EC_SingleAnimation : public CEyecandy
{
	public:
		EC_SingleAnimation(gfxSprite *nspr, int nx, int ny, int iframes, int irate);
		void draw();
		void update();

	protected:
		gfxSprite *spr;
		float x, y;
		int frames, frame, counter;
		int iw, ih;
		int rate;
};

class EC_LoopingAnimation : public EC_SingleAnimation
{
	public:
		EC_LoopingAnimation(gfxSprite *nspr, int nx, int ny, int iframes, int irate, int iloops);
		void update();
	
	private:
		int countloops;
		int loops;
};

//actual eyecandy implementations
class EC_Award : public EC_LoopingAnimation
{
	public:
		EC_Award(gfxSprite *nspr, int ix, int iy, int destx, int desty, int iframes, int irate);
		~EC_Award(){;};
		void update();

	private:
		float dx, dy;
		float vely, velx;
};

class EC_ExplodingAward : public CEyecandy
{
	public:
		EC_ExplodingAward(gfxSprite *nspr, int nx, int ny, float nvelx, float nvely, int timetolive, int awardID);
		void draw();
		void update();

	private:
		gfxSprite *spr;
		float x, y;
		int w, h;
		int timer, ttl, id;
		float velx, vely;
};

class EC_SwirlingAward : public CEyecandy
{
	public:
		EC_SwirlingAward(gfxSprite *nspr, int nx, int ny, float nangle, float nradius, float nvel, int timetolive, int awardID);
		void draw();
		void update();

	private:
		gfxSprite *spr;
		int x, y;
		int w, h;
		int timer, ttl, id;
		float vel, angle, radius;
};

class EC_RocketAward : public CEyecandy
{
	public:
		EC_RocketAward(gfxSprite *nspr, int nx, int ny, float nvelx, float nvely, int timetolive, int awardID, int numAwards);
		void draw();
		void update();

	private:
		gfxSprite *spr;
		float x, y;
		int w,h;
		float velx, vely;
		int timer, ttl, id;
};

class EC_SoulsAward : public CEyecandy
{
	public:
		EC_SoulsAward(gfxSprite *nspr, gfxSprite *nspr2, int nx, int ny, int timetolive, float nSpeed, int nSouls, int * nSoulArray);
		~EC_SoulsAward();
		void draw();
		void update();

	private:
		gfxSprite *spr;
		gfxSprite *spawnspr;

		int x, y;
		int numSouls;
		int * id;
		int ttl, timer, count;
		float speed;

		int animationCount, frame;
		bool endmode;
		int w, h;
};

class EC_Door : public CEyecandy
{
	public:
		EC_Door(gfxSprite *nspr, gfxSprite *mario, int nx, int ny, int irate);
		virtual void draw();
		virtual void update();
	
	protected:
		gfxSprite *spr;
		gfxSprite *mariospr;

		int x, y;
		int frame, timer;
		int iw, ih;
		int rate;
		int state;
		int offsety;
};

//eyecandy container
class CEyecandyContainer
{
	public:
		CEyecandyContainer();
		~CEyecandyContainer();

		int add(CEyecandy *ec);

		void update()
		{
			for(int i = 0; i<list_end; i++)
				list[i]->update();
		};
		void draw()
		{
			for(int i = 0; i<list_end; i++)
				list[i]->draw();
		};

		void clean();

		void cleandeadobjects();

	public:
		CEyecandy *list[MAXEYECANDY];
		int		 list_end;

		void remove(int i);
};

#endif

