#include "global.h"
extern	gfxFont			font[3];

#include <string.h>

extern int lastgamewinner;
extern int lookupteamid(int id);
extern int countaliveteams(int * lastteam);
extern char *convertPath(char *source);

void removeplayersbutteam(int teamid)
{
	for(int i = 0; i < list_players_cnt; i++)
	{
		if(list_players[i]->teamID != teamid)
		{
			list_players[i]->state = player_dead;
		}
	}
}

void removeplayersbuthighestscoring()
{
	int maxscore = -1;
	int i;

	//Figure out what the maximum score is
	for(i = 0; i < score_cnt; i++)
	{
		if(score[i].score > maxscore)
			maxscore = score[i].score;
	}

	//Remove all players that don't have that max score
	for(i = 0; i < list_players_cnt; i++)
	{
		if(list_players[i]->score->score < maxscore)
			list_players[i]->state = player_dead;
	}
}

void setupscoreboard()
{
	showscoreboard = true;
	lastgamewinner = -1;

	int i, j;
	
	bool draw[4] = {false, false, false, false};
	
	int max;
	for(i = 0; i < score_cnt; i++)
	{
		max = -1;
		for(j = 0; j < score_cnt; j++)
		{
			if(!draw[j])
			{	
				//If this player's score is bigger
				if(max == -1 || score[j].score > score[max].score)
					max = j;
				else if(score[j].score == score[max].score && score[j].order > score[max].order) //or it is tied but they died later in the game
					max = j;
			}
		}

		score[max].place = i;
		draw[max] = true;	//this is the next biggest score - it doesn't belong to the remaining scores from now on
	}


	for(i = 0; i < score_cnt; i++)
	{
		score[i].destx = 300 - 34 * game_values.teamcounts[i];
		score[i].desty = score[i].place * 45 + 150;
	}

	if(game_values.music)
	{
		ifsoundonstop(invinciblemusic);
		ifsoundonstop(timewarning);
		ifsoundonstop(slowdownmusic);
		
		backgroundmusic[1].play(true, false);
	}
}

void removeteam(int teamid)
{
	//kill all players on the dead team
	for(int k = 0; k < list_players_cnt; k++)
		if(list_players[k]->teamID == teamid)
			list_players[k]->state = player_dead;

	score[teamid].order = game_values.teamdeadcounter;
	game_values.teamdeadcounter++;
}

void CGameMode::displayplayertext()
{
	if(winningteam > -1)
	{
		for(int k = 0; k < list_players_cnt; k++)
		{
			if(list_players[k]->teamID == winningteam)
				list_players[k]->spawnText("Winner!");
		}
	}
}

void CGameMode::playwarningsound()
{
	playedwarningsound = true;
	iHurryUpMusicChannel = ifsoundonplay(timewarning);
	ifsoundonstop(invinciblemusic);

	if(game_values.music && game_values.sound)
		backgroundmusic[0].stop();
}

//fraglimit:
void CGM_DM_Fraglimit::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	playedwarningsound = false;
}


void CGM_DM_Fraglimit::think()
{
	if(gameover)
		displayplayertext();
}


bool CGM_DM_Fraglimit::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	if(!gameover)
	{
		//Penalize killing your team mates
		if(inflictor.teamID == other.teamID)
		{
			inflictor.score->score -= 2;
			if(inflictor.score->score < 0)
				inflictor.score->score = 0;
		}

		if(inflictor.score->score >= goal)
		{
			winningteam = inflictor.teamID;
			gameover = true;

			removeplayersbutteam(winningteam);
			setupscoreboard();

			return true;	//we removed a player
		}
		else if(inflictor.score->score >= goal - 2 && !playedwarningsound)
		{
			playwarningsound();
		}
	}

	return false;		//we have not removed a player
}

bool CGM_DM_Fraglimit::playerkilledself(CPlayer &player)
{
	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	if(player.score->score > 0 && !gameover)
		player.score->score--;

	return false;		//we have not removed a player
}

void CGM_DM_Fraglimit::player1up(CPlayer &player)
{
	if(!gameover)
	{
		player.score->score++;

		if(player.score->score >= goal)
		{
			winningteam = player.teamID;
			gameover = true;

			removeplayersbutteam(winningteam);
			setupscoreboard();
		}
		else if(player.score->score >= goal - 2 && !playedwarningsound)
		{
			playwarningsound();
		}
	}
}


char * CGM_DM_Fraglimit::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Frag Limit %d", goal);
	return buffer64;
}


void CGM_DM_Fraglimit::toggleoptions()
{
	goal += 5;
	if(goal > 50)
		goal = 5;
}





//timelimit
void CGM_DM_Timelimit::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	timeleft = goal;
	framesleft_persecond = 1000 / WAITTIME;
	playedwarningsound = false;
}


void CGM_DM_Timelimit::think()
{
	if(gameover)
		displayplayertext();

	if(timeleft > 0)
	{
		if(framesleft_persecond > 0)
		{
			framesleft_persecond--;
		}			
		else
		{
			framesleft_persecond = 1000 / WAITTIME;

			if(--timeleft == 20 && !playedwarningsound)
			{
				playwarningsound();
			}
		}
	}
	else if(timeleft == 0)
	{		//the game ends
		setupscoreboard();
		timeleft--;

		removeplayersbuthighestscoring();
		gameover = true;

		countaliveteams(&winningteam);
	}
}


bool CGM_DM_Timelimit::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	if(!gameover)
	{
		//Penalize killing your team mates
		if(inflictor.teamID == other.teamID)
		{
			inflictor.score->score -= 2;
			if(inflictor.score->score < 0)
				inflictor.score->score = 0;
		}
	}
	else
	{
		//game over -> don't count kills
		inflictor.score->score--;
	}

	return false;		//we have not removed a player
}

bool CGM_DM_Timelimit::playerkilledself(CPlayer &player)
{
	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	if(player.score->score > 0 && timeleft > -1)
		player.score->score--;

	return false;		//we have not removed a player
}

void CGM_DM_Timelimit::player1up(CPlayer &player)
{	
	if(timeleft > -1)
		player.score->score++;
}


void CGM_DM_Timelimit::draw()
{
	char buffer[64];

	if(timeleft > 0)
	{
		sprintf(buffer, "%d", timeleft);
		font[2].draw(5, 7, buffer);
	}
}

	
char * CGM_DM_Timelimit::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Time Limit %d", goal);
	return buffer64;
}


void CGM_DM_Timelimit::toggleoptions()
{
	goal += 30;
	if(goal > 300)
		goal = 30;
}




//mariowar (x lives - counting down)
void CGM_MarioWar::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	playedwarningsound = false;

	for(int i=0;i<list_players_cnt;i++)
	{
		list_players[i]->score->score = goal;
	}
}


void CGM_MarioWar::think()
{
	if(gameover)
		displayplayertext();
}


bool CGM_MarioWar::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	(inflictor.score->score)--;	//no kills are counted

	if(!gameover)
	{
		(other.score->score)--;

		int countscore = 0;
		for(int k = 0; k < score_cnt; k++)
		{
			if(inflictor.score == &score[k])
				continue;

			countscore += score[k].score;
		}

		if(countscore <= 2 && !playedwarningsound)
		{
			playwarningsound();
		}

		if(other.score->score <= 0)
		{
			removeteam(other.teamID);
			return true;	//we removed a player
		}
	}

	return false;		//we have not removed a player
}

bool CGM_MarioWar::playerkilledself(CPlayer &player)
{
	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	if(!gameover)
	{
		(player.score->score)--;

		int countscore = 0;
		bool playwarning = false;
		for(int j = 0; j < score_cnt; j++)
		{
			for(int k = 0; k < score_cnt; k++)
			{
				if(j == k)
					continue;

				countscore += score[k].score;
			}

			if(countscore <= 2)
			{
				playwarning = true;
				break;
			}

			countscore = 0;
		}

		if(playwarning && !playedwarningsound)
		{
			playwarningsound();
		}

		if(player.score->score <= 0)
		{
			removeteam(player.teamID);
			return true;	//we removed a player
		}
	}

	return false;		//we have not removed a player
}

void CGM_MarioWar::player1up(CPlayer &player)
{
	if(!gameover)
		(player.score->score)++;
}


char * CGM_MarioWar::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Classic %d", goal);
	return buffer64;
}


void CGM_MarioWar::toggleoptions()
{
	goal += 5;
	if(goal > 50)
		goal = 5;
}




//capture the chicken
//one player is the chicken
//if he is stomped the attacker becomes the chicken.
//get points for being the chicken
void CGM_CaptureTheChicken::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	gamemode = game_mode_chicken;
	playedwarningsound = false;
}

void CGM_CaptureTheChicken::think()
{
	if(gameover)
	{
		displayplayertext();
	}
	else
	{
		if(chicken)
		{
			if( chicken->velx > VELMOVING_CHICKEN )
				chicken->velx = VELMOVING_CHICKEN;
			else if(chicken->velx < -VELMOVING_CHICKEN)
				chicken->velx = -VELMOVING_CHICKEN;

			static int counter = 0;

			if(chicken->isready())
			{
				if(++counter >= 10)
				{
					counter = 0;
					(chicken->score->score)++;
				}
			}

			if(chicken->score->score >= goal)
			{
				winningteam = chicken->teamID;
				gameover = true;

				setupscoreboard();
				removeplayersbutteam(winningteam);
			}
			else if(chicken->score->score >= goal * 0.8 && !playedwarningsound)
			{
				playwarningsound();
			}
		}	
	}
}

void CGM_CaptureTheChicken::draw()
{
	if(!gameover && chicken)
	{
		if(chicken->iswarping())
		{
			int ownedWarpOffsetX = chicken->warpOffsetX + (chicken->state == player_entering_warp_left || chicken->state == player_exiting_warp_right ? 16 : 0);
			int ownedWarpOffsetY = chicken->warpOffsetY + (chicken->state == player_entering_warp_up || chicken->state == player_exiting_warp_down ? 16 : 0);
			int ownedWarpWidthOffsetLeading = chicken->warpWidthOffsetLeading + (chicken->state == player_entering_warp_left || chicken->state == player_exiting_warp_right ? 16 : 0);
			int ownedWarpHeightOffsetLeading = chicken->warpHeightOffsetLeading + (chicken->state == player_entering_warp_up || chicken->state == player_exiting_warp_down ? 16 : 0);
			int ownedWarpWidthOffsetTrailing = chicken->warpWidthOffsetTrailing + (chicken->state == player_entering_warp_right || chicken->state == player_exiting_warp_left ? -16 : 0);
			int ownedWarpHeightOffsetTrailing = chicken->warpHeightOffsetTrailing + (chicken->state == player_entering_warp_down || chicken->state == player_exiting_warp_up ? -16 : 0);

			spr_chicken.draw(chicken->ix - PWOFFSET - 16 + ownedWarpOffsetX, chicken->iy - PHOFFSET - 16 + ownedWarpOffsetY, ownedWarpWidthOffsetLeading, ownedWarpHeightOffsetLeading, 64 + ownedWarpWidthOffsetTrailing, 64 + ownedWarpHeightOffsetTrailing);

			if(chicken->ix - PWOFFSET - 16 < 0)
				spr_chicken.draw(chicken->ix - PWOFFSET + 624 + ownedWarpOffsetX, chicken->iy - PHOFFSET - 16 + ownedWarpOffsetY, ownedWarpWidthOffsetLeading, ownedWarpHeightOffsetLeading, 64 + ownedWarpWidthOffsetTrailing, 64 + ownedWarpHeightOffsetTrailing);
			else if(chicken->ix - PWOFFSET + 48 >= 640)
				spr_chicken.draw(chicken->ix - PWOFFSET - 656 + ownedWarpOffsetX, chicken->iy - PHOFFSET - 16 + ownedWarpOffsetY, ownedWarpWidthOffsetLeading, ownedWarpHeightOffsetLeading, 64 + ownedWarpWidthOffsetTrailing, 64 + ownedWarpHeightOffsetTrailing);
		}
		else if(chicken->isready())
		{
			spr_chicken.draw(chicken->ix + HALFPW - 32, chicken->iy + HALFPH - 32);

			if(chicken->ix + HALFPW + 32 > 639)
				spr_chicken.draw(chicken->ix + HALFPW - 672, chicken->iy + HALFPH - 32);
			else if(chicken->ix + HALFPW - 32 < 0)
				spr_chicken.draw(chicken->ix + HALFPW + 608, chicken->iy + HALFPH - 32);
		}
	}
}


bool CGM_CaptureTheChicken::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	if(!chicken)
	{
		chicken = &inflictor;
		eyecandyfront.add(new EC_GravText(&font[2], inflictor.ix + (HALFPW), inflictor.iy + PH, "Chicken!", -VELJUMP*1.5));
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, inflictor.ix + (HALFPW) - 16, inflictor.iy + (HALFPH) - 16, 3, 8));
		ifsoundonplay(transform);
	}
	else
	{
		if(&other == chicken)
		{
			other.diedas = 1; //flag to use chocobo corpse sprite
			chicken = &inflictor;
			eyecandyfront.add(new EC_GravText(&font[2], inflictor.ix + (HALFPW), inflictor.iy + PH, "Chicken!", -VELJUMP*1.5));
			eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, inflictor.ix + (HALFPW) - 16, inflictor.iy + (HALFPH) - 16, 3, 8));
			ifsoundonplay(transform);
		}
		else if(&inflictor == chicken)
		{
			if(!gameover)
			{
				inflictor.score->score += 15;

				if(inflictor.score->score >= goal)
				{
					inflictor.score->score = goal;
					winningteam = inflictor.teamID;
					gameover = true;

					setupscoreboard();
					removeplayersbutteam(winningteam);
					return true;
				}
				else if(inflictor.score->score >= goal * 0.8 && !playedwarningsound)
				{
					playwarningsound();
				}
			}
		}
	}
	
	//Don't count this kill for chicken goal
	inflictor.score->score--;

	return false;		//we have not removed a player
}

bool CGM_CaptureTheChicken::playerkilledself(CPlayer &player)
{
	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	if(chicken == &player)
	{
		player.diedas = 1; //flag to use chocobo corpse sprite

		if(!gameover)
			chicken = NULL;
	}
	
	return false;		//we have not removed a player
}

void CGM_CaptureTheChicken::player1up(CPlayer &player)
{
	if(!gameover)
	{
		player.score->score += 25;

		if(player.score->score >= goal)
		{
			player.score->score = goal;
			winningteam = player.teamID;
			gameover = true;

			setupscoreboard();
			removeplayersbutteam(winningteam);
		}
		else if(player.score->score >= goal * 0.8 && !playedwarningsound)
		{
			playwarningsound();
		}
	}
}

CGM_CaptureTheChicken::CGM_CaptureTheChicken()
{
	goal = 400;
}

char * CGM_CaptureTheChicken::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Chicken %d", goal);
	return buffer64;
}

void CGM_CaptureTheChicken::toggleoptions()
{
	goal += 100;
	if(goal > 1000)
		goal = 100;
}





//tag mode (leper mode suggestion from ziotok)
//one player is "it"
//if he killes another player, they become "it"
//the player that is "it" loses life until dead.
//the "it" player is chosen at random.  Someone is
//always "it".
void CGM_Tag::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	gamemode = game_mode_tag;
	playedwarningsound = false;

	for(int i = 0; i < score_cnt; i++)
	{
		score[i].score = goal;
	}
}

void CGM_Tag::think()
{
	if(gameover)
	{
		displayplayertext();
	}
	else
	{
		if(!tagged)
		{
			int i, j;
			int count = 1;
			int tiedplayers[4];
			tiedplayers[0] = 0;

			for(j = 0; j < list_players_cnt; j++)
			{
				if(!list_players[j]->isdead())
				{
					count = 1;
					tiedplayers[0] = j;
					break;
				}
			}

			for(i = j + 1; i < list_players_cnt; i++)
			{
				if(!list_players[i]->isdead())
				{
					if(list_players[i]->score->score > list_players[tiedplayers[0]]->score->score)
					{
						count = 1;
						tiedplayers[0] = i;
					}
					else if(list_players[i]->score->score == list_players[tiedplayers[0]]->score->score)
					{
						tiedplayers[count] = i;
						count++;
					}
				}
			}

			tagged = list_players[tiedplayers[rand()%count]];
		}

		static int counter = 0;

		if(tagged->isready())
		{
			if(++counter >= 10)
			{
				counter = 0;
				(tagged->score->score)--;
			}
		}
		
		int countscore = 0;
		bool playwarning = false;
		for(int j = 0; j < score_cnt; j++)
		{
			for(int k = 0; k < score_cnt; k++)
			{
				if(j == k)
					continue;

				countscore += score[k].score;
			}

			if(countscore <= goal * 0.2)
			{
				playwarning = true;
				break;
			}

			countscore = 0;
		}

		if(playwarning && !playedwarningsound)
		{
			playwarningsound();
		}

		if(tagged->score->score <= 0)
		{
			removeteam(tagged->teamID);
			tagged = NULL;
		}
	}
}

bool CGM_Tag::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	//Don't count this kill for tag goal
	inflictor.score->score--;

	if(&inflictor == tagged)
	{
		tagged = &other;
		inflictor.spawninvincible = true;
		inflictor.spawninvincibletimer = 60;
		eyecandyfront.add(new EC_GravText(&font[2], other.ix + (HALFPW), other.iy + PH, "Tagged!", -VELJUMP*1.5));
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, other.ix + (HALFPW) - 16, other.iy + (HALFPH) - 16, 3, 8));
		ifsoundonplay(transform);
	}

	if(!gameover)
	{
		other.score->score -= 15;

		int countscore = 0;
		for(int k = 0; k < score_cnt; k++)
		{
			if(inflictor.score == &score[k])
				continue;

			countscore += score[k].score;
		}

		if(countscore <= goal * 0.2 && !playedwarningsound)
		{
			playwarningsound();
		}

		if(other.score->score <= 0)
		{
			other.score->score = 0;
			
			removeteam(other.teamID);
			return true;	//we removed a player
		}
	}

	return false;		//we have not removed a player
}

bool CGM_Tag::playerkilledself(CPlayer &player)
{
	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	if(!gameover)
	{
		player.score->score -= 15;

		int countscore = 0;
		bool playwarning = false;
		for(int j = 0; j < score_cnt; j++)
		{
			for(int k = 0; k < score_cnt; k++)
			{
				if(j == k)
					continue;

				countscore += score[k].score;
			}

			if(countscore <= goal * 0.2)
			{
				playwarning = true;
				break;
			}

			countscore = 0;
		}

		if(playwarning && !playedwarningsound)
		{
			playwarningsound();
		}

		if(player.score->score <= 0)
		{
			player.score->score = 0;
			
			removeteam(player.teamID);
			return true;	//we removed a player
		}
	}

	return false;		//we have not removed a player
}

void CGM_Tag::player1up(CPlayer &player)
{
	if(!gameover)
		player.score->score += 25;
}

CGM_Tag::CGM_Tag()
{
	goal = 400;
}

char * CGM_Tag::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Tag %d", goal);
	return buffer64;
}

void CGM_Tag::toggleoptions()
{
	goal += 100;
	if(goal > 1000)
		goal = 100;
}



//Coin mode:
//Collect randomly appearing coins on map
//First one to set amount wins
void CGM_Coins::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	gamemode = game_mode_coinlimit;
	playedwarningsound = false;

	interactionobjectsplayer.add(new PU_Coin(&spr_coin, 4, 8));
}


void CGM_Coins::think()
{
	if(gameover)
	{
		displayplayertext();
	}
	else
	{
		for(int i = 0; i < list_players_cnt; i++)
		{
			if(list_players[i]->score->score >= goal)
			{
				winningteam = list_players[i]->teamID;
				gameover = true;

				removeplayersbutteam(winningteam);
				setupscoreboard();
			}
			else if(list_players[i]->score->score >= goal - 2 && !playedwarningsound)
			{
				playwarningsound();
			}
		}
	}
}


bool CGM_Coins::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	//Kills don't count in coin mode
	inflictor.score->score--;
	return false;
}

bool CGM_Coins::playerkilledself(CPlayer &player)
{
	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	return false;		//we have not removed a player
}

void CGM_Coins::player1up(CPlayer &player)
{
	if(!gameover)
	{
		player.score->score++;

		if(player.score->score >= goal)
		{
			winningteam = player.teamID;
			gameover = true;

			removeplayersbutteam(winningteam);
			setupscoreboard();
		}
		else if(player.score->score >= goal - 2 && !playedwarningsound)
		{
			playwarningsound();
		}
	}
}


char * CGM_Coins::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Coin Limit %d", goal);
	return buffer64;
}


void CGM_Coins::toggleoptions()
{
	goal += 5;
	if(goal > 50)
		goal = 5;
}

//Egg mode:
//Grab the egg and return it to the basket
//Score 1 point for each
void CGM_Eggs::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	gamemode = game_mode_egglimit;
	playedwarningsound = false;

	interactionobjectsplayer.add(new OMO_Yoshi(&spr_yoshi));
	interactionobjectsplayer.add(new OMO_Egg(&spr_egg));
}


void CGM_Eggs::think()
{
	if(gameover)
	{
		displayplayertext();
	}
	else
	{
		for(int i = 0; i < list_players_cnt; i++)
		{
			if(list_players[i]->score->score >= goal)
			{
				winningteam = list_players[i]->teamID;
				gameover = true;

				removeplayersbutteam(winningteam);
				setupscoreboard();
			}
			else if(list_players[i]->score->score >= goal - 2 && !playedwarningsound)
			{
				playwarningsound();
			}
		}
	}
}


bool CGM_Eggs::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	if(other.ownedEgg)
	{
		other.ownedEgg->owner = NULL;
		other.ownedEgg = NULL;
	}

	//Kills don't count in egg mode
	inflictor.score->score--;
	return false;
}

bool CGM_Eggs::playerkilledself(CPlayer &player)
{
	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	if(player.ownedEgg)
	{
		player.ownedEgg->placeEgg();
		player.ownedEgg->owner = NULL;
		player.ownedEgg = NULL;
	}

	return false;		//we have not removed a player
}

void CGM_Eggs::player1up(CPlayer &player)
{
	if(!gameover)
		player.score->score++;

	if(player.score->score >= goal)
	{
		winningteam = player.teamID;
		gameover = true;

		removeplayersbutteam(winningteam);
		setupscoreboard();
	}
	else if(player.score->score >= goal - 2 && !playedwarningsound)
	{
		playwarningsound();
	}
}


char * CGM_Eggs::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Yoshi's Eggs %d", goal);
	return buffer64;
}


void CGM_Eggs::toggleoptions()
{
	goal += 5;
	if(goal > 50)
		goal = 5;
}


//Fireball:
//Frag limit death match, but firepower cards appear randomly
CGM_Fireball::CGM_Fireball() :
	CGM_DM_Fraglimit()
{
	gamemode = game_mode_fireball;
}

void CGM_Fireball::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	playedwarningsound = false;
	timer = 0;
}


void CGM_Fireball::think()
{
	if(gameover)
	{
		displayplayertext();
	}
	else
	{
		if(++timer >= 200 && interactionobjectsplayer.countTypes(object_fireflowercard) < list_players_cnt - 1)
		{
			interactionobjectsplayer.add(new PU_FireFlowerCard(&spr_fireflowercard, 6, 8));
			timer = 0;
		}
	}
}

char * CGM_Fireball::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Fire Frenzy %d", goal);
	return buffer64;
}


//Hammer:
//Frag limit death match, but hammer cards appear randomly
CGM_Hammer::CGM_Hammer() :
	CGM_DM_Fraglimit()
{
	gamemode = game_mode_hammer;
}

void CGM_Hammer::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	playedwarningsound = false;
	timer = 0;
}


void CGM_Hammer::think()
{
	if(gameover)
	{
		displayplayertext();
	}
	else
	{
		if(++timer >= 200 && interactionobjectsplayer.countTypes(object_hammercard) < list_players_cnt - 1)
		{
			interactionobjectsplayer.add(new PU_HammerCard(&spr_hammercard, 6, 8));
			timer = 0;
		}
	}
}

char * CGM_Hammer::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Hammers %d", goal);
	return buffer64;
}


//Thwomp Mode! - just like mario war classic, but you have
// to dodge thwomps from the sky.  Idea from ziotok.
CGM_Thwomp::CGM_Thwomp() :
	CGM_MarioWar()
{
	goal = 20;
	gamemode = game_mode_thwomp;
}

void CGM_Thwomp::init()
{
	CGM_MarioWar::init();

	rate = 60;
	timer = rand() % 21 - 10 + rate;
	ratetimer = 0;
}

void CGM_Thwomp::think()
{
	if(gameover) 
	{
		displayplayertext();
	}
	else
	{
		if(--timer <= 0)
		{
			timer = rand() % 21 - 10 + rate;

			if(++ratetimer == 10)
			{
				ratetimer = 0;
				
				if(--rate < 20)
					rate = 20;
			}

			interactionobjectsfront.add(new OMO_Thwomp(&spr_thwomp, rand() % 591, 2.0f + (float)(rand()%20)/10.0f));
		}
	}
}

char * CGM_Thwomp::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Survival %d", goal);
	return buffer64;
}


//Domination (capture the area blocks)
//Touch all the dotted blocks to turn them your color
//The more blocks you have, the faster you rack up points
void CGM_Domination::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	gamemode = game_mode_domination;
	playedwarningsound = false;

	for(int k = 0; k < list_players_cnt + 1; k++)
		interactionobjectcollisionitems.add(new OMO_Area(&spr_areas, list_players_cnt + 1));
	
}

void CGM_Domination::think()
{
	if(gameover)
	{
		displayplayertext();
	}
	else
	{
		for(int i = 0; i < list_players_cnt; i++)
		{
			if(list_players[i]->score->score >= goal)
			{
				winningteam = list_players[i]->teamID;
				gameover = true;

				removeplayersbutteam(winningteam);
				setupscoreboard();
			}
			else if(list_players[i]->score->score >= goal * 0.8 && !playedwarningsound)
			{
				playwarningsound();
			}
		}
	}
}

bool CGM_Domination::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	//Don't count this kill for domination goal
	inflictor.score->score--;

	//Player loses control of his areas
	interactionobjectcollisionitems.removePlayerAreas(other.globalID);

	return false;		//we have not removed a player
}

bool CGM_Domination::playerkilledself(CPlayer &player)
{
	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	//Player loses control of his areas
	interactionobjectcollisionitems.removePlayerAreas(player.globalID);

	return false;		//we have not removed a player
}

void CGM_Domination::player1up(CPlayer &player)
{
	if(!gameover)
	{
		player.score->score += 25;

		if(player.score->score >= goal)
		{
			player.score->score = goal;
			winningteam = player.teamID;
			gameover = true;

			setupscoreboard();
			removeplayersbutteam(winningteam);
		}
		else if(player.score->score >= goal * 0.8 && !playedwarningsound)
		{
			playwarningsound();
		}
	}
}

CGM_Domination::CGM_Domination()
{
	goal = 400;
}

char * CGM_Domination::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Domination %d", goal);
	return buffer64;
}

void CGM_Domination::toggleoptions()
{
	goal += 100;
	if(goal > 1000)
		goal = 100;
}


//Bob-omb:
//Frag limit death match, but players can turn into a bob-omb for extra killing power!
CGM_Bobomb::CGM_Bobomb() :
	CGM_DM_Fraglimit()
{
	gamemode = game_mode_bobomb;
}

void CGM_Bobomb::think()
{
	if(gameover)
	{
		displayplayertext();
	}
	else
	{
		bool foundbobomb = false;
		for(int k = 0; k < list_players_cnt; k++)
		{
			if(list_players[k]->bobomb)
			{
				foundbobomb = true;
				break;
			}
		}

		if(!foundbobomb && !interactionobjectsplayer.isBombReady())
		{
			interactionobjectsplayer.add(new PU_BobombCard(&spr_bobombcard, 6, 8));
		}
	}
}

char * CGM_Bobomb::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Bob-Omb %d", goal);
	return buffer64;
}


//Owned:
//Players rack up points like domination for each player they have "owned"
CGM_Owned::CGM_Owned()
{
	gamemode = game_mode_owned;
	goal = 400;
}

void CGM_Owned::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	playedwarningsound = false;
}

void CGM_Owned::think()
{
	if(gameover)
	{
		displayplayertext();
	}
	else
	{
		static int counter = 0;

		if(++counter >= 10)
		{
			counter = 0;

			for(int i = 0; i < list_players_cnt; i++)
			{
				for(int k = 0; k < list_players_cnt; k++)
				{
					if(i == k)
						continue;

					if(list_players[k]->ownerPlayerID == list_players[i]->globalID)
						list_players[i]->score->score++;
				}
				
				if(list_players[i]->score->score >= goal)
				{
					list_players[i]->score->score = goal;

					winningteam = list_players[i]->teamID;
					gameover = true;

					removeplayersbutteam(winningteam);
					setupscoreboard();
				}
				else if(list_players[i]->score->score >= goal * 0.8 && !playedwarningsound)
				{
					playwarningsound();
				}
			}
		}
	}
}

bool CGM_Owned::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	//Don't count this kill
	inflictor.score->score--;

	if(!gameover)
	{
		for(int i = 0; i < list_players_cnt; i++)
		{
			if(list_players[i]->ownerPlayerID == other.globalID)
			{
				list_players[i]->ownerPlayerID = -1;
				list_players[i]->ownerColorID = -1;
			}
		}

		if(other.teamID != inflictor.teamID)
		{
			other.ownerPlayerID = inflictor.globalID;
			other.ownerColorID = inflictor.colorID;
		}
	}

	return false;	//we have not removed a player
}

bool CGM_Owned::playerkilledself(CPlayer &player)
{
	for(int i = 0; i < list_players_cnt; i++)
	{
		if(list_players[i]->ownerPlayerID == player.globalID)
		{
			list_players[i]->ownerPlayerID = -1;
			list_players[i]->ownerColorID = -1;
		}
	}

	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	return false;
}

void CGM_Owned::player1up(CPlayer &player)
{
	if(!gameover)
	{
		player.score->score += 25;

		if(player.score->score >= goal)
		{
			player.score->score = goal;
			winningteam = player.teamID;
			gameover = true;

			setupscoreboard();
			removeplayersbutteam(winningteam);
		}
		else if(player.score->score >= goal * 0.8 && !playedwarningsound)
		{
			playwarningsound();
		}
	}
}

char * CGM_Owned::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Owned %d", goal);
	return buffer64;
}

void CGM_Owned::toggleoptions()
{
	goal += 100;
	if(goal > 1000)
		goal = 100;
}


//Owned:
//Frag limit death match, but players get bonus frags for the number of players they have "owned"
CGM_Jail::CGM_Jail() :
	CGM_DM_Fraglimit()
{
	gamemode = game_mode_jail;
	goal = 20;
}

bool CGM_Jail::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	if(!gameover)
	{
		//Penalize killing your team mates
		if(inflictor.teamID == other.teamID)
		{
			inflictor.score->score -= 2;
			if(inflictor.score->score < 0)
				inflictor.score->score = 0;
		}
		else
		{
			if(inflictor.jailed > 0)
			{
				eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, inflictor.ix + (HALFPW) - 16, inflictor.iy + (HALFPH) - 16, 3, 8));
				ifsoundonplay(transform);
				inflictor.jailed = 0;
			}

			other.jailed = 1200;

			int jailedteams[4];
			
			int i;
			for(i = 0; i < score_cnt; i++)
				jailedteams[i] = game_values.teamcounts[i];

			//Figure out which teams have been jailed
			for(i = 0; i < list_players_cnt; i++)
			{
				if(list_players[i]->jailed > 0)
				{
					jailedteams[list_players[i]->teamID]--;
				}
			}

			//Determine if a single team is the only one not completely jailed
			int iTeamPoint = -1;
			for(i = 0; i < score_cnt; i++)
			{
				if(jailedteams[i] == 0)
					continue;

				if(iTeamPoint < 0)
				{
					iTeamPoint = i;
				}
				else
				{
					iTeamPoint = -1;
					break;
				}
			}

			if(iTeamPoint >= 0)
			{
				int numjailedplayers = 0;

				for(int i = 0; i < list_players_cnt; i++)
				{
					//If they weren't just the one killed and they were jailed, give them a transform cloud
					if(list_players[i]->jailed < 1200 && list_players[i]->jailed > 0)
					{
						eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, list_players[i]->ix + (HALFPW) - 16, list_players[i]->iy + (HALFPH) - 16, 3, 8));
						ifsoundonplay(transform);
					}

					if(list_players[i]->jailed > 0 && list_players[i]->teamID != inflictor.teamID)
						numjailedplayers++;
					
					list_players[i]->jailed = 0;
				}

				//Give extra bonus score for being on the non-jailed team
				if(numjailedplayers > 1)
					score[iTeamPoint].score++;
			}
		}

		if(inflictor.score->score >= goal)
		{
			winningteam = inflictor.teamID;
			gameover = true;

			removeplayersbutteam(winningteam);

			setupscoreboard();

			return true;	//we removed a player
		}
		else if(inflictor.score->score >= goal - 3 && !playedwarningsound)
		{
			playwarningsound();
		}
	}

	return false;	//we have not removed a player
}

void CGM_Jail::player1up(CPlayer &player)
{
	if(!gameover)
	{
		player.score->score++;
		player.jailed = 0;

		if(player.score->score >= goal)
		{
			player.score->score = goal;
			winningteam = player.teamID;
			gameover = true;

			removeplayersbutteam(winningteam);
			setupscoreboard();
		}
		else if(player.score->score >= goal - 3 && !playedwarningsound)
		{
			playwarningsound();
		}
	}
}

char * CGM_Jail::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Jail Limit %d", goal);
	return buffer64;
}


//Goomba Limit mode:
//Kill randomly appearing goomobas on map
//First one to kill the limit wins
void CGM_Goomba::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	gamemode = game_mode_goomba;
	playedwarningsound = false;

	spawntimer = (rand() % 90) + 90;
}


void CGM_Goomba::think()
{
	CGM_Coins::think();

	if(!gameover)
	{
		//Randomly spawn goombas
		if(--spawntimer <= 0)
		{
			interactionobjectcollisionitems.add(new MO_Goomba(&spr_goomba, 2, 8, rand() % 2 == 0, 30, 20, 1, 11));
			spawntimer = (rand() % 90) + 90;
		}
	}
}

char * CGM_Goomba::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Goomba %d", goal);
	return buffer64;
}


//Race
//Touch all the flying blocks in order
//Each successful curcuit you complete (before getting killed)
//Counts as one point
void CGM_Race::init()
{
	winningteam = -1;
	gameover = false;

	chicken = NULL;
	tagged = NULL;
	gamemode = game_mode_race;
	playedwarningsound = false;

	for(int k = 0; k < 4; k++)
	{
		interactionobjectsfront.add(new OMO_RaceGoal(&spr_racegoal, k));
		nextGoal[k] = 0;
	}
}

void CGM_Race::think()
{
	if(gameover)
		displayplayertext();
}

bool CGM_Race::playerkilledplayer(CPlayer &inflictor, CPlayer &other)
{
	//Don't count this kill for domination goal
	inflictor.score->score--;

	//Player loses control of his areas
	nextGoal[other.teamID] = 0;
	interactionobjectsfront.removePlayerRaceGoals(other.teamID);
	
	return false;		//we have not removed a player
}

bool CGM_Race::playerkilledself(CPlayer &player)
{
	if(player.bobomb)
		player.diedas = 2; //flag to use bobomb corpse sprite

	//Player loses control of his areas
	nextGoal[player.teamID] = 0;
	interactionobjectsfront.removePlayerRaceGoals(player.teamID);
	
	return false;		//we have not removed a player
}

void CGM_Race::player1up(CPlayer &player)
{
	if(!gameover)
	{
		player.score->score++;

		if(player.score->score >= goal)
		{
			player.score->score = goal;
			winningteam = player.teamID;
			gameover = true;

			removeplayersbutteam(winningteam);
			setupscoreboard();
		}
		else if(player.score->score >= goal - 2 && !playedwarningsound)
		{
			playwarningsound();
		}
	}
}

CGM_Race::CGM_Race()
{
	goal = 10;
}

char * CGM_Race::getMenuString(char *buffer64)
{
	sprintf(buffer64, "Race %d", goal);
	return buffer64;
}

void CGM_Race::toggleoptions()
{
	goal += 5;
	if(goal > 50)
		goal = 5;
}

void CGM_Race::setNextGoal(int teamID) 
{
	if(++nextGoal[teamID] >= 4)
	{
		nextGoal[teamID] = 0;
		interactionobjectsfront.removePlayerRaceGoals(teamID);
		
		if(!gameover)
		{
			score[teamID].score++;

			if(score[teamID].score >= goal)
			{
				score[teamID].score = goal;
				winningteam = teamID;
				gameover = true;
				
				removeplayersbutteam(winningteam);
				setupscoreboard();

			}
			else if(score[teamID].score >= goal - 2 && !playedwarningsound)
			{
				playwarningsound();
			}
		}
	}
}

