
#include "gfx.h"
#include <stdarg.h>		//atexit?
#include <stdlib.h>		//atexit?


#ifdef _WIN32
	#ifndef _XBOX
		#pragma comment(lib, "SDL.lib")
		#pragma comment(lib, "SDLmain.lib")
	#endif
#endif

extern SDL_Surface *blitdest;
extern SDL_Surface *screen;

#define GFX_BPP		0
#define GFX_FLAGS	SDL_SWSURFACE /*| SDL_HWACCEL*/
//#define GFX_FLAGS	SDL_HWSURFACE | SDL_DOUBLEBUF

//[colorcomponents][numcolors]
Uint8 * colorcodes[3];

//[numplayers][colorscheme][colorcomponents][numcolors]
Uint8 * colorschemes[4][5][3]; 
int numcolors = 0;

extern int x_shake;
extern int y_shake;

//gfx_init
bool gfx_init(int w, int h, bool fullscreen)
{
	printf("init SDL\n");
	if( SDL_Init(SDL_INIT_VIDEO ) < 0 )
	{
        printf("Couldn't initialize SDL: %s\n", SDL_GetError());
        return false;
    }

    // Clean up on exit
    atexit(SDL_Quit);
    
    if(fullscreen)
		screen = SDL_SetVideoMode(w, h, GFX_BPP, GFX_FLAGS | SDL_FULLSCREEN);
	else
		screen = SDL_SetVideoMode(w, h, GFX_BPP, GFX_FLAGS);

    if ( screen == NULL ) 
	{
        printf("Couldn't set video mode %dx%d: %s\n", w, h, SDL_GetError());
		return false;
    }

    printf(" running @ %dx%d %dbpp (done)\n", w, h, screen->format->BitsPerPixel);	

#ifdef _XBOX
	SDL_Surface * palette = SDL_LoadBMP("d:\\gfx\\palette.bmp");
#else
	SDL_Surface * palette = SDL_LoadBMP("gfx/palette.bmp");
#endif

	if ( palette == NULL ) 
	{
        printf("Couldn't load color palette %dx%d: %s\n", w, h, SDL_GetError());
		return false;
    }

	numcolors = palette->w;

	for(int k = 0; k < 3; k++)
	{
		colorcodes[k] = new Uint8[numcolors];

		for(int i = 0; i < 4; i++)
			for(int j = 0; j < 5; j++)
				colorschemes[i][j][k] = new Uint8[numcolors];
	}

	if(SDL_MUSTLOCK(palette))
		SDL_LockSurface(palette);

	int counter = 0;

	for(int k = 0; k < numcolors; k++)
	{
		colorcodes[0][k] = ((Uint8*)palette->pixels)[counter++];
		colorcodes[1][k] = ((Uint8*)palette->pixels)[counter++];
		colorcodes[2][k] = ((Uint8*)palette->pixels)[counter++];
	}

	counter += palette->pitch - palette->w * 3;

	for(int i = 0; i < 4; i++)
	{
		for(int j = 0; j < 5; j++)
		{
			for(int m = 0; m < numcolors; m++)
			{
				colorschemes[i][j][0][m] = ((Uint8*)palette->pixels)[counter++];
				colorschemes[i][j][1][m] = ((Uint8*)palette->pixels)[counter++];
				colorschemes[i][j][2][m] = ((Uint8*)palette->pixels)[counter++];
			}

			counter += palette->pitch - palette->w * 3;
		}
	}
	
	SDL_UnlockSurface(palette);
	SDL_FreeSurface(palette);

	return true;
}

void gfx_setresolution(int w, int h, bool fullscreen)
{
	Uint32 flags = GFX_FLAGS;
	if(fullscreen)
		flags |= SDL_FULLSCREEN;
	screen = SDL_SetVideoMode(w, h, GFX_BPP, flags);
}

void gfx_close()
{
	for(int i = 0; i < 3; i++)
	{
		delete [] colorcodes[i];

		for(int j = 0; j < 4; j++)
			for(int k = 0; k < 5; k++)
				delete [] colorschemes[j][k][i];
	}
}



SDL_Surface * gfx_createskinsurface(SDL_Surface * skin, int spriteindex, Uint8 r, Uint8 g, Uint8 b, int colorScheme, bool expand, bool reverse)
{
	int loops = 1;
	if(expand)
		loops = 5;

	//Blit over loaded skin into player image set
	SDL_Surface * temp = SDL_CreateRGBSurface(skin->flags, 32 * loops, 32, skin->format->BitsPerPixel, 0, 0, 0, 0);
	
	if(SDL_MUSTLOCK(temp))
		SDL_LockSurface(temp);

	if(SDL_MUSTLOCK(skin))
		SDL_LockSurface(skin);

	int skincounter = spriteindex * 96;
	int tempcounter = 0;

	int reverseoffset = 0;

	for(int j = 0; j < 32; j++)
	{
		for(int i = 0; i < 32; i++)
		{
			if(reverse)
				reverseoffset = (31 - (i * 2)) * 3;

			bool fFoundColor = false;
			for(int m = 0; m < numcolors; m++)
			{
				if( ((Uint8*)skin->pixels)[skincounter] == colorcodes[0][m] && 
					((Uint8*)skin->pixels)[skincounter + 1] == colorcodes[1][m] &&
					((Uint8*)skin->pixels)[skincounter + 2] == colorcodes[2][m])
				{
					for(int k = 0; k < loops; k++)
					{
						((Uint8*)temp->pixels)[tempcounter + k * 96 + reverseoffset] = colorschemes[colorScheme][k][0][m];
						((Uint8*)temp->pixels)[tempcounter + k * 96 + reverseoffset + 1] = colorschemes[colorScheme][k][1][m];
						((Uint8*)temp->pixels)[tempcounter + k * 96 + reverseoffset + 2] = colorschemes[colorScheme][k][2][m];
					}
					
					fFoundColor = true;
					break;
				}
			}

			if(!fFoundColor)
			{
				for(int k = 0; k < loops; k++)
				{
					((Uint8*)temp->pixels)[tempcounter + k * 96 + reverseoffset] = ((Uint8*)skin->pixels)[skincounter];
					((Uint8*)temp->pixels)[tempcounter + k * 96 + reverseoffset + 1] = ((Uint8*)skin->pixels)[skincounter + 1];
					((Uint8*)temp->pixels)[tempcounter + k * 96 + reverseoffset + 2] = ((Uint8*)skin->pixels)[skincounter + 2];
				}
			}

			skincounter += 3;
			tempcounter += 3;
		}

		skincounter += 480 + skin->pitch - (skin->w * 3);
		tempcounter += 96 * (loops - 1) + temp->pitch - (temp->w * 3);
	}

	SDL_UnlockSurface(skin);
	SDL_UnlockSurface(temp);

	if( SDL_SetColorKey(temp, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(temp->format, r, g, b)) < 0)
	{
		printf("\n ERROR: Couldn't set ColorKey + RLE for new skin surface: %s\n", SDL_GetError());
		return NULL;
	}

	SDL_Surface * final = NULL;
	if( (final = SDL_DisplayFormat(temp)) == NULL)
	{
		printf("\n ERROR: Couldn't create new surface using SDL_DisplayFormat(): %s\n", SDL_GetError());
		return NULL;
	}
	SDL_FreeSurface(temp);

	return final;
}


bool gfx_createmenuskin(gfxSprite * gSprite, const char *filename, Uint8 r, Uint8 g, Uint8 b, int colorScheme)
{
    // Load the BMP file into a surface
	SDL_Surface * skin = SDL_LoadBMP(filename);

    if (skin == NULL)
	{
		printf("\n ERROR: Couldn't load %s: %s\n", filename, SDL_GetError());
        return false;
    }

	SDL_Surface * skinSurface = gfx_createskinsurface(skin, 0, r, g, b, colorScheme, true, false);

	SDL_FreeSurface(skin);

	if (skinSurface == NULL)
	{
		printf("\n ERROR: Couldn't create menu skin from %s: %s\n", filename, SDL_GetError());
        return false;
    }

	gSprite->setSurface(skinSurface);

	return true;
}


bool gfx_createfullskin(gfxSprite ** gSprites, const char *filename, Uint8 r, Uint8 g, Uint8 b, int colorScheme)
{
    // Load the BMP file into a surface
	SDL_Surface * skin = SDL_LoadBMP(filename);

    if (skin == NULL)
	{
		printf("\n ERROR: Couldn't load %s: %s\n", filename, SDL_GetError());
        return false;
    }

	for(int k = 0; k < 4; k++)
	{
		for(int j = 0; j < 2; j++)
		{
			SDL_Surface * skinSurface = gfx_createskinsurface(skin, k, r, g, b, colorScheme, true, j != 0);

			if (skinSurface == NULL)
			{
				printf("\n ERROR: Couldn't create menu skin from %s: %s\n", filename, SDL_GetError());
				SDL_FreeSurface(skin);
				return false;
			}

			gSprites[(k * 2) + j]->setSurface(skinSurface);
		}
	}

	//Dead Flying Sprite
	SDL_Surface * skinSurface = gfx_createskinsurface(skin, 4, r, g, b, colorScheme, false, false);

	if (skinSurface == NULL)
	{
		printf("\n ERROR: Couldn't create menu skin from %s: %s\n", filename, SDL_GetError());
		SDL_FreeSurface(skin);
		return false;
	}

	gSprites[8]->setSurface(skinSurface);

	//Dead Stomped Sprite
	skinSurface = gfx_createskinsurface(skin, 5, r, g, b, colorScheme, false, false);

	if (skinSurface == NULL)
	{
		printf("\n ERROR: Couldn't create menu skin from %s: %s\n", filename, SDL_GetError());
		SDL_FreeSurface(skin);
		return false;
	}

	gSprites[9]->setSurface(skinSurface);

	SDL_FreeSurface(skin);
	
	return true;
}

//gfxSprite

gfxSprite::gfxSprite()
{
	m_bltrect.x = 0;
	m_bltrect.y = 0;
	m_bltrect.w = 0;
	m_bltrect.h = 0;
	m_picture = NULL;
}

gfxSprite::~gfxSprite()
{
	//free the allocated BMP surface 
	freeSurface();
}

bool gfxSprite::init(const char *filename, Uint8 r, Uint8 g, Uint8 b)
{
	printf("loading sprite %s ... ", filename);

	if(m_picture != NULL)
	{
		printf("sprite already loaded ... deleting ...");
		
		SDL_FreeSurface(m_picture);
		m_picture = NULL;
		
		printf("ok ...");
	}

    // Load the BMP file into a surface
	m_picture = SDL_LoadBMP(filename);

    if (m_picture == NULL)
	{
		printf("\n ERROR: Couldn't load %s: %s\n", filename, SDL_GetError());
        return false;
    }

	if( SDL_SetColorKey(m_picture, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(m_picture->format, r, g, b)) < 0)
	{
		printf("\n ERROR: Couldn't set ColorKey + RLE for %s: %s\n", filename, SDL_GetError());
		return false;
	}

	SDL_Surface *temp;
	if( (temp = SDL_DisplayFormat(m_picture)) == NULL)
	{
		printf("\n ERROR: couldn't convert %s to display format: %s\n", filename, SDL_GetError());
		return false;
	}

	SDL_FreeSurface(m_picture);
	m_picture = temp;

	m_bltrect.w = m_picture->w;
	m_bltrect.h = m_picture->h;

	printf("done\n");
	return true;
}

bool gfxSprite::init(const char *filename, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
{
	printf("loading sprite %s ... ", filename);

	if(m_picture != NULL)
	{
		printf("sprite already loaded ... deleting ...");
		
		SDL_FreeSurface(m_picture);
		m_picture = NULL;
		
		printf("ok ...");
	}

    // Load the BMP file into a surface
	m_picture = SDL_LoadBMP(filename);

    if (m_picture == NULL)
	{
		printf("\n ERROR: Couldn't load %s: %s\n", filename, SDL_GetError());
        return false;
    }

	if( SDL_SetColorKey(m_picture, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(m_picture->format, r, g, b)) < 0)
	{
		printf("\n ERROR: Couldn't set ColorKey + RLE for %s: %s\n", filename, SDL_GetError());
		return false;
	}

	if( (SDL_SetAlpha(m_picture, SDL_SRCALPHA | SDL_RLEACCEL, a)) < 0)
	{
		printf("\n ERROR: couldn't set alpha  on file %s: %s\n", filename, SDL_GetError());
		return false;
	}
	
	SDL_Surface *temp;
	if( (temp = SDL_DisplayFormatAlpha(m_picture)) == NULL)
	{
		printf("\n ERROR: couldn't convert %s to display format: %s\n", filename, SDL_GetError());
		return false;
	}
	SDL_FreeSurface(m_picture);
	m_picture = temp;


	m_bltrect.w = m_picture->w;
	m_bltrect.h = m_picture->h;

	printf("done\n");
	return true;
}

bool gfxSprite::init(const char *filename) 
{
	printf("loading sprite %s ... ", filename);

	if(m_picture != NULL)
	{
		printf("sprite already loaded - deleting ...");
		
		SDL_FreeSurface(m_picture);
		m_picture = NULL;
		
		printf("ok ...");
	}

    // Load the BMP file into a surface
	m_picture = SDL_LoadBMP(filename);

    if (m_picture == NULL)
	{
		printf("\n ERROR: Couldn't load %s: %s\n", filename, SDL_GetError());
        return false;
    }

	SDL_Surface *temp;
	if( (temp = SDL_DisplayFormat(m_picture)) == NULL)
	{
		printf("\n ERROR: couldn't convert %s to display format: %s\n", filename, SDL_GetError());
		return false;
	}

	SDL_FreeSurface(m_picture);
	m_picture = temp;

	m_bltrect.w = m_picture->w;
	m_bltrect.h = m_picture->h;

	m_srcrect.x = 0;
	m_srcrect.y = 0;
	m_srcrect.w = m_picture->w;
	m_srcrect.h = m_picture->h;

	printf("done\n");
	return true;
}


bool gfxSprite::draw(int x, int y)
{
	m_bltrect.x = x + x_shake;
	m_bltrect.y = y + y_shake;

	if(SDL_BlitSurface(m_picture, NULL, blitdest, &m_bltrect) < 0)
	{
		fprintf(stderr, "BlitSurface error: %s\n", SDL_GetError());
		return false;
	}

	return true;
}

bool gfxSprite::draw(int x, int y, int srcx, int srcy, int w, int h)
{
	m_bltrect.x = x + x_shake;
	m_bltrect.y = y + y_shake;
	m_bltrect.w = w;
	m_bltrect.h = h;

	m_srcrect.x = srcx;
	m_srcrect.y = srcy;
	m_srcrect.w = w;
	m_srcrect.h = h;

	// Blit onto the screen surface
	if(SDL_BlitSurface(m_picture, &m_srcrect, blitdest, &m_bltrect) < 0)
	{
		fprintf(stderr, "BlitSurface error: %s\n", SDL_GetError());
		return false;
	}

	return true;
}

void gfxSprite::setalpha(int alpha)
{
	if( (SDL_SetAlpha(m_picture, SDL_SRCALPHA | SDL_RLEACCEL, alpha)) < 0)
	{
		printf("\n ERROR: couldn't set alpha on sprite: %s\n", SDL_GetError());
	}
}

void gfxSprite::freeSurface()
{
	if(m_picture)
	{
		SDL_FreeSurface(m_picture);
		m_picture = NULL;
	}
}


// gfxFont

bool gfxFont::init(const char *filename)
{
	printf("loading font %s ... ", filename);

	SDL_Surface *fontsurf = SDL_LoadBMP(filename);
	if(fontsurf == NULL)
	{
		printf("\n ERROR: couldn't load file %s: %s\n", filename, SDL_GetError());
		return false;
	}

	m_font = SFont_InitFont(fontsurf);
	if(!m_font) {
		printf("\n ERROR: an error occured while loading the font.\n");
		return false;
	}

	printf("done\n");
	return true;
}

void gfxFont::draw(int x, int y, const char *s)
{
	if(y + getHeight() < 0)
		return;

	SFont_Write(blitdest, m_font, x, y, s);
}

void gfxFont::drawChopRight(int x, int y, int width, const char *s)
{
	if(y + getHeight() < 0)
		return;
		
	SFont_WriteChopRight(blitdest, m_font, x, y, width, s);
}

void gfxFont::drawCentered(int y, const char *text)
{
	if(y + getHeight() < 0)
		return;
		
	SFont_WriteCenter(blitdest, m_font, y, text);
};

void gfxFont::drawRightJustified(int x, int y, const char *s, ...)
{
	char buffer[256];

	va_list zeiger;
	va_start(zeiger, s);
	vsprintf(buffer, s, zeiger);
	va_end(zeiger);

	if(y + getHeight() < 0)
		return;
		
	SFont_WriteRight(blitdest, m_font, x, y, buffer);
};


void gfxFont::drawf(int x, int y, char *s, ...)
{
	char buffer[256];

	va_list zeiger;
	va_start(zeiger, s);
	vsprintf(buffer, s, zeiger);
	va_end(zeiger);

	draw(x,y,buffer);
}

void gfxFont::setalpha(int alpha)
{
	if( (SDL_SetAlpha(m_font->Surface, SDL_SRCALPHA | SDL_RLEACCEL, alpha)) < 0)
	{
		printf("\n ERROR: couldn't set alpha on sprite: %s\n", SDL_GetError());
	}
}
