#include <string.h>
#include <math.h>

#include "global.h"

void removeifprojectile(IO_MovingObject * object, bool playsound)
{
	if((object->movingObjectType == movingobject_fireball || object->movingObjectType == movingobject_hammer) && !object->dead)
	{
		if(projectiles[((MO_Fireball*)object)->playerID] > 0)
			projectiles[((MO_Fireball*)object)->playerID]--;

		object->dead = true;

		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, object->ix + (object->iw >> 1) - 16, object->iy + (object->ih >> 1) - 16, 3, 4));
		
		if(playsound)
			ifsoundonplay(hit);
	}
}

extern void killplayer(CPlayer &killer, CPlayer &killed, int deathstyle, int killstyle);
extern void addawardkill(CPlayer * killer, CPlayer * killed, int killstyle);
extern int lookupteamid(int id);

//------------------------------------------------------------------------------
// class Interaction Object base class
//------------------------------------------------------------------------------
CInteractionObject::CInteractionObject(gfxSprite *nspr1, int x, int y)
{
	dead = false;
	
	spr = nspr1;
	ix = x;
	iy = y;

	iw = spr->getWidth();
	ih = spr->getHeight();

	nx = (float)(ix);
	ny = (float)(iy);
	
	velx = 0.0f;
	vely = 0.0f;
	
	state = 0;

	collisionWidth = iw;
	collisionHeight = ih;
	collisionOffsetX = 0;
	collisionOffsetY = 0;
}

//------------------------------------------------------------------------------
// class Block base class
//------------------------------------------------------------------------------

IO_Block::IO_Block(gfxSprite *nspr, int x, int y) :
	CInteractionObject(nspr, x, y)
{
	bumpPlayer = NULL;

	fposx = nx;
	fposy = ny;

	iposx = x;
	iposy = y;
}


void IO_Block::draw()
{
	spr->draw(ix, iy);
}


void IO_Block::update()
{

}

bool IO_Block::collide(CPlayer * player, int direction, bool useBehavior)
{
	if(direction == 2)
		return hittop(player, useBehavior);
	else if(direction == 0)
		return hitbottom(player, useBehavior);
	else if(direction == 1)
		return hitleft(player, useBehavior);
	else
		return hitright(player, useBehavior);
}

bool IO_Block::hittop(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->yi(iposy - PH - 1);
		player->inair = false;
		player->fallthrough = false;
		player->killsinrowinair = 0;
		player->vely = 1.0f;
	}

	return false;
}

bool IO_Block::hitbottom(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		//Player bounces off 
		player->vely = CapVelocity(-player->vely * BOUNCESTRENGTH);
		player->yi(iposy + ih + 1);
	}

	return false;
}

bool IO_Block::hitright(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->xi(iposx + iw);
		player->oldx = player->ix;
		player->velx = 0.0f;
	}

	return false;
}

bool IO_Block::hitleft(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->xi(iposx - PW - 1);
		player->oldx = player->ix;
		player->velx = 0.0f;
	}

	return false;
}

bool IO_Block::collide(IO_MovingObject * object, int direction)
{
	if(direction == 2)
		return hittop(object);
	else if(direction == 0)
		return hitbottom(object);
	else if(direction == 1)
		return hitleft(object);
	else
		return hitright(object);
}

bool IO_Block::hittop(IO_MovingObject * object)
{
	object->yi(iposy - object->collisionHeight - 1);
	object->oldy = object->iy;
	object->vely = object->bounce;
	return true;
}

bool IO_Block::hitbottom(IO_MovingObject * object)
{
	object->yi(iposy + ih + 1);
	object->oldy = object->iy;
	object->vely = -object->vely;
	return true;
}

bool IO_Block::hitright(IO_MovingObject * object)
{
	object->xi(iposx + iw);
	object->oldx = object->ix;
	object->velx = -object->velx;
	return true;
}

bool IO_Block::hitleft(IO_MovingObject * object)
{
	object->xi(iposx - object->collisionWidth - 1);
	object->oldx = object->ix;
	object->velx = -object->velx;
	return true;
}

//------------------------------------------------------------------------------
// class powerup block
//------------------------------------------------------------------------------
B_PowerupBlock::B_PowerupBlock(gfxSprite *nspr1, int x, int y, int iNumSpr, int aniSpeed) :
	IO_Block(nspr1, x, y)
{
	iw = spr->getWidth() >> 2;
	ih = spr->getHeight() >> 1;  //This sprite has two images (unused and used blocks)
	collisionWidth = iw;
	timer = 0;
	side = true;
	iNumSprites = iNumSpr;
	animationSpeed = aniSpeed;
	animationTimer = 0;
	drawFrame = 0;
}


void B_PowerupBlock::draw()
{
	if(state == 0)
		spr->draw(ix, iy, drawFrame, 0, iw, ih);
	else
		spr->draw(ix, iy, 0, ih, iw, ih);
}


void B_PowerupBlock::update()
{
	if(state > 0)
	{
		yf(ny + vely);
		
		if(state == 1 && fabsf(fposy - ny) > 10.0f)
		{
			vely = -vely;
			state = 2;
			bumpPlayer = NULL;
		}
		else if(state == 2 && fabsf(fposy - ny) < VELBLOCKBOUNCE)
		{
			vely = 0.0f;
			state = 3;
			yi(iposy);

			int rnd = rand() % 120;
			if(rnd >= 0 && rnd < 20)
				interactionobjectcollisionitems.add(new PU_StarPowerup(&spr_starpowerup, ix + 1, iy - 1, 3, side, 2, 30, 30, 1, 1));
			else if(rnd >= 20 && rnd < 45)
				interactionobjectcollisionitems.add(new PU_1upPowerup(&spr_1uppowerup, ix + 1, iy - 1, 1, side, 1000000, 30, 30, 1, 1));
			else if(rnd >= 45 && rnd < 70)
				interactionobjectcollisionitems.add(new PU_FirePowerup(&spr_firepowerup, ix + 1, iy - 1, 1, side, 1000000, 30, 30, 1, 1));
			else if(rnd >= 70 && rnd < 80)
				interactionobjectcollisionitems.add(new PU_HammerPowerup(&spr_hammerpowerup, ix + 1, iy - 1, 1, side, 1000000, 30, 30, 1, 1));
			else if(rnd >= 80 && rnd < 90)
				interactionobjectcollisionitems.add(new PU_PoisonPowerup(&spr_poisonpowerup, ix + 1, iy - 1, 1, side, 1000000, 30, 30, 1, 1));
			else if(rnd >= 90 && rnd < 100)
				interactionobjectcollisionitems.add(new PU_ClockPowerup(&spr_clockpowerup, ix + 1, iy - 1, 1, side, 1000000, 30, 30, 1, 1));
			else if(rnd >= 100 && rnd < 110)
				interactionobjectcollisionitems.add(new PU_BobombPowerup(&spr_bobombpowerup, ix + 1, iy - 1, 1, side, 1000000, 30, 30, 1, 1));
			else if(rnd >= 110 && rnd < 115)
				interactionobjectcollisionitems.add(new PU_PowPowerup(&spr_powpowerup, ix + 1, iy - 1, 1, side, 1000000, 30, 30, 1, 1));
			else
				interactionobjectcollisionitems.add(new PU_BulletBillPowerup(&spr_bulletbillpowerup, ix + 1, iy - 1, 1, side, 1000000, 30, 30, 1, 1));

			ifsoundonplay(sprout);
		}
		else if(state == 3)
		{
			if(++timer == 2000)
			{
				timer = 0;
				state = 0;
			}
		}
	}

	if(++animationTimer >= animationSpeed)
	{
		animationTimer = 0;

		drawFrame += iw;
		if(drawFrame >= iNumSprites * iw)
		{
			drawFrame = 0;
		}
	}
}

bool B_PowerupBlock::hittop(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->yi(iposy - PH - 1);
		player->inair = false;
		player->fallthrough = false;
		player->killsinrowinair = 0;
	}

	if(state == 1)
		if(bumpPlayer && !player->invincible && !player->spawninvincible && (player->teamID != bumpPlayer->teamID || game_values.friendlyfire))
			killplayer(*bumpPlayer, *player, death_style_jump, kill_style_bounce);
		else
			player->vely = -VELNOTEBLOCKREPEL;
	else if(useBehavior)
		player->vely = 1.0f;

	return false;
}

bool B_PowerupBlock::hitbottom(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		//Player bounces off 
		player->vely = CapVelocity(-player->vely * BOUNCESTRENGTH);
		player->yi(iposy + ih + 1);

		ifsoundonplay(bump);

		if(state == 0)
		{
			bumpPlayer = player;
			vely = -VELBLOCKBOUNCE;
			state = 1;
			side = player->ix + HALFPW < ix + (iw >> 1);
		}
	}

	return false;
}

bool B_PowerupBlock::hittop(IO_MovingObject * object)
{
	object->yi(iposy - object->collisionHeight - 1);
	object->oldy = object->iy;
	
	if(state == 1 && object->bounce == 1.0f)
	{
		if(object->getMovingObjectType() == movingobject_goomba)
		{
			object->dead = true;
			ifsoundonplay(kicksound);
			eyecandyfront.add(new EC_FallingObject(&spr_goombadeadflying, object->ix, object->iy, -VELJUMP / 2.0f));
	
			if(!game_values.gamemode->gameover && bumpPlayer)
			{
				addawardkill(bumpPlayer, NULL, kill_style_goomba);
				bumpPlayer->score->score++;
			}
		}
		else
		{
			object->vely = -VELNOTEBLOCKREPEL;
		}

		return false;
	}
	else
	{
		object->vely = object->bounce;
	}

	return true;
}


//------------------------------------------------------------------------------
// class breakable block
//------------------------------------------------------------------------------
B_BreakableBlock::B_BreakableBlock(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed) :
	IO_Block(nspr, x, y)
{
	col = x / TILESIZE;
	row = y / TILESIZE;

	iw = nspr->getWidth() >> 2;
	iNumSprites = iNumSpr;
	animationSpeed = aniSpeed;
	animationTimer = 0;
	drawFrame = 0;
}

void B_BreakableBlock::draw()
{
	if(state == 0)
		spr->draw(ix, iy, drawFrame, 0, iw, ih);
}

void B_BreakableBlock::update()
{
	if(state == 1)
	{
		state = 2;
	}
	else if(state == 2)
	{
		bumpPlayer = NULL;
		dead = true;
		map.blockdata[col][row] = NULL;
	}

	if(++animationTimer >= animationSpeed)
	{
		animationTimer = 0;

		drawFrame += iw;
		if(drawFrame >= iNumSprites * iw)
		{
			drawFrame = 0;
		}
	}
}


bool B_BreakableBlock::hittop(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->yi(iposy - PH - 1);
		player->inair = false;
		player->fallthrough = false;
		player->killsinrowinair = 0;
	}

	if(state == 1 || state == 2)
		if(bumpPlayer && !player->invincible && !player->spawninvincible && (player->teamID != bumpPlayer->teamID || game_values.friendlyfire))
			killplayer(*bumpPlayer, *player, death_style_jump, kill_style_bounce);
		else
			player->vely = -VELNOTEBLOCKREPEL;
	else if(useBehavior)
		player->vely = 1.0f;

	return false;
}

bool B_BreakableBlock::hitbottom(CPlayer * player, bool useBehavior)
{
	if(useBehavior && state == 0)
	{
		eyecandyfront.add(new EC_BrokenBlock(&spr_brokenBlock[0], ix, iy, -2.0f, -7.0f));
		eyecandyfront.add(new EC_BrokenBlock(&spr_brokenBlock[1], ix + 16, iy, 2.0f, -7.0f));
		eyecandyfront.add(new EC_BrokenBlock(&spr_brokenBlock[2], ix, iy + 16, -2.0f, -4.0f));
		eyecandyfront.add(new EC_BrokenBlock(&spr_brokenBlock[3], ix + 16, iy + 16, 2.0f, -4.0f));
		
		//When breaking a block, you smash through with a small velocity, but this allows for breaking two blocks at once
		/*
		if(player->vely < -VELMAXBREAKBLOCK)
			player->vely = -VELMAXBREAKBLOCK;
		*/
		player->vely = CapVelocity(-player->vely * BOUNCESTRENGTH);
		player->yi(iposy + ih + 1);
		
		state = 1;
		bumpPlayer = player;

		ifsoundonplay(breakblock);
	}

	return false;
}

bool B_BreakableBlock::hittop(IO_MovingObject * object)
{
	object->yi(iposy - object->collisionHeight - 1);
	object->oldy = object->iy;
	
	if((state == 1  || state == 2) && object->bounce == 1.0f)
	{
		if(object->getMovingObjectType() == movingobject_goomba)
		{
			object->dead = true;
			ifsoundonplay(kicksound);
			eyecandyfront.add(new EC_FallingObject(&spr_goombadeadflying, object->ix, object->iy, -VELJUMP / 2.0f));
	
			if(!game_values.gamemode->gameover && bumpPlayer)
			{
				addawardkill(bumpPlayer, NULL, kill_style_goomba);
				bumpPlayer->score->score++;
			}
		}
		else
		{
			object->vely = -VELNOTEBLOCKREPEL;
		}

		return false;
	}
	else
	{
		object->vely = object->bounce;
	}

	return true;
}

//------------------------------------------------------------------------------
// class note block
//------------------------------------------------------------------------------
B_NoteBlock::B_NoteBlock(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed) :
	IO_Block(nspr, x, y)
{
	iw = nspr->getWidth() >> 2;
	iNumSprites = iNumSpr;
	animationSpeed = aniSpeed;
	animationTimer = 0;
	drawFrame = 0;
}

void B_NoteBlock::draw()
{
	spr->draw(ix, iy, drawFrame, 0, iw, ih);
}

void B_NoteBlock::update()
{
	if(state > 0)
	{
		xf(nx + velx);
		yf(ny + vely);

		if(state == 1 && fabsf(fposx - nx) > 10.0f)
		{
			velx = -velx;
			state = 2;
		}
		else if(state == 2 && fabsf(fposx - nx) < VELNOTEBLOCKBOUNCE)
		{
			velx = 0.0f;
			state = 0;
			xf(fposx);
			yf(fposy);
		}
		else if(state == 3 && fabsf(fposy - ny) > 10.0f)
		{
			vely = -vely;
			state = 4;
		}
		else if(state == 4 && fabsf(fposy - ny) < VELNOTEBLOCKBOUNCE)
		{
			vely = 0.0f;
			state = 0;
			xf(fposx);
			yf(fposy);
			bumpPlayer = NULL;
		}
	}

	if(++animationTimer >= animationSpeed)
	{
		animationTimer = 0;

		drawFrame += iw;
		if(drawFrame >= iNumSprites * iw)
		{
			drawFrame = 0;
		}
	}
}

bool B_NoteBlock::hittop(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->yi(iposy - PH - 1);
		player->vely = -VELNOTEBLOCKREPEL;
		player->inair = false;
		player->fallthrough = false;
		player->killsinrowinair = 0;
		player->superjumptimer = 4;
	
		if(state == 0)
		{
			vely = VELNOTEBLOCKBOUNCE;
			state = 3;
		}

		ifsoundonplay(bump);
	}
	
	return false;
}

bool B_NoteBlock::hitbottom(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->yi(iposy + ih + 1);
		player->vely = VELNOTEBLOCKREPEL;

		if(state == 0)
		{
			bumpPlayer = player;
			vely = -VELNOTEBLOCKBOUNCE;
			state = 3;
		}

		ifsoundonplay(bump);
	}

	return false;
}

bool B_NoteBlock::hitright(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->xi(iposx + iw);
		player->oldx = player->ix;
		player->velx = VELNOTEBLOCKREPEL;

		if(state == 0)
		{
			velx = -VELNOTEBLOCKBOUNCE;
			state = 1;
		}

		ifsoundonplay(bump);
	}

	return false;
}

bool B_NoteBlock::hitleft(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->xi(iposx - PW - 1);
		player->oldx = player->ix;
		player->velx = -VELNOTEBLOCKREPEL;

		if(state == 0)
		{
			velx = VELNOTEBLOCKBOUNCE;
			state = 1;
		}

		ifsoundonplay(bump);
	}

	return false;
}

bool B_NoteBlock::hittop(IO_MovingObject * object)
{
	object->yi(iposy - object->collisionHeight - 1);
	object->oldy = object->iy;
	
	if(state == 3 && object->bounce == 1.0f)
	{
		if(object->getMovingObjectType() == movingobject_goomba)
		{
			object->dead = true;
			ifsoundonplay(kicksound);
			eyecandyfront.add(new EC_FallingObject(&spr_goombadeadflying, object->ix, object->iy, -VELJUMP / 2.0f));
	
			if(!game_values.gamemode->gameover && bumpPlayer)
			{
				addawardkill(bumpPlayer, NULL, kill_style_goomba);
				bumpPlayer->score->score++;
			}
		}
		else
		{
			object->vely = -VELNOTEBLOCKREPEL;
		}

		return false;
	}
	else
	{
		object->vely = object->bounce;
	}

	return true;
}

//------------------------------------------------------------------------------
// class donut block
//------------------------------------------------------------------------------
B_DonutBlock::B_DonutBlock(gfxSprite *nspr, int x, int y) :
	IO_Block(nspr, x, y)
{
	counter = 0;
	jigglex = 0;
	jigglecounter = 0;
	col = x / TILESIZE;
	row = y / TILESIZE;
}


void B_DonutBlock::draw()
{
	spr->draw(ix + jigglex, iy);
}


void B_DonutBlock::update()
{
	//If a player is standing on us, jiggle and then fall
	if(state == 0)
	{
		counter = 0;
		jigglex = 0;
		jigglecounter = 0;
	}
	else
	{
		if(++jigglecounter > 1)
		{
			jigglecounter = 0;

			if(jigglex == 2)
				jigglex = -2;
			else
				jigglex = 2;
		}

		if(++counter > 50)
		{
			eyecandyfront.add(new EC_FallingObject(&spr_donutblock, ix, iy, 0.0f));
			dead = true;
			map.blockdata[col][row] = NULL;
		}
	}

	state = 0;
}

bool B_DonutBlock::hittop(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->yi(iposy - PH - 1);
		player->inair = false;
		player->fallthrough = false;
		player->killsinrowinair = 0;
		player->vely = 1.0f;

		state = 1;
	}

	return false;
}

//------------------------------------------------------------------------------
// class flip block
// Known Bug:  If you have two flip blocks vertically aligned like this:
//
// ['']  or  ['']['']  etc.
// ['']      ['']['']
//
// And you hit both of them from the bottom, then keep jumping so that the bottom one
// stops to support you, and then the top one stops spinning, you can "hide" under the
// top block.  I was thinking about fixing this, but I don't know what the correct 
// behavior should be if you're in a flip block when it stops spinning.  Plus, it is kind
// of cool to hide under these blocks.  You are somewhat invincible, but someone can come 
// along and hit your support out from under you and you'll fall out and if they have star
// power, you're dead.  Plus fireballs have a large enough collision box that you can be hit
// with them too.
//------------------------------------------------------------------------------
B_FlipBlock::B_FlipBlock(gfxSprite *nspr, int x, int y) :
	IO_Block(nspr, x, y)
{
	iw = spr->getWidth() >> 2;
	collisionWidth = iw;

	counter = 0;
	frame = 0;
	timer = 0;
}


void B_FlipBlock::draw()
{
	spr->draw(ix, iy, frame * iw, 0, iw, ih);
}


void B_FlipBlock::update()
{
	if(state == 1)
	{
		if(++counter >= 10)
		{
			counter = 0;

			if(++frame > 3)
			{
				frame = 0;
			}
		}

		if(++timer >= 240)
		{
			frame = 0;
			counter = 0;
			timer = 0;
			state = 0;
		}
	}
}

bool B_FlipBlock::collide(CPlayer * player, int direction, bool useBehavior)
{
	if(player->oldy + PH <= iposy && direction == 2)
		return hittop(player, useBehavior);
	else if(player->oldy >= iposy + ih && direction == 0)
		return hitbottom(player, useBehavior);
	else if(player->oldx + PW <= iposx && direction == 1)
		return hitleft(player, useBehavior);
	else if(player->oldx >= iposx + iw && direction == 3)
		return hitright(player, useBehavior);

	return true;
}

bool B_FlipBlock::hittop(CPlayer * player, bool useBehavior)
{
	if(useBehavior && state == 0)
	{
		player->yi(iposy - PH - 1);
		player->inair = false;
		player->fallthrough = false;
		player->killsinrowinair = 0;
		player->vely = 1.0f;
		return false;
	}

	return true;
}

bool B_FlipBlock::hitbottom(CPlayer * player, bool useBehavior)
{
	if(useBehavior && state == 0)
	{
		player->vely = CapVelocity(-player->vely * BOUNCESTRENGTH);
		player->yi(iposy + ih + 1);
		vely = -VELBLOCKBOUNCE;
		
		state = 1;
		frame = 1;
		return false;
	}

	return true;
}

bool B_FlipBlock::hitright(CPlayer * player, bool useBehavior)
{
	if(useBehavior && state == 0)
	{
		player->xi(iposx + iw);
		player->oldx = player->ix;
		player->velx = 0.0f;
		return false;
	}
	
	return true;
}

bool B_FlipBlock::hitleft(CPlayer * player, bool useBehavior)
{
	if(useBehavior && state == 0)
	{
		player->xi(iposx - PW - 1);
		player->oldx = player->ix;
		player->velx = 0.0f;
		return false;
	}

	return true;
}

bool B_FlipBlock::collide(IO_MovingObject * object, int direction)
{
	if(object->oldy + object->collisionHeight <= iposy && direction == 2)
		return hittop(object);
	else if(object->oldy >= iposy + ih && direction == 0)
		return hitbottom(object);
	else if(object->oldx + object->collisionWidth <= iposx && direction == 1)
		return hitleft(object);
	else if(object->oldx >= iposx + iw && direction == 3)
		return hitright(object);

	return true;
}

bool B_FlipBlock::hittop(IO_MovingObject * object)
{
	if(state == 0)
	{
		object->yi(iposy - object->collisionHeight - 1);
		object->oldy = object->iy;
		object->vely = object->bounce;
	}

	return true;
}

bool B_FlipBlock::hitbottom(IO_MovingObject * object)
{
	if(state == 0)
	{
		object->yi(iposy + ih + 1);
		object->vely = -object->vely;
	}
	
	return true;
}

bool B_FlipBlock::hitright(IO_MovingObject * object)
{
	if(state == 0)
	{
		object->xi(iposx + iw);
		object->velx = -object->velx;
	}

	return true;
}

bool B_FlipBlock::hitleft(IO_MovingObject * object)
{
	if(state == 0)
	{
		object->xi(iposx - object->collisionWidth - 1);
		object->velx = -object->velx;
	}

	return true;
}

//------------------------------------------------------------------------------
// class bounce block
//------------------------------------------------------------------------------
B_BounceBlock::B_BounceBlock(gfxSprite *nspr1, int x, int y) :
	IO_Block(nspr1, x, y)
{}

void B_BounceBlock::update()
{
	if(state > 0)
	{
		yf(ny + vely);

		if(state == 1 && fabsf(fposy - ny) > 10.0f)
		{
			bumpPlayer = NULL;
			vely = -vely;
			state = 2;
		}
		else if(state == 2 && fabsf(fposy - ny) < VELBLOCKBOUNCE)
		{
			vely = 0.0f;
			state = 0;
			yf(fposy);
		}
	}
}

bool B_BounceBlock::hittop(CPlayer * player, bool useBehavior)
{
	if(useBehavior)
	{
		player->yi(iposy - PH - 1);
		player->inair = false;
		player->fallthrough = false;
		player->killsinrowinair = 0;
	}

	if(state == 1)
		if(bumpPlayer && !player->invincible && !player->spawninvincible && (player->teamID != bumpPlayer->teamID || game_values.friendlyfire))
			killplayer(*bumpPlayer, *player, death_style_jump, kill_style_bounce);
		else
			player->vely = -VELNOTEBLOCKREPEL;
	else if(useBehavior)
		player->vely = 1.0f;

	return false;
}

bool B_BounceBlock::hitbottom(CPlayer * player, bool useBehavior)
{
	//Player bounces off 
	if(useBehavior)
	{
		player->vely = CapVelocity(-player->vely * BOUNCESTRENGTH);
		player->yi(iposy + ih + 1);

		ifsoundonplay(bump);

		if(state == 0)
		{
			bumpPlayer = player;
			vely = -VELBLOCKBOUNCE;
			state = 1;
		}
	}

	return false;
}

bool B_BounceBlock::hittop(IO_MovingObject * object)
{
	object->yi(iposy - object->collisionHeight - 1);
	object->oldy = object->iy;
	
	if(state == 1 && object->bounce == 1.0f)
	{
		if(object->getMovingObjectType() == movingobject_goomba)
		{
			object->dead = true;
			ifsoundonplay(kicksound);
			eyecandyfront.add(new EC_FallingObject(&spr_goombadeadflying, object->ix, object->iy, -VELJUMP / 2.0f));
	
			if(!game_values.gamemode->gameover && bumpPlayer)
			{
				addawardkill(bumpPlayer, NULL, kill_style_goomba);
				bumpPlayer->score->score++;
			}
		}
		else
		{
			object->vely = -VELNOTEBLOCKREPEL;
		}

		return false;
	}
	else
	{
		object->vely = object->bounce;
	}

	return true;
}


//------------------------------------------------------------------------------
// class MovingObject (all moving objects inheirit from this class)
//------------------------------------------------------------------------------
IO_MovingObject::IO_MovingObject(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	CInteractionObject(nspr, x, y)
{
	iNumSprites = iNumSpr;

	iw = spr->getWidth() / iNumSprites;
	drawframe = 0;
	animationtimer = 0;

	oldx = ix;
	oldy = iy - ih;

	animationspeed = aniSpeed;
	
	objectType = object_moving;
	movingObjectType = movingobject_none;

	collisionWidth = iCollisionWidth;
	collisionHeight = iCollisionHeight;
	collisionOffsetX = iCollisionOffsetX;
	collisionOffsetY = iCollisionOffsetY;
}

IO_MovingObject::IO_MovingObject(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed) :
	CInteractionObject(nspr, x, y)
{
	iNumSprites = iNumSpr;

	iw = spr->getWidth() / iNumSprites;
	collisionWidth = iw;
	drawframe = 0;
	animationtimer = 0;

	oldx = ix;
	oldy = iy - ih;

	animationspeed = aniSpeed;

	objectType = object_moving;
	movingObjectType = movingobject_none;
}

void IO_MovingObject::draw()
{
	spr->draw(ix - collisionOffsetX, iy - collisionOffsetY, drawframe * iw, 0, iw, ih);

	if(ix < 0)
		spr->draw(ix - collisionOffsetX + 640, iy - collisionOffsetY, drawframe * iw, 0, iw, ih);
}


void IO_MovingObject::update()
{
	oldx = ix;
	oldy = iy;

	collision_detection_map();
	
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}
}

void IO_MovingObject::collide(CPlayer * player)
{
	dead = true;
}

void IO_MovingObject::collision_detection_map()
{
	//Could be optimized with bit shift >> 5
	int ty = (int)ny / TILESIZE;
	int ty2 = ((int)ny + collisionHeight) / TILESIZE;
	
	int tx = -1;
	int txl = -1, txr = -1;

	xf(nx + velx);
	float nyvely = ny + vely;  //Fixes weird float rounding error.  Must be computed here before casting to int.  Otherwise, this will miss the bottom collision, but then hit the side collision and the player can slide out of 1x1 spaces.

	flipsidesifneeded();

	if(nyvely + collisionHeight < 0.0f)
	{				// on top outside of the screen
		yf(nyvely);
		vely = CapVelocity(GRAVITATION + vely);
		inair = true;
		return;
	}
	else if(nyvely + collisionHeight >= 480.0f)
	{	//on ground outside of the screen?
		yi(-collisionHeight - 1);
		oldy = -collisionHeight - 2;
		return;
	}

	//-----------------------------------------------------------------
	//  x axis first (--)
	//-----------------------------------------------------------------
	if(ny + collisionHeight >= 0.0f)
	{
		if(velx > 0)
		{		//moving right	
			tx = (ix + collisionWidth) / TILESIZE;	//no special case here when on both sides

			IO_Block * topblock = map.block(tx, ty);
			IO_Block * bottomblock = map.block(tx, ty2);

			if((topblock != NULL && !topblock->isTransparent()) || 
				(bottomblock != NULL && !bottomblock->isTransparent()))
			{
				bool processOtherBlock = true;
				if(topblock != NULL && !topblock->isTransparent()) //collide with top block
				{	
					topblock->collide(this, 1);
					flipsidesifneeded();
					removeifprojectile(this, true);

					processOtherBlock = false;
				}
				
				if(processOtherBlock && bottomblock != NULL && !bottomblock->isTransparent()) //then bottom
				{	
					bottomblock->collide(this, 1);
					flipsidesifneeded();
					removeifprojectile(this, true);
				}
			}
			else if(map.map(tx, ty) == tile_solid || map.map(tx, ty) == tile_death_on_top || 
				map.map(tx, ty) == tile_death_on_bottom || map.map(tx, ty) == tile_ice ||
				map.map(tx, ty2) == tile_solid || map.map(tx, ty2) == tile_death_on_top || 
				map.map(tx, ty2) == tile_death_on_bottom || map.map(tx, ty2) == tile_ice)
			{	//collision on the right side.

				xi(tx * TILESIZE - collisionWidth - 1);			//move to the edge of the tile (tile on the right -> mind the player width)
				oldx = ix;
				velx = -velx;

				flipsidesifneeded();
				removeifprojectile(this, true);
			}
		}
		else if(velx < 0)
		{	//moving left
			if(ix < 0)
			{
				tx = (640+ix)/TILESIZE;	//test against tiles on the right side of the sceen. x is negativ!
				oldx += 640;
			}
			else
				tx = ix/TILESIZE;

			IO_Block * topblock = map.block(tx, ty);
			IO_Block * bottomblock = map.block(tx, ty2);

			if((topblock != NULL && !topblock->isTransparent()) || 
				(bottomblock != NULL && !bottomblock->isTransparent()))
			{
				bool processOtherBlock = true;
				if(topblock != NULL && !topblock->isTransparent()) //collide with top block
				{	
					topblock->collide(this, 3);
					flipsidesifneeded();
					removeifprojectile(this, true);

					processOtherBlock = false;
				}
				
				if(processOtherBlock && bottomblock != NULL && !bottomblock->isTransparent()) //then bottom
				{	
					bottomblock->collide(this, 3);
					flipsidesifneeded();
					removeifprojectile(this, true);
				}
			}
			else if(map.map(tx, ty) == tile_solid || map.map(tx, ty) == tile_death_on_top || 
				map.map(tx, ty) == tile_death_on_bottom || map.map(tx, ty) == tile_ice ||
				map.map(tx, ty2) == tile_solid || map.map(tx, ty2) == tile_death_on_top || 
				map.map(tx, ty2) == tile_death_on_bottom || map.map(tx, ty2) == tile_ice)
			{

				xi(tx * TILESIZE + TILESIZE);			//move to the edge of the tile
				oldx = ix;
				velx = -velx;

				flipsidesifneeded();
				removeifprojectile(this, true);
			}
		}
	}

	if(ix < 0)
		txl = (640 + ix) / TILESIZE;
	else
		txl = ix / TILESIZE;

	txr = (ix + collisionWidth) / TILESIZE;

	//-----------------------------------------------------------------
	//  then y axis (|)
	//-----------------------------------------------------------------

	if(vely < 0)
	{	//moving up

		ty = (int)(nyvely) / TILESIZE;

		IO_Block * leftblock = map.block(txl, ty);
		IO_Block * rightblock = map.block(txr, ty);
		
		if(leftblock != NULL && !leftblock->isTransparent()) //then left
		{	
			leftblock->collide(this, 0);
			return;
		}
		
		if(rightblock != NULL && !rightblock->isTransparent()) //then right
		{	
			rightblock->collide(this, 0);
			return;
		}

		if((map.map(txl, ty) & 0x05) > 0 || (map.map(txr, ty) & 0x05) > 0)
		{
			yi(ty * TILESIZE + TILESIZE + 1);
			vely = -vely;	
		}
		else
		{
			yf(nyvely);
			vely = CapVelocity(GRAVITATION + vely);
		}

		inair = true;
	}		 
	else
	{	//moving down / on ground
		ty = ((int)nyvely + collisionHeight) / TILESIZE;
		
		IO_Block * leftblock = map.block(txl, ty);
		IO_Block * rightblock = map.block(txr, ty);

		if((leftblock != NULL && !leftblock->isTransparent()) || 
			(rightblock != NULL && !rightblock->isTransparent()))
		{
			bool processOtherBlock = true;
			if(leftblock != NULL && !leftblock->isTransparent()) //collide with left block
			{	
				processOtherBlock = leftblock->collide(this, 2);
			}
			
			if(processOtherBlock && rightblock != NULL && !rightblock->isTransparent()) //then right
			{	
				rightblock->collide(this, 2);
					return;
			}

			return;
		}

		if(map.map(txl, ty) == tile_solid_on_top || map.map(txr, ty) == tile_solid_on_top)
		{
			if((oldy + collisionHeight) / TILESIZE < ty)
			{
				vely = bounce;
				yi(ty * TILESIZE - collisionHeight - 1);
				oldy = iy;
				inair = false;

				return;
			}
		}

		if((map.map(txl, ty) & 0x05) > 0 || (map.map(txr, ty) & 0x05) > 0)
		{	
			vely = bounce;
			yi(ty * TILESIZE - collisionHeight - 1);
			oldy = iy;
			inair = false;
		}
		else
		{
			yf(nyvely);
			vely = CapVelocity(GRAVITATION + vely);
			inair = true;
		}
	}
}

void IO_MovingObject::flipsidesifneeded()
{
	if(ix < -collisionWidth)
	{		//out of -on-both-sides- area
	   xf(nx + 640.0f); //x is negativ!
	   oldx = 640 + oldx;
	}
	else if(ix + collisionWidth > 639)
	{		//entered -on-both-sides- area (mind: the screen ends at 639 !!!)
	   xf(nx - 640.0f);
	   oldx = oldx - 640;
	}
}

//------------------------------------------------------------------------------
// class powerup
//------------------------------------------------------------------------------
MO_Powerup::MO_Powerup(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed)  :
	IO_MovingObject(nspr, x, y, iNumSpr, aniSpeed)
{
	desty = ny - ih;
	movingObjectType = movingobject_powerup;
}

MO_Powerup::MO_Powerup(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	IO_MovingObject(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	desty = ny - collisionHeight;
	movingObjectType = movingobject_powerup;
}

void MO_Powerup::draw()
{
	if(state == 0)
		spr->draw(ix - collisionOffsetX, iy - collisionOffsetY, drawframe * iw, 0, iw, (int)(ih - ny + desty));
	else
	{
		spr->draw(ix - collisionOffsetX, iy - collisionOffsetY, drawframe * iw, 0, iw, ih);

		if(ix < 0)	//if the powerup is in the both-sides-of-the-screen
			spr->draw(ix + 640 - collisionOffsetX, iy - collisionOffsetY, drawframe * iw, 0, iw, ih);
	}
}


void MO_Powerup::update()
{
	//Have the powerup grow out of the powerup block
	if(state == 0)
	{
		yf(ny - 2.0f);

		if(ny <= desty)
		{
			state = 1;
			vely = 1.0f;
		}
	}
	else //Then have it obey the physics
	{
		oldx = ix;
		oldy = iy;

		collision_detection_map();
	}
	
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}

}

void MO_Powerup::collide(CPlayer * player)
{
	dead = true;
}

//------------------------------------------------------------------------------
// class star powerup
//------------------------------------------------------------------------------
PU_StarPowerup::PU_StarPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	MO_Powerup(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	if(moveToRight)
		velx = 2.0f;
	else
		velx = -2.0f;

	bounce = -VELPOWERUPBOUNCE;
}

void PU_StarPowerup::collide(CPlayer * player)
{
	dead = true;

	if(!player->invincible)
	{
		player->makeinvincible();
	}
	else
	{
		ifsoundonplay(storepowerup);
		game_values.gamepowerups[player->globalID] = 2;
	}
}

//------------------------------------------------------------------------------
// class 1up powerup
//------------------------------------------------------------------------------
PU_1upPowerup::PU_1upPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	MO_Powerup(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	if(moveToRight)
		velx = 2.0f;
	else
		velx = -2.0f;

	bounce = 1.0f;
}

void PU_1upPowerup::collide(CPlayer * player)
{
	game_values.gamemode->player1up(*player);
	ifsoundonplay(extraguysound);

	dead = true;
}

//------------------------------------------------------------------------------
// class poison powerup
//------------------------------------------------------------------------------
PU_PoisonPowerup::PU_PoisonPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	MO_Powerup(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	if(moveToRight)
		velx = 2.0f;
	else
		velx = -2.0f;

	bounce = 1.0f;
	movingObjectType = movingobject_poisonpowerup;
}

void PU_PoisonPowerup::collide(CPlayer * player)
{
	if(!player->invincible && !player->spawninvincible)
	{
		if( !game_values.gamemode->playerkilledself(*player) )
		{
			player->deathAwards();
			player->jumpDeath();
		}

		ifsoundonplay(deathsound);

		dead = true;
	}

	if(player->invincible)
		dead = true;
}

//------------------------------------------------------------------------------
// class fire powerup
//------------------------------------------------------------------------------
PU_FirePowerup::PU_FirePowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	MO_Powerup(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	velx = 0.0f;
	bounce = 1.0f;
}

void PU_FirePowerup::collide(CPlayer * player)
{
	ifsoundonplay(collectpowerup);
	dead = true;

	if(player->powerup == 1)
		game_values.gamepowerups[player->globalID] = 1;
	else if(player->powerup == 2)
		game_values.gamepowerups[player->globalID] = 7;
	
	player->powerup = 1;
}

//------------------------------------------------------------------------------
// class hammer powerup
//------------------------------------------------------------------------------
PU_HammerPowerup::PU_HammerPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	MO_Powerup(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	velx = 0.0f;
	bounce = 1.0f;
}

void PU_HammerPowerup::collide(CPlayer * player)
{
	dead = true;
	ifsoundonplay(collectpowerup);
	
	if(player->powerup == 1)
		game_values.gamepowerups[player->globalID] = 1;
	else if(player->powerup == 2)
		game_values.gamepowerups[player->globalID] = 7;
	
	player->powerup = 2;
}

//------------------------------------------------------------------------------
// class clock powerup
//------------------------------------------------------------------------------
PU_ClockPowerup::PU_ClockPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	MO_Powerup(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	velx = 0.0f;
	bounce = 1.0f;
}

void PU_ClockPowerup::collide(CPlayer * player)
{
	dead = true;
	ifsoundonplay(storepowerup);
	game_values.gamepowerups[player->globalID] = 3;
}

//------------------------------------------------------------------------------
// class bobomb powerup
//------------------------------------------------------------------------------
PU_BobombPowerup::PU_BobombPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	MO_Powerup(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	velx = 0.0f;
	bounce = 1.0f;
}

void PU_BobombPowerup::collide(CPlayer * player)
{
	dead = true;

	if(player->bobomb)
	{
		ifsoundonplay(storepowerup);
		game_values.gamepowerups[player->globalID] = 4;
	}
	else
	{
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, ix + (HALFPW) - 16, iy + (HALFPH) - 16, 3, 8));
		ifsoundonplay(transform);
		player->bobomb = true;
	}
}

//------------------------------------------------------------------------------
// class POW powerup
//------------------------------------------------------------------------------
PU_PowPowerup::PU_PowPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	MO_Powerup(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	velx = 0.0f;
	bounce = 1.0f;
}

void PU_PowPowerup::collide(CPlayer * player)
{
	dead = true;
	ifsoundonplay(storepowerup);
	game_values.gamepowerups[player->globalID] = 5;
}

//------------------------------------------------------------------------------
// class BulletBill powerup
//------------------------------------------------------------------------------
PU_BulletBillPowerup::PU_BulletBillPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	MO_Powerup(nspr, x, y, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	velx = 0.0f;
	bounce = 1.0f;
}

void PU_BulletBillPowerup::collide(CPlayer * player)
{
	dead = true;
	ifsoundonplay(storepowerup);
	game_values.gamepowerups[player->globalID] = 6;
}

//------------------------------------------------------------------------------
// class fireball
//------------------------------------------------------------------------------
MO_Fireball::MO_Fireball(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int id, int colorID) :
	IO_MovingObject(nspr, x, y, iNumSpr, aniSpeed, nspr->getWidth() >> 2, nspr->getHeight() >> 3, 0, 0)
{
	if(moveToRight)
		velx = 5.0f;
	else
		velx = -5.0f;

	//fireball sprites have both right and left sprites in them
	ih = ih >> 3;

	bounce = -FIREBALLBOUNCE;

	playerID = id;
	colorOffset = colorID * 36;
	movingObjectType = movingobject_fireball;
	state = 1;
	direction = moveToRight;

	ttl = 300;
}

void MO_Fireball::update()
{
	IO_MovingObject::update();

	if(--ttl <= 0)
	{
		removeifprojectile(this, true);
	}
}

void MO_Fireball::collide(CPlayer * player)
{
	int teamID = lookupteamid(playerID);

	if(playerID != player->globalID && (game_values.friendlyfire || teamID != player->teamID))
	{
		if(!player->spawninvincible)
			removeifprojectile(this, false);

		if(!player->invincible && !player->spawninvincible)
		{
			//Find the player that shot this fireball so we can attribute a kill
			CPlayer * killer = NULL;
			for(int i = 0; i < list_players_cnt; i++)
			{
				if(list_players[i]->globalID == playerID)
				{
					killer = list_players[i];
					break;
				}
			}

			if(killer)
			{
				killplayer(*killer, *player, death_style_jump, kill_style_fireball);
			}
			else
			{
				if( ! game_values.gamemode->playerkilledself(*player) )
				{
					player->deathAwards();
					player->jumpDeath();
				}
			}

			ifsoundonplay(deathsound);
		}
	}
}

void MO_Fireball::draw()
{
	spr->draw(ix - collisionOffsetX, iy - collisionOffsetY, drawframe * iw, (direction ? 0 : 18) + colorOffset, iw, ih);

	if(ix < 0)
		spr->draw(ix - collisionOffsetX + 640, iy - collisionOffsetY, drawframe * iw, (direction ? 0 : 18) + colorOffset, iw, ih);
}


//------------------------------------------------------------------------------
// class fireball
//------------------------------------------------------------------------------
OMO_Hammer::OMO_Hammer(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int id, int colorID) :
	IO_MovingObject(nspr, x, y, iNumSpr, aniSpeed, nspr->getWidth() / iNumSpr, nspr->getHeight() >> 2, 0, 0)
{
	if(moveToRight)
		velx = 3.0f;
	else
		velx = -3.0f;

	//fireball sprites have both right and left sprites in them
	ih = ih >> 2;

	playerID = id;
	colorOffset = colorID * 28;
	movingObjectType = movingobject_hammer;
	state = 1;
	direction = moveToRight;
	vely = -HAMMERTHROW;
	ttl = 50;
}

void OMO_Hammer::update()
{
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		
		if(velx > 0 && ++drawframe >= iNumSprites)
			drawframe = 0;
		else if(velx < 0 && --drawframe < 0)
			drawframe = iNumSprites - 1;
	}

	xf(nx + velx);
	yf(ny + vely);
	vely = CapVelocity(GRAVITATION + vely);

	if(ix < -iw)
		xi(ix + 640);
	else if(ix + iw > 639)
		xi(ix - 640);

	if(iy > 480 || --ttl <= 0)
		removeifprojectile(this, false);
}

void OMO_Hammer::collide(CPlayer * player)
{
	int teamID = lookupteamid(playerID);

	if(playerID != player->globalID && (game_values.friendlyfire || teamID != player->teamID))
	{
		if(!player->spawninvincible)
			removeifprojectile(this, false);

		if(!player->invincible && !player->spawninvincible)
		{
			//Find the player that shot this hammer so we can attribute a kill
			CPlayer * killer = NULL;
			for(int i = 0; i < list_players_cnt; i++)
			{
				if(list_players[i]->globalID == playerID)
				{
					killer = list_players[i];
					break;
				}
			}

			if(killer)
			{
				killplayer(*killer, *player, death_style_jump, kill_style_hammer);
			}
			else
			{
				if( ! game_values.gamemode->playerkilledself(*player) )
				{
					player->deathAwards();
					player->jumpDeath();
				}
			}

			ifsoundonplay(deathsound);
		}
	}
}

void OMO_Hammer::draw()
{
	spr->draw(ix - collisionOffsetX, iy - collisionOffsetY, drawframe * iw, colorOffset, iw, ih);

	if(ix < 0)
		spr->draw(ix - collisionOffsetX + 640, iy - collisionOffsetY, drawframe * iw, colorOffset, iw, ih);
}

//------------------------------------------------------------------------------
// class coin (for coin mode)
//------------------------------------------------------------------------------
PU_Coin::PU_Coin(gfxSprite *nspr, int iNumSpr, int aniSpeed) :
	MO_Powerup(nspr, 0, 0, iNumSpr, aniSpeed)
{
	state = 1;
	placeCoin();
}

void PU_Coin::collide(CPlayer * player)
{
	if(!game_values.gamemode->gameover)
		player->score->score++;

	ifsoundonplay(coin);
	placeCoin();
}

void PU_Coin::update()
{
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}

	if(++timer > 1500)
		placeCoin();
}

void PU_Coin::placeCoin()
{
	timer = 0;
	map.findspawnpoint(&ix, &iy, collisionWidth, collisionHeight, false);
}

//------------------------------------------------------------------------------
// class OverMapObject - moving objects that don't collide with map or objects, just player
//------------------------------------------------------------------------------
IO_OverMapObject::IO_OverMapObject(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed) :
	CInteractionObject(nspr, x, y)
{
	iNumSprites = iNumSpr;

	iw = spr->getWidth() / iNumSprites;
	collisionWidth = iw;
	drawframe = 0;

	animationspeed = aniSpeed;
	animationtimer = 0;

	objectType = object_overmap;
	movingObjectType = movingobject_none;
}


void IO_OverMapObject::draw()
{
	spr->draw(ix, iy, drawframe * iw, 0, iw, ih);

	if((ix + iw) > 639) //draw on both sides of the screen if needed
		spr->draw(0-(iw-(ix+iw)%640), iy, drawframe * iw, 0, iw, ih);
	else if(ix < 0)
		spr->draw(ix + 640, iy, drawframe * iw, 0, iw, ih);
}


void IO_OverMapObject::update()
{
	xf(nx + velx);
	yf(ny + vely);

	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}
}


//------------------------------------------------------------------------------
// class thwomp (for thwomp mode)
//------------------------------------------------------------------------------
OMO_Thwomp::OMO_Thwomp(gfxSprite *nspr, int x, float nspeed) :
	IO_OverMapObject(nspr, x, -nspr->getHeight(), 1, 1000000)
{
	vely = nspeed;
}

void OMO_Thwomp::update()
{
	IO_OverMapObject::update();

	if(iy > 479)
		dead = true;
}

void OMO_Thwomp::collide(CPlayer * player)
{
	if(!player->invincible && !player->spawninvincible && player->score->score > 0)
	{
		if( !game_values.gamemode->playerkilledself(*player) )
		{
			player->deathAwards();
			player->jumpDeath();
		}

		ifsoundonplay(deathsound);
	}
}


//------------------------------------------------------------------------------
// class thwomp (for thwomp mode)
//------------------------------------------------------------------------------
OMO_BulletBill::OMO_BulletBill(gfxSprite *nspr, int y, float nspeed) :
	IO_MovingObject(nspr, 0, y, 3, 8, 30, 28, 1, 2)
{
	velx = nspeed;
	ih = nspr->getHeight() >> 1;
	collisionHeight = ih;
	
	if(velx < 0)
		xi(640 + iw);
	else
		xi(-iw);

	movingObjectType = movingobject_bulletbill;
	state = 1;
}

void OMO_BulletBill::update()
{
	xf(nx + velx);
	yf(ny + vely);

	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}

	if((velx < 0 && ix < -iw) || (velx > 0 && ix > 640))
		dead = true;
}

void OMO_BulletBill::draw()
{
	spr->draw(ix, iy, drawframe * iw, (velx < 0 ? 0 : 32), iw, ih);
}

void OMO_BulletBill::collide(CPlayer * player)
{
	if(player->invincible)
	{
		dead = true;
		addawardkill(player, NULL, kill_style_bulletbill);
		ifsoundonplay(kicksound);
		
		if(velx < 0)
			eyecandyfront.add(new EC_FallingObject(&spr_bulletbilldeadflyingleft, ix, iy, -VELJUMP / 2.0f));
		else
			eyecandyfront.add(new EC_FallingObject(&spr_bulletbilldeadflyingright, ix, iy, -VELJUMP / 2.0f));
	}
	else if(!player->spawninvincible)
	{
		int flipx = 0;

		if(player->ix + PW < 320 && ix > 320)
			flipx = 640;
		else if(ix + iw < 320 && player->ix > 320)
			flipx = -640;

		if(player->oldy+PH <= iy && player->iy+PH >= iy)
			hittop(player);
		else if(player->oldy >= iy + ih && player->iy <= iy + ih)
			hitother(player);
		else if(player->oldx + flipx + PW <= ix && player->ix + flipx + PW >= ix)
			hitother(player);
		else if(player->oldx + flipx >= ix + iw && player->ix + flipx <= ix + iw)
			hitother(player);
		else if(player->ix + flipx + PW >= ix && player->ix + flipx <= ix)
			hitother(player);
		else if(player->ix + flipx <= ix + iw && player->ix + flipx + PW >= ix + iw)
			hitother(player);
		else if(player->iy <= iy + ih && player->iy+PH >= iy + ih)
			hitother(player);
		else if(player->iy+PH >= iy && player->iy <= iy)
			hittop(player);
	}
}


void OMO_BulletBill::hittop(CPlayer * player)
{
	player->yi(iy - PH - 1);
	player->bouncejump();
	player->collision_detection_checktop();
	dead = true;

	addawardkill(player, NULL, kill_style_bulletbill);

	ifsoundonplay(mip);
	
	if(velx < 0)
		eyecandyfront.add(new EC_FallingObject(&spr_bulletbilldeadflyingleft, ix, iy, -VELJUMP / 2.0f));
	else
		eyecandyfront.add(new EC_FallingObject(&spr_bulletbilldeadflyingright, ix, iy, -VELJUMP / 2.0f));
}

void OMO_BulletBill::hitother(CPlayer * player)
{
	if(!game_values.gamemode->playerkilledself(*player) )
	{
		player->deathAwards();
		player->jumpDeath();
	}

	ifsoundonplay(deathsound);
}

void OMO_BulletBill::collide(IO_MovingObject * object)
{
	removeifprojectile(object, true);
}

//------------------------------------------------------------------------------
// class egg (for egg mode)
//------------------------------------------------------------------------------
OMO_Egg::OMO_Egg(gfxSprite *nspr) :
	IO_OverMapObject(nspr, 0, 0, 2, 8)
{
	state = 1;
	owner = NULL;
	objectType = object_egg;

	placeEgg();
}

void OMO_Egg::collide(CPlayer * player)
{
	if(owner == NULL)
	{
		owner = player;
		player->ownedEgg = this;
	}
}

void OMO_Egg::update()
{
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}

	if(owner)
	{
		xi(owner->ix + HALFPW - 16);
		yi(owner->iy + HALFPH - 22);
		timer = 0;
	}
	else if(++timer > 1500)
	{
		placeEgg();
	}
}

void OMO_Egg::draw()
{
	if(owner && owner->iswarping())
	{
		int warpOffsetX = owner->warpOffsetX;
		int warpOffsetY = owner->warpOffsetY + (owner->state == player_entering_warp_up || owner->state == player_exiting_warp_down ? 6 : 0);
		int warpWidthOffsetLeading = owner->warpWidthOffsetLeading;
		int warpHeightOffsetLeading = owner->warpHeightOffsetLeading + (owner->state == player_entering_warp_up || owner->state == player_exiting_warp_down ? 6 : 0);
		int warpWidthOffsetTrailing = owner->warpWidthOffsetTrailing;
		int warpHeightOffsetTrailing = owner->warpHeightOffsetTrailing + (owner->state == player_entering_warp_down || owner->state == player_exiting_warp_up ? 6 : 0);

		spr->draw(ix + warpOffsetX, iy + warpOffsetY, drawframe * iw + warpWidthOffsetLeading, warpHeightOffsetLeading, iw + warpWidthOffsetTrailing, ih + warpHeightOffsetTrailing);

		if((ix + iw) > 639) //draw on both sides of the screen if needed
			spr->draw(ix + warpOffsetX - 640, iy + warpOffsetY, drawframe * iw + warpWidthOffsetLeading, warpHeightOffsetLeading, iw + warpWidthOffsetTrailing, ih + warpHeightOffsetTrailing);
		else if(ix < 0)
			spr->draw(ix + warpOffsetX + 640, iy + warpOffsetY, drawframe * iw + warpWidthOffsetLeading, warpHeightOffsetLeading, iw + warpWidthOffsetTrailing, ih + warpHeightOffsetTrailing);
	}
	else
	{
		spr->draw(ix, iy, drawframe * iw, 0, iw, ih);

		if((ix + iw) > 639) //draw on both sides of the screen if needed
			spr->draw(ix - 640, iy, drawframe * iw, 0, iw, ih);
		else if(ix < 0)
			spr->draw(ix + 640, iy, drawframe * iw, 0, iw, ih);
	}
}

void OMO_Egg::placeEgg()
{
	timer = 0;

	int tries = 0;
	do
	{
		if(++tries > 32)
			break;

		map.findspawnpoint(&ix, &iy, collisionWidth, collisionHeight, false);
	}
	while(interactionobjectsplayer.getClosestObject(ix, iy, object_yoshi) < 250.0f);
}


//------------------------------------------------------------------------------
// class yoshi (for egg mode)
//------------------------------------------------------------------------------
OMO_Yoshi::OMO_Yoshi(gfxSprite *nspr) :
	IO_OverMapObject(nspr, 0, 0, 2, 8)
{
	state = 1;
	placeYoshi();
	objectType = object_yoshi;
}

void OMO_Yoshi::collide(CPlayer * player)
{
	if(player->ownedEgg != NULL)
	{
		if(!game_values.gamemode->gameover)
			player->score->score++;
	
		placeYoshi();

		player->ownedEgg->placeEgg();
		player->ownedEgg->owner = NULL;
		player->ownedEgg = NULL;
		
		ifsoundonplay(yoshi);
	}
}

void OMO_Yoshi::update()
{
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}

	if(++timer > 2000)
		placeYoshi();
}

void OMO_Yoshi::placeYoshi()
{
	timer = 0;

	for(int tries = 0; tries < 64; tries++)
	{
		ix = (rand()%(640 - iw));
		iy = (rand()%(480 - ih - TILESIZE));	//don't spawn too low

		int ixl = ix / TILESIZE;
		int ixr = (ix + iw) / TILESIZE;
		int iyt = iy / TILESIZE;
		int iyb = (iy + ih) / TILESIZE;

		if((map.map(ixl, iyt) == tile_nonsolid || map.map(ixl, iyt) == tile_solid_on_top) &&
		   (map.map(ixr, iyt) == tile_nonsolid || map.map(ixr, iyt) == tile_solid_on_top) &&
		   (map.map(ixl, iyb) == tile_nonsolid || map.map(ixl, iyb) == tile_solid_on_top) &&
		   (map.map(ixr, iyb) == tile_nonsolid || map.map(ixr, iyb) == tile_solid_on_top) &&
			!map.block(ixl, iyt) && !map.block(ixr, iyt) && !map.block(ixl, iyb) && !map.block(ixr, iyb))
		{
			//spawn on ground, but not on spikes
			int iDeathY = (iy+ih)/TILESIZE;
			int iDeathX1 = ix/TILESIZE;
			int iDeathX2 = (ix+iw)/TILESIZE;
			
			while(iDeathY < MAPHEIGHT)
			{
				TileType ttLeftTile = map.map(iDeathX1, iDeathY);
				TileType ttRightTile = map.map(iDeathX2, iDeathY);

				if(ttLeftTile == tile_solid || ttLeftTile == tile_solid_on_top || ttLeftTile == tile_death_on_bottom || ttLeftTile == tile_ice ||
					ttRightTile == tile_solid || ttRightTile == tile_solid_on_top || ttRightTile == tile_death_on_bottom || ttRightTile == tile_ice ||
					map.block(iDeathX1, iDeathY) || map.block(iDeathX2, iDeathY))
				{
					int top = (iDeathY * TILESIZE - ih) / TILESIZE;
					
					if(map.spawn(iDeathX1, top) && map.spawn(iDeathX2, top) && 
						map.spawn(iDeathX1, iDeathY - 1) && map.spawn(iDeathX2, iDeathY - 1))
					{
						xi(ix);
						yi(iDeathY * TILESIZE - ih);
						return;
					}

					break;
				}
				else if(ttLeftTile == tile_death_on_top || ttRightTile == tile_death_on_top ||
					ttLeftTile == tile_death || ttRightTile == tile_death)
				{
					break;
				}

				iDeathY++;
			}
		}
	}

	ix = 320;
	iy = 240;
}

//------------------------------------------------------------------------------
// class area (for Domination mode)
//------------------------------------------------------------------------------
OMO_Area::OMO_Area(gfxSprite *nspr, int numAreas) :
	IO_OverMapObject(nspr, 0, 0, 5, 1000000)
{
	iw = spr->getWidth() /5;
	collisionWidth = iw;

	objectType = object_area;
	playerID = -1;
	colorID = -1;
	scoretimer = 0;
	state = 1;
	
	placeArea(numAreas);
}

void OMO_Area::collide(CPlayer * player)
{
	if(colorID != player->colorID)
	{
		colorID = player->colorID;
		playerID = player->localID;
		ifsoundonplay(areatag);
	}
}

void OMO_Area::draw()
{
	spr->draw(ix, iy, (colorID + 1) * iw, 0, iw, ih);
}

void OMO_Area::update()
{
	if(playerID != -1 && !game_values.gamemode->gameover)
	{
		if(++scoretimer == 20)
		{
			scoretimer = 0;
			list_players[playerID]->score->score++;
		}
	}
}

void OMO_Area::placeArea(int numAreas)
{
	int tries = 0;
	do
	{
		if(++tries > 32)
			break;

		map.findspawnpoint(&ix, &iy, collisionWidth, collisionHeight, true);
	}
	while(interactionobjectcollisionitems.getClosestObject(ix, iy, object_area) <= (200.0f - ((numAreas - 3) * 25.0f)));
}

void OMO_Area::reset()
{
	playerID = -1;
	colorID = -1;
	scoretimer = 0;
}


int flagpositions[3][4][2] = { { {18, 20}, {34, 20}, {0, 0}, {0, 0} },
							{ {18, 20}, {34, 20}, {26, 36}, {0, 0} },
							{ {18, 20}, {34, 20}, {18, 36}, {34, 36} } };

//------------------------------------------------------------------------------
// class race goal (for Race mode)
//------------------------------------------------------------------------------
OMO_RaceGoal::OMO_RaceGoal(gfxSprite *nspr, int id) :
	IO_OverMapObject(nspr, 0, 0, 2, 8)
{
	iw = spr->getWidth() >> 1;
	ih = spr->getHeight() >> 1;
	collisionWidth = 36;
	collisionHeight = 36;
	collisionOffsetX = 16;
	collisionOffsetY = 18;

	objectType = object_race_goal;
	state = 1;

	goalID = id;

	for(int k = 0; k < 4; k++)
		tagged[k] = -1;

	angle = (float)(rand()%1000 * 0.00628f);
	anglechange = (float)(rand()%100 * 0.0002f);
	anglechangetimer = rand()%50 + 100;

	velx = sin(angle);
	vely = cos(angle);

	placeRaceGoal();
}

void OMO_RaceGoal::collide(CPlayer * player)
{
	if(game_values.gamemode->getgamemode() == game_mode_race)
	{
		CGM_Race * gamemode = (CGM_Race*)game_values.gamemode;
	
		if(tagged[player->teamID] != player->colorID && gamemode->getNextGoal(player->teamID) >= goalID)
		{
			tagged[player->teamID] = player->colorID;
			
			if(goalID == 3)
                ifsoundonplay(racesound);
			else
				ifsoundonplay(areatag);
		}

		if(gamemode->getNextGoal(player->teamID) == goalID)
			gamemode->setNextGoal(player->teamID);
	}
}

void OMO_RaceGoal::draw()
{
	if(goalID == 3)
	{
		spr->draw(ix - collisionOffsetX, iy - collisionOffsetY, drawframe * iw, 54, iw, ih);	
	}
	else
	{
		spr->draw(ix - collisionOffsetX, iy - collisionOffsetY, drawframe * iw, 0, iw, ih);	

		for(int k = 0; k < score_cnt; k++)
		{
			if(tagged[k] > -1)
				spr_bonus[tagged[k]].draw(ix - collisionOffsetX + flagpositions[score_cnt - 2][k][0], iy - collisionOffsetY + flagpositions[score_cnt - 2][k][1]);
		}
	}

	spr_racegoalnumbers.draw(ix - collisionOffsetX + 24, iy - collisionOffsetY, goalID * 20, 0, 20, 20);
}

void OMO_RaceGoal::update()
{
	if(--anglechangetimer <= 0)
	{
		anglechange = (float)((rand()%101 - 50) * 0.0002f);
		anglechangetimer = rand()%50 + 100;
	}
	
	angle += anglechange;

	velx = sin(angle);
	vely = cos(angle);
	
	IO_OverMapObject::update();

	ix = (int)nx;
	iy = (int)ny;

	if(ix < 0)
	{
		velx = -velx;
		ix = 0;
		nx = (float)ix;

		angle = atan2(velx, vely);
	}
	else if(ix + collisionWidth > 639)
	{
		velx = -velx;
		ix = 639 - collisionWidth;
		nx = (float)ix;

		angle = atan2(velx, vely);
	}
	
	if(iy < 0)
	{
		vely = -vely;
		iy = 0;
		ny = (float)iy;

		angle = atan2(velx, vely);
	}
	else if(iy + collisionHeight > 479)
	{
		vely = -vely;
		iy = 479 - collisionHeight;
		ny = (float)iy;

		angle = atan2(velx, vely);
	}
}

void OMO_RaceGoal::placeRaceGoal()
{
	int tries = 0;
	do
	{
		if(++tries > 32)
			break;

		ix = rand() % (640 - collisionWidth);
		iy = rand() % (480 - collisionHeight);
	}
	while(interactionobjectsfront.getClosestObject(ix, iy, object_race_goal) <= 150.0f);

	nx = (float)ix;
	ny = (float)iy;
}

//------------------------------------------------------------------------------
// class bobombcard (for bob-omb mode)
//------------------------------------------------------------------------------
PU_BobombCard::PU_BobombCard(gfxSprite *nspr, int iNumSpr, int aniSpeed) :
	MO_Powerup(nspr, 0, 0, iNumSpr, aniSpeed)
{
	state = 1;
	objectType = object_bobombcard;

	placeBobombCard();
}

void PU_BobombCard::collide(CPlayer * player)
{
	dead = true;

	if(player->bobomb)
	{
		ifsoundonplay(storepowerup);
		game_values.gamepowerups[player->globalID] = 4;
	}
	else
	{
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, ix + (HALFPW) - 16, iy + (HALFPH) - 16, 3, 8));
		ifsoundonplay(transform);
		player->bobomb = true;
	}
}

void PU_BobombCard::update()
{
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}

	if(++timer > 1500)
		placeBobombCard();

}

void PU_BobombCard::placeBobombCard()
{
	timer = 0;
	map.findspawnpoint(&ix, &iy, collisionWidth, collisionHeight, false);
}

//------------------------------------------------------------------------------
// class fireflowercard (for fire frenzy mode)
//------------------------------------------------------------------------------
PU_FireFlowerCard::PU_FireFlowerCard(gfxSprite *nspr, int iNumSpr, int aniSpeed) :
	MO_Powerup(nspr, 0, 0, iNumSpr, aniSpeed)
{
	state = 1;
	objectType = object_fireflowercard;

	placeFireFlowerCard();
}

void PU_FireFlowerCard::collide(CPlayer * player)
{
	ifsoundonplay(transform);
	dead = true;

	if(player->powerup == 1)
		game_values.gamepowerups[player->globalID] = 1;
	else if(player->powerup == 2)
		game_values.gamepowerups[player->globalID] = 7;
	
	player->powerup = 1;
}

void PU_FireFlowerCard::update()
{
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}

	if(++timer > 1500)
		placeFireFlowerCard();
}

void PU_FireFlowerCard::placeFireFlowerCard()
{
	timer = 0;
	map.findspawnpoint(&ix, &iy, collisionWidth, collisionHeight, false);
}


//------------------------------------------------------------------------------
// class hammercard (for hammer frenzy mode)
//------------------------------------------------------------------------------
PU_HammerCard::PU_HammerCard(gfxSprite *nspr, int iNumSpr, int aniSpeed) :
	MO_Powerup(nspr, 0, 0, iNumSpr, aniSpeed)
{
	state = 1;
	objectType = object_hammercard;

	placeHammerCard();
}

void PU_HammerCard::collide(CPlayer * player)
{
	ifsoundonplay(transform);
	dead = true;

	if(player->powerup == 1)
		game_values.gamepowerups[player->globalID] = 1;
	else if(player->powerup == 2)
		game_values.gamepowerups[player->globalID] = 7;
	
	player->powerup = 2;
}

void PU_HammerCard::update()
{
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}

	if(++timer > 1500)
		placeHammerCard();
}

void PU_HammerCard::placeHammerCard()
{
	timer = 0;
	map.findspawnpoint(&ix, &iy, collisionWidth, collisionHeight, false);
}

//------------------------------------------------------------------------------
// class explosion (for bob-omb mode)
//------------------------------------------------------------------------------
OMO_Explosion::OMO_Explosion(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed, int id) :
	IO_OverMapObject(nspr, x, y, iNumSpr, aniSpeed)
{
	state = 1;
	playerID = id;
	timer = 0;
	objectType = object_explosion;
}

void OMO_Explosion::collide(CPlayer * player)
{
	int teamID = lookupteamid(playerID);
	
	if(player->globalID != playerID && (game_values.friendlyfire || teamID != player->teamID) && !player->invincible && !player->spawninvincible)
	{
		//Find the player that made this explosion so we can attribute a kill
		CPlayer * killer = NULL;
		for(int i = 0; i < list_players_cnt; i++)
		{
			if(list_players[i]->globalID == playerID)
			{
				killer = list_players[i];
				break;
			}
		}

		if(killer)
		{
			killplayer(*killer, *player, death_style_jump, kill_style_bobomb);
		}
		else
		{
			if(!game_values.gamemode->playerkilledself(*player))
			{
				player->deathAwards();
				player->jumpDeath();
			}
		}
	}
}

void OMO_Explosion::update()
{
	if(++animationtimer == animationspeed)
	{
		animationtimer = 0;
		if(++drawframe >= iNumSprites)
			drawframe = 0;
	}

	if(++timer >= 48)
		dead = true;
}

void OMO_Explosion::collide(IO_MovingObject * object)
{
	if(object->getMovingObjectType() == movingobject_goomba || object->getMovingObjectType() == movingobject_bulletbill)
	{
		object->dead = true;
		
		//Attribute the kill
		CPlayer * killer = NULL;
		for(int i = 0; i < list_players_cnt; i++)
		{
			if(list_players[i]->globalID == playerID)
			{
				killer = list_players[i];
				break;
			}
		}

		if(killer)
		{
			if(object->getMovingObjectType() == movingobject_goomba)
				addawardkill(killer, NULL, kill_style_goomba);
			else if(object->getMovingObjectType() == movingobject_bulletbill)
				addawardkill(killer, NULL, kill_style_bulletbill);

			if(game_values.gamemode->gamemode == game_mode_goomba && object->getMovingObjectType() == movingobject_goomba && !game_values.gamemode->gameover)
				killer->score->score++;
		}

		ifsoundonplay(kicksound);

		if(object->getMovingObjectType() == movingobject_goomba)
		{
			eyecandyfront.add(new EC_FallingObject(&spr_goombadeadflying, object->ix, object->iy, -VELJUMP / 2.0f));
		}
		else if(object->getMovingObjectType() == movingobject_bulletbill)
		{
			if(object->velx < 0)
				eyecandyfront.add(new EC_FallingObject(&spr_bulletbilldeadflyingleft, object->ix, object->iy, -VELJUMP / 2.0f));
			else
				eyecandyfront.add(new EC_FallingObject(&spr_bulletbilldeadflyingright, object->ix, object->iy, -VELJUMP / 2.0f));
		}
	}
}

//------------------------------------------------------------------------------
// class goomba
//------------------------------------------------------------------------------
MO_Goomba::MO_Goomba(gfxSprite *nspr, int iNumSpr, int aniSpeed, bool moveToRight, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY) :
	IO_MovingObject(nspr, 0, 0, iNumSpr, aniSpeed, iCollisionWidth, iCollisionHeight, iCollisionOffsetX, iCollisionOffsetY)
{
	if(moveToRight)
		velx = 1.0f;
	else
		velx = -1.0f;

	movingObjectType = movingobject_goomba;
	bounce = 1.0f;
	
	spawnradius = 100.0f;
	spawnangle = (float)(rand()%1000 * 0.00628f);
	inair = true;

	place();
}

void MO_Goomba::draw()
{
	//TODO:  Add spawn swirl and then switch state = 1 when done
	//Then add goomba limit mode
	
	if(state == 0)
	{
		int numeyecandy = 8;
		float addangle = TWO_PI / numeyecandy;
		float displayangle = spawnangle;

		for(int k = 0; k < numeyecandy; k++)
		{
			int spawnX = ix + (collisionWidth >> 1) - 8 + (int)(spawnradius * cos(displayangle));
			int spawnY = iy + (collisionHeight >> 1) - 8 + (int)(spawnradius * sin(displayangle));

			displayangle += addangle;
		
			menu_mode.draw(spawnX, spawnY, 80, 0, 16, 16);
		}
	}
	else
	{
		IO_MovingObject::draw();
	}
}

void MO_Goomba::update()
{
	if(state == 0)
	{
		spawnradius -= 2.0f;
		spawnangle += 0.05f;

		if(spawnradius < 10.0f)
			state = 1;
	}
	else
	{
		IO_MovingObject::update();
	}
}
	

void MO_Goomba::collide(CPlayer * player)
{
	if(player->invincible)
	{
		dead = true;
		
		addawardkill(player, NULL, kill_style_goomba);

		if(game_values.gamemode->gamemode == game_mode_goomba && !game_values.gamemode->gameover)
			player->score->score++;

		ifsoundonplay(kicksound);
		eyecandyfront.add(new EC_FallingObject(&spr_goombadeadflying, ix, iy, -VELJUMP / 2.0f));
	}
	else if(!player->spawninvincible)
	{
		int flipx = 0;

		if(player->ix + PW < 320 && ix > 320)
			flipx = 640;
		else if(ix + iw < 320 && player->ix > 320)
			flipx = -640;

		if(player->oldy + PH <= iy && player->iy + PH >= iy)
			hittop(player);
		else if(player->oldy >= iy + ih && player->iy <= iy + ih)
			hitother(player);
		else if(player->oldx + flipx + PW <= ix && player->ix + flipx + PW >= ix)
			hitother(player);
		else if(player->oldx + flipx >= ix + iw && player->ix + flipx <= ix + iw)
			hitother(player);
		else if(player->iy <= iy + ih && player->iy+PH >= iy + ih)
			hitother(player);
		else if(player->ix + flipx + PW >= ix && player->ix + flipx <= ix)
			hitother(player);
		else if(player->ix + flipx <= ix + iw && player->ix + flipx + PW >= ix + iw)
			hitother(player);
		else if(player->iy+PH >= iy && player->iy <= iy)
			hittop(player);
	}
}

void MO_Goomba::hittop(CPlayer * player)
{
	player->yi(iy - PH - 1);
	player->bouncejump();
	player->collision_detection_checktop();
	dead = true;

	addawardkill(player, NULL, kill_style_goomba);

	if(game_values.gamemode->gamemode == game_mode_goomba && !game_values.gamemode->gameover)
		player->score->score++;

	ifsoundonplay(mip);
	eyecandyback.add(new EC_Corpse(&spr_goombadead, (float)(ix - collisionOffsetX), (float)(iy + collisionHeight - 32)));
}

void MO_Goomba::hitother(CPlayer * player)
{
	if(!game_values.gamemode->playerkilledself(*player) )
	{
		player->deathAwards();
		player->jumpDeath();
	}

	ifsoundonplay(deathsound);
}

void MO_Goomba::collide(IO_MovingObject * object)
{
	if(!object->GetDead())
	{
		removeifprojectile(object, false);

		if(object->getMovingObjectType() == movingobject_fireball || object->getMovingObjectType() == movingobject_hammer)
		{
			//Find the player that shot this fireball so we can attribute a kill
			CPlayer * killer = NULL;
			for(int i = 0; i < list_players_cnt; i++)
			{
				if(list_players[i]->globalID == ((MO_Fireball*)object)->playerID)
				{
					killer = list_players[i];
					break;
				}
			}

			if(killer)
			{
				addawardkill(killer, NULL, kill_style_goomba);

				if(game_values.gamemode->gamemode == game_mode_goomba && !game_values.gamemode->gameover)
					killer->score->score++;
			}

			dead = true;
			ifsoundonplay(kicksound);
			eyecandyfront.add(new EC_FallingObject(&spr_goombadeadflying, ix, iy, -VELJUMP / 2.0f));
		}
	}
}


void MO_Goomba::place()
{
	map.findspawnpoint(&ix, &iy, collisionWidth, collisionHeight, false);
	nx = (float)ix;
	ny = (float)iy;
}

//------------------------------------------------------------------------------
// class interactionobject_container
//------------------------------------------------------------------------------
CInteractionObjectContainer::CInteractionObjectContainer()
{
	for(int i = 0; i < MAXINTERACTIONOBJECTS; i++)
		list[i] = NULL;

	list_end = 0;
}


CInteractionObjectContainer::~CInteractionObjectContainer()
{
	clean();
}


void CInteractionObjectContainer::clean()
{
	for(int i = 0; i < list_end; i++)
	{
		delete list[i];
		list[i] = NULL;
	}
	list_end = 0;
}


void CInteractionObjectContainer::add(CInteractionObject *ec)
{
	if(list_end < MAXINTERACTIONOBJECTS)
	{
		list[list_end] = ec;
		ec->index = list_end;	//save index for removing
		list_end++;
	}
	else
	{
		delete ec;	//otherwise memory leak!
		//printf("eyecandy list full!\n");
	}
}

bool CInteractionObjectContainer::isBlockAt(int x, int y)
{
	for(int i = 0; i < list_end; i++)
	{
		if(x >= list[i]->ix && x < list[i]->ix + list[i]->iw && 
			y >= list[i]->iy && y < list[i]->iy + list[i]->ih && 
			list[i]->getObjectType() == object_block)
		{
			return true;
		}
	}

	return false;
}

float CInteractionObjectContainer::getClosestObject(int ix, int iy, int objectType)
{
	float dist = 800.0f;  //Longest distance from corner to corner

	for(int i = 0; i < list_end; i++)
	{
		if(list[i]->getObjectType() == objectType)
		{
			int x = list[i]->ix - ix;
			int y = list[i]->iy - iy;

			float calcdist = (float)sqrt((double)(x * x + y * y));

			if(calcdist < dist)
				dist = calcdist;
		}
	}

	return dist;
}

bool CInteractionObjectContainer::isBombReady()
{
	for(int i = 0; i < list_end; i++)
	{
		if(list[i]->getObjectType() == object_bobombcard)
		{
			return true;
		}
	}

	return false;
}

int CInteractionObjectContainer::countTypes(ObjectType type)
{
	int count = 0;

	for(int i = 0; i < list_end; i++)
	{
		if(list[i]->getObjectType() == type)
		{
			count++;
		}
	}

	return count;
}

void CInteractionObjectContainer::removePlayerAreas(int id)
{
	for(int i = 0; i < list_end; i++)
	{
		if(list[i]->getObjectType() == object_area)
		{
			OMO_Area * area = (OMO_Area*)list[i];

			if(area->colorID == id)
				area->reset();
		}
	}
}

void CInteractionObjectContainer::removePlayerRaceGoals(int id)
{
	for(int i = 0; i < list_end; i++)
	{
		if(list[i]->getObjectType() == object_race_goal)
		{
			OMO_RaceGoal * goal = (OMO_RaceGoal*)list[i];
			goal->reset(id);
		}
	}
}

void CInteractionObjectContainer::cleandeadobjects()
{
	for(int i = 0; i < list_end; i++)
	{
		if(list[i]->dead)
		{
			delete list[i];
			list_end--;

			if(i != list_end)
			{
				list[i] = list[list_end];
			}

			i--;
		}
	}
}

