#ifndef _INTERACTIONOBJECT_H
#define _INTERACTIONOBJECT_H

enum ObjectType{object_none = 0, object_block = 1, object_moving = 2, object_overmap = 3, object_area = 4, object_bobombcard = 5, object_egg = 6, object_fireflowercard = 7, object_yoshi = 8, object_explosion = 9, object_race_goal = 10, object_hammercard = 11};
enum MovingObjectType{movingobject_none = 0, movingobject_powerup = 1, movingobject_fireball = 2, movingobject_goomba = 3, movingobject_bulletbill = 4, movingobject_hammer = 5, movingobject_poisonpowerup = 6};

class IO_MovingObject;
//interactionobject base class
class CInteractionObject
{
	public:
		CInteractionObject(gfxSprite *nspr, int x, int y);
		virtual void draw(){printf("CInteractionObject::draw() - NO!\n");};
		virtual void update(){printf("CInteractionObject::update() - NO!\n");};
		virtual void collide(CPlayer * player){printf("CInteractionObject::collide() - NO!\n");};
		virtual void collide(IO_MovingObject * object){printf("CInteractionObject::collide() - NO!\n");};
		virtual ~CInteractionObject(){};
		virtual ObjectType getObjectType(){printf("CInteractionObject::getObjectType() - NO!\n"); return object_none;};
		
		void xf(float xf){nx = xf; ix = (int)nx;};
 		void xi(int xi){ix = xi; nx = (float)ix;};
		void yf(float yf){ny = yf; iy = (int)ny;};
		void yi(int yi){iy = yi; ny = (float)iy;};

		int ix, iy;

		int collisionWidth;
		int collisionHeight;
		int collisionOffsetX;
		int collisionOffsetY;

		int GetState() {return state;}
		bool GetDead() {return dead;}

	protected:
		int iw, ih;

		int index;
		gfxSprite *spr;
		int state;
		bool dead;

		float nx, ny;
		float velx, vely;
	
	friend class CInteractionObjectContainer;
	friend class CPlayer;
	friend void rungame();
};

class IO_Block : public CInteractionObject
{
	public:
		IO_Block(gfxSprite *nspr, int x, int y);
		~IO_Block(){};
		virtual void draw();
		virtual void update();
		virtual bool collide(CPlayer * player, int direction, bool useBehavior);
		virtual bool collide(IO_MovingObject * object, int direction);
		ObjectType getObjectType(){return object_block;}

		virtual bool isTransparent() {return false;}

		virtual bool hittop(CPlayer * player, bool useBehavior);
		virtual bool hitbottom(CPlayer * player, bool useBehavior);
		virtual bool hitright(CPlayer * player, bool useBehavior);
		virtual bool hitleft(CPlayer * player, bool useBehavior);

		virtual bool hittop(IO_MovingObject * object);
		virtual bool hitbottom(IO_MovingObject * object);
		virtual bool hitright(IO_MovingObject * object);
		virtual bool hitleft(IO_MovingObject * object);
	
	protected:
		CPlayer * bumpPlayer;
		float fposx, fposy;  //position to return to (for note and bumpable blocks)
		int	  iposx, iposy; //position of the block (doesn't move)
};

class B_PowerupBlock : public IO_Block
{
	public:
		B_PowerupBlock(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed);
		~B_PowerupBlock(){};
		void draw();
		void update();

		bool hittop(CPlayer * player, bool useBehavior);
		bool hitbottom(CPlayer * player, bool useBehavior);

		bool hittop(IO_MovingObject * object);

	protected:
		int timer;
		bool side;
		int iNumSprites;
		int animationSpeed;
		int drawFrame;
		int animationTimer;
};

class B_BreakableBlock : public IO_Block
{
	public:
		B_BreakableBlock(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed);
		~B_BreakableBlock(){};
		void draw();
		void update();

		bool hittop(CPlayer * player, bool useBehavior);
		bool hitbottom(CPlayer * player, bool useBehavior);

		bool hittop(IO_MovingObject * object);

	private:
		int col, row;
		int iNumSprites;
		int animationSpeed;
		int drawFrame;
		int animationTimer;
};


class B_NoteBlock : public IO_Block
{
	public:
		B_NoteBlock(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed);
		~B_NoteBlock(){};
		void draw();
		void update();

		bool hittop(CPlayer * player, bool useBehavior);
		bool hitbottom(CPlayer * player, bool useBehavior);
		bool hitright(CPlayer * player, bool useBehavior);
		bool hitleft(CPlayer * player, bool useBehavior);

		bool hittop(IO_MovingObject * object);

	private:
		int iNumSprites;
		int animationSpeed;
		int drawFrame;
		int animationTimer;
};

class B_DonutBlock : public IO_Block
{
	public:
		B_DonutBlock(gfxSprite *nspr, int x, int y);
		~B_DonutBlock(){};
		void draw();
		void update();

		bool hittop(CPlayer * player, bool useBehavior);

	private:
		int counter;
		int jigglex;
		int jigglecounter;
		int col, row;
};

class B_FlipBlock : public IO_Block
{
	public:
		B_FlipBlock(gfxSprite *nspr, int x, int y);
		~B_FlipBlock(){};
		void draw();
		void update();
		bool collide(CPlayer * player, int direction, bool useBehavior);
		bool collide(IO_MovingObject * object, int direction);
		bool isTransparent() {return state != 0;}

		bool hittop(CPlayer * player, bool useBehavior);
		bool hitbottom(CPlayer * player, bool useBehavior);
		bool hitright(CPlayer * player, bool useBehavior);
		bool hitleft(CPlayer * player, bool useBehavior);

		bool hittop(IO_MovingObject * object);
		bool hitbottom(IO_MovingObject * object);
		bool hitright(IO_MovingObject * object);
		bool hitleft(IO_MovingObject * object);
	
	private:
		int counter;
		int frame;
		int timer;
};

class B_BounceBlock : public IO_Block
{
	public:
		B_BounceBlock(gfxSprite *nspr, int x, int y);
		~B_BounceBlock(){};
		void update();

		bool hittop(CPlayer * player, bool useBehavior);
		bool hitbottom(CPlayer * player, bool useBehavior);

		bool hittop(IO_MovingObject * object);
};

class IO_MovingObject : public CInteractionObject
{
	public:
		IO_MovingObject(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		IO_MovingObject(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed);
		virtual ~IO_MovingObject(){};
		virtual void draw();
		virtual void update();
		virtual void collide(CPlayer * player);
		
		void collide(IO_MovingObject * object){}
		ObjectType getObjectType(){return objectType;}
		MovingObjectType getMovingObjectType() {return movingObjectType;}
		void collision_detection_map();
		void flipsidesifneeded();

	protected:
		int oldx, oldy;

		int iNumSprites;
		int drawframe;
		int animationtimer;

		float bounce;

		int animationspeed;
		bool inair;

		ObjectType objectType;
		MovingObjectType movingObjectType;

	friend class IO_Block;
	friend class B_PowerupBlock;
	friend class B_BreakableBlock;
	friend class B_NoteBlock;
	friend class B_DonutBlock;
	friend class B_FlipBlock;
	friend class B_BounceBlock;
	friend class OMO_Explosion;

	friend void removeifprojectile(IO_MovingObject * object, bool playsound);
	friend void rungame();
};

class MO_Powerup : public IO_MovingObject
{
	public:
		MO_Powerup(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		MO_Powerup(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed);
		virtual ~MO_Powerup(){};
		void draw();
		void update();
		virtual void collide(CPlayer * player);
		MovingObjectType getMovingObjectType() {return movingObjectType;}

	protected:
		float desty;
};

class PU_StarPowerup : public MO_Powerup
{
	public:
		PU_StarPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~PU_StarPowerup(){};
		void collide(CPlayer * player);

};

class PU_1upPowerup : public MO_Powerup
{
	public:
		PU_1upPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~PU_1upPowerup(){};
		void collide(CPlayer * player);
};

class PU_PoisonPowerup : public MO_Powerup
{
	public:
		PU_PoisonPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~PU_PoisonPowerup(){};
		void collide(CPlayer * player);
};

class PU_FirePowerup : public MO_Powerup
{
	public:
		PU_FirePowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~PU_FirePowerup(){};
		void collide(CPlayer * player);

};

class PU_HammerPowerup : public MO_Powerup
{
	public:
		PU_HammerPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~PU_HammerPowerup(){};
		void collide(CPlayer * player);
};

class PU_ClockPowerup : public MO_Powerup
{
	public:
		PU_ClockPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~PU_ClockPowerup(){};
		void collide(CPlayer * player);
};

class PU_BobombPowerup : public MO_Powerup
{
	public:
		PU_BobombPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~PU_BobombPowerup(){};
		void collide(CPlayer * player);
};

class PU_PowPowerup : public MO_Powerup
{
	public:
		PU_PowPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~PU_PowPowerup(){};
		void collide(CPlayer * player);
};

class PU_BulletBillPowerup : public MO_Powerup
{
	public:
		PU_BulletBillPowerup(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~PU_BulletBillPowerup(){};
		void collide(CPlayer * player);
};

class MO_Fireball : public IO_MovingObject
{
	public:
		MO_Fireball(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int playerID, int colorID);
		~MO_Fireball(){};
		void update();
		void collide(CPlayer * player);
		void draw();

		int playerID;
		bool direction;
		int colorOffset;

	private:
		int ttl;
};

class OMO_Hammer : public IO_MovingObject
{
	public:
		OMO_Hammer(gfxSprite *nspr, int x, int y, int iNumSpr, bool moveToRight, int aniSpeed, int playerID, int colorID);
		~OMO_Hammer(){};
		void update();
		void collide(CPlayer * player);
		void draw();

		int playerID;
		bool direction;
		int colorOffset;

	private:
		int ttl;
};

class PU_Coin : public MO_Powerup
{
	public:
		PU_Coin(gfxSprite *nspr, int iNumSpr, int aniSpeed);
		~PU_Coin(){};
		void update();
		void collide(CPlayer * player);
		void placeCoin();

	private:
		int timer;
};

class IO_OverMapObject : public CInteractionObject
{
	public:
		IO_OverMapObject(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed);
		virtual ~IO_OverMapObject(){};
		virtual void draw();
		virtual void update();
		virtual void collide(CPlayer * player){}
		virtual void collide(IO_MovingObject * object){}

		ObjectType getObjectType(){return objectType;}
		MovingObjectType getMovingObjectType() {return movingObjectType;}
		
	protected:

		int iNumSprites;
		int drawframe;
		int animationtimer;

		float bounce;

		int animationspeed;

		ObjectType objectType;
		MovingObjectType movingObjectType;
};

class OMO_Thwomp : public IO_OverMapObject
{
	public:
		OMO_Thwomp(gfxSprite *nspr, int x, float nspeed);
		~OMO_Thwomp(){};
		void update();
		void collide(CPlayer * player);
};

class OMO_Egg : public IO_OverMapObject
{
	public:
		OMO_Egg(gfxSprite *nspr);
		~OMO_Egg(){};
		void update();
		void draw();
		void collide(CPlayer * player);
		void placeEgg();

	private:
		CPlayer * owner;
		int timer;

	friend class CPlayer;
	friend class OMO_Yoshi;
	friend class CGM_Eggs;
};

class OMO_Yoshi : public IO_OverMapObject
{
	public:
		OMO_Yoshi(gfxSprite *nspr);
		~OMO_Yoshi(){};
		void update();
		void collide(CPlayer * player);
		void placeYoshi();

	private:
		int timer;
};

class OMO_Area : public IO_OverMapObject
{
	public:
		OMO_Area(gfxSprite *nspr, int numAreas);
		~OMO_Area(){};
		void draw();
		void update();
		void collide(CPlayer * player);
		void placeArea(int numAreas);
		void reset();
		int getColorID() {return colorID;}
	
	private:
		int playerID;
		int colorID;
		int scoretimer;

	friend class CInteractionObjectContainer;
};

class OMO_RaceGoal : public IO_OverMapObject
{
	public:
		OMO_RaceGoal(gfxSprite *nspr, int id);
		~OMO_RaceGoal(){};
		void draw();
		void update();
		void collide(CPlayer * player);
		void placeRaceGoal();
		void reset(int teamID) {tagged[teamID] = -1;}
		int isTagged(int teamID) {return tagged[teamID];}
		int getGoalID() {return goalID;}
	
	private:
		int tagged[4];
		int goalID;
		float angle;
		float anglechange;
		int anglechangetimer;
			
	friend class CInteractionObjectContainer;
};

class PU_BobombCard : public MO_Powerup
{
	public:
		PU_BobombCard(gfxSprite *nspr, int iNumSpr, int aniSpeed);
		~PU_BobombCard(){};
		void update();
		void collide(CPlayer * player);
		void placeBobombCard();

	private:
		int timer;
};

class PU_FireFlowerCard : public MO_Powerup
{
	public:
		PU_FireFlowerCard(gfxSprite *nspr, int iNumSpr, int aniSpeed);
		~PU_FireFlowerCard(){};
		void update();
		void collide(CPlayer * player);
		void placeFireFlowerCard();

	private:
		int timer;
};

class PU_HammerCard : public MO_Powerup
{
	public:
		PU_HammerCard(gfxSprite *nspr, int iNumSpr, int aniSpeed);
		~PU_HammerCard(){};
		void update();
		void collide(CPlayer * player);
		void placeHammerCard();

	private:
		int timer;
};

class OMO_Explosion : public IO_OverMapObject
{
	public:
		OMO_Explosion(gfxSprite *nspr, int x, int y, int iNumSpr, int aniSpeed, int id);
		~OMO_Explosion(){};
		void update();
		void collide(CPlayer * player);
		void collide(IO_MovingObject * object);
	private:
		int playerID;
		int timer;
};

class MO_Goomba : public IO_MovingObject
{
	public:
		MO_Goomba(gfxSprite *nspr, int iNumSpr, int aniSpeed, bool moveToRight, int iCollisionWidth, int iCollisionHeight, int iCollisionOffsetX, int iCollisionOffsetY);
		~MO_Goomba(){};
		void draw();
		void update();
		void collide(CPlayer * player);
		void collide(IO_MovingObject * object);
		void place();

		void hittop(CPlayer * player);
		void hitother(CPlayer * player);

	private:
		float spawnradius;
		float spawnangle;
};

class OMO_BulletBill : public IO_MovingObject
{
	public:
		OMO_BulletBill(gfxSprite *nspr, int y, float nspeed);
		~OMO_BulletBill(){};
		void update();
		void draw();
		void collide(CPlayer * player);
		void hittop(CPlayer * player);
		void hitother(CPlayer * player);
		void collide(IO_MovingObject * object);
};

//interaction object container
class CInteractionObjectContainer
{
	public:
		CInteractionObjectContainer();
		~CInteractionObjectContainer();

		void add(CInteractionObject *cio);

		void update()
		{
			for(int i = 0; i<list_end; i++)
				list[i]->update();
		};

		void draw()
		{
			for(int i = 0; i<list_end; i++)
				list[i]->draw();
		};

		void clean();

		bool isBlockAt(int x, int y);

		void cleandeadobjects();

		float getClosestObject(int x, int y, int objectType);
		bool isBombReady();
		int countTypes(ObjectType type);
		void removePlayerAreas(int id);
		void removePlayerRaceGoals(int id);

	public:
		CInteractionObject *list[MAXINTERACTIONOBJECTS];
		int		 list_end;
};

#endif

