#include "global.h"

extern gfxSprite spr_frontmap;

CMap::~CMap()
{
	if(tilesetsurface != NULL)
	{
		SDL_FreeSurface(tilesetsurface);
		tilesetsurface = NULL;
	}
}


void CMap::loadTileSet(const char *tilesetfile, const char *tilesetbmp)
{
	FILE *tsf;
	int i;

	clearTileSet();

	//1. load tileset file
	printf("loading tile set from %s ... ", tilesetfile);

	tsf = fopen(tilesetfile, "rb");
	if(tsf == NULL)
	{
		printf("\n ERROR: couldn't open tileset file.\n");
		return;
	}
	
	for(i = 0; i < TILESETSIZE; i++)
	{
		tileset[i] = (TileType)ReadInt(tsf);
	}

	tileset[TILESETSIZE] = tile_nonsolid;	//this is the no tile selected tile

	fclose(tsf);
	printf("done\n");
	
	

	//2. load tileset graphics
	printf("loading tile set suface from %s ... ", tilesetbmp);

	if(tilesetsurface != NULL)
	{
		SDL_FreeSurface(tilesetsurface);
		tilesetsurface = NULL;
	}


	tilesetsurface = SDL_LoadBMP(tilesetbmp);

    if (tilesetsurface == NULL)
	{
		printf("\n ERROR: Couldn't load %s: %s\n", tilesetbmp, SDL_GetError());
        return;
    }

	if( SDL_SetColorKey(tilesetsurface, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(tilesetsurface->format, 255, 0, 255)) < 0)
	{
		printf("\n ERROR: Couldn't set ColorKey + RLE: %s\n", SDL_GetError());
		return;
	}

	SDL_Surface *temp;
	if( (temp = SDL_DisplayFormat(tilesetsurface)) == NULL)
	{
		printf("\n ERROR: couldn't convert to display format: %s\n", SDL_GetError());
		return;
	}

	SDL_FreeSurface(tilesetsurface);
	tilesetsurface = temp;

	printf("done\n");
}

//With the new 20x15 tile set, we need to convert old maps to use the 
//correct indexes into the tile set
void CMap::convertMap()
{
	int i, j, k;

	bool fTilesNeedConversion = true;
	bool fBlocksNeedConversion = true;

	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			for(k = 0; k < MAPLAYERS; k++)
			{
				//if we find an empty tile (value 300), then we already have a converted map
				if(mapdata[i][j][k] == TILESETSIZE)
					fTilesNeedConversion = false;
			}

			if(interactionobjectdata[i][j] == INTERACTBLOCKSETSIZE)
				fBlocksNeedConversion = false;
		}
	}

	if(!fTilesNeedConversion && !fBlocksNeedConversion)
		return;

	//Otherwise, convert this map
	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			/*
			int data = mapdata[i][j];

			if(data == 96)
				mapdata[i][j] = TILESETSIZE;
			else
				mapdata[i][j] = (data / 12 * TILESETWIDTH) + (data % 12);
			*/

			int data = interactionobjectdata[i][j];

			if(data == 5)
				interactionobjectdata[i][j] = INTERACTBLOCKSETSIZE;
			else if(data == 4)
				interactionobjectdata[i][j] = 5;
		}
	}
}

void CMap::saveTileSet(const char *tilesetfile)
{
	FILE *mapfile;
	int i;

	printf("saving tileset %s ... ", tilesetfile);

	mapfile = fopen(tilesetfile, "wb");
	if(mapfile == NULL)
	{
		printf("\n ERROR: couldn't open tileset file\n");
		return;
	}

	//save tileset
	for(i = 0; i < TILESETSIZE; i++)
	{
		WriteInt(tileset[i], mapfile);
	}
	

	fclose(mapfile);

	printf("done\n");
}


void CMap::clearTileSet()
{
	int i;
	for(i=0; i<TILESETSIZE; i++)
	{
		tileset[i] = tile_nonsolid;
	}
	tilebltrect.w = TILESIZE;
	tilebltrect.h = TILESIZE;
}

void CMap::clearMap()
{
	int i, j, k;

	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			for(k = 0; k < MAPLAYERS; k++)
			{
				//reset tile
				mapdata[i][j][k] = TILESETSIZE;	//no tile selected
			}

			interactionobjectdata[i][j] = INTERACTBLOCKSETSIZE;
			warpdata[i][j].direction = -1;
			warpdata[i][j].connection = -1;
		}
	}

	bltrect.w = TILESIZE;
	bltrect.h = TILESIZE;
}

void CMap::loadMap(const char *file)
{
	FILE *mapfile;
	int i, j, k;

	printf("loading map %s ...", file);

	//clear map
	//clearMap();
	
	mapfile = fopen(file, "rb");
	if(mapfile == NULL)
	{
		printf("\n ERROR: couldn't open map\n");
		return;
	}

	//Load version number
	int version[4];
	version[0] = ReadInt(mapfile); //Major
	version[1] = ReadInt(mapfile); //Minor
	version[2] = ReadInt(mapfile); //Micro
	version[3] = ReadInt(mapfile); //Build

	if(version[0] == 1 && version[1] == 6)
	{
		printf("[Version %d.%d.%d.%d Map Detected]", version[0], version[1], version[2], version[3]);

		//2. load map data
		for(j = 0; j < MAPHEIGHT; j++)
		{
			for(i = 0; i < MAPWIDTH; i++)
			{
				for(k = 0; k < MAPLAYERS; k++)
				{
					mapdata[i][j][k] = ReadInt(mapfile);
				}

				mapdatatop[i][j] = tile_nonsolid;
				for(k = MAPLAYERS - 1; k >= 0; k--)
				{
					if(tileset[mapdata[i][j][k]] != tile_nonsolid)
					{
						mapdatatop[i][j] = tileset[mapdata[i][j][k]];
						break;
					}
				}

				interactionobjectdata[i][j] = ReadInt(mapfile);
				warpdata[i][j].direction = ReadInt(mapfile);
				warpdata[i][j].connection = ReadInt(mapfile);
				warpdata[i][j].id = ReadInt(mapfile);

				if(version[2] == 1)
					nospawn[i][j] = ReadInt(mapfile) == 0 ? false : true;
			}
		}

		//Read in background to use
		backgroundID = ReadInt(mapfile);

		//Read in eyecandy to use
		eyecandyID = ReadInt(mapfile);

		maxConnection = 0;

		numwarpexits = ReadInt(mapfile);
		for(i = 0; i < numwarpexits && i < MAXWARPS; i++)
		{
			warpexits[i].direction = ReadInt(mapfile);
			warpexits[i].connection = ReadInt(mapfile);
			warpexits[i].id = ReadInt(mapfile);
			warpexits[i].x = ReadInt(mapfile);
			warpexits[i].y = ReadInt(mapfile);

			warpexits[i].lockx = ReadInt(mapfile);
			warpexits[i].locky = ReadInt(mapfile);

			warpexits[i].warpx = ReadInt(mapfile);
			warpexits[i].warpy = ReadInt(mapfile);
			warpexits[i].numblocks = ReadInt(mapfile);

			if(warpexits[i].connection > maxConnection)
				maxConnection = warpexits[i].connection;
		}

		//Ignore any more warps than the max
		for(i = 0; i < numwarpexits - MAXWARPS; i++)
		{
			for(j = 0; j < 10; j++)
				ReadInt(mapfile);
		}

		if(numwarpexits > MAXWARPS)
			numwarpexits = MAXWARPS;

		//Read spawn areas
		totalspawnsize = 0;
		numspawnareas = ReadInt(mapfile);

		if(numspawnareas > MAXSPAWNAREAS)
		{
			printf("\n ERROR: number of spawn areas (%d) was greater than max allowed (%d)\n", numspawnareas, MAXSPAWNAREAS);
			return;
		}

		for(int m = 0; m < numspawnareas; m++)
		{
			spawnareas[m].left = ReadInt(mapfile);
			spawnareas[m].top = ReadInt(mapfile);
			spawnareas[m].width = ReadInt(mapfile);
			spawnareas[m].height = ReadInt(mapfile);
			spawnareas[m].size = ReadInt(mapfile);
		
			if(version[2] == 0)
			{
				spawnareas[m].width -= spawnareas[m].left;
				spawnareas[m].height -= spawnareas[m].top;
			}

			totalspawnsize += spawnareas[m].size;
		}

		if(version[2] == 1)
		{
			numdrawareas = ReadInt(mapfile);

			if(numdrawareas > MAXDRAWAREAS)
			{
				printf("\n ERROR: number of draw areas (%d) was greater than max allowed (%d)\n", numdrawareas, MAXDRAWAREAS);
				return;
			}

			for(int m = 0; m < numdrawareas; m++)
			{
				drawareas[m].x = ReadInt(mapfile);
				drawareas[m].y = ReadInt(mapfile);
				drawareas[m].w = ReadInt(mapfile);
				drawareas[m].h = ReadInt(mapfile);
			}
		}
	}
	else //If the version is unrecognized (1.5 maps didn't have version numbers)
	{
		printf("[Version 1.5 Map Detected]");

		//Reset position of read cursor
		rewind(mapfile);

		//clear map (we won't be reading in all the layers so it needs to be cleared)
		clearMap();
		
		//2. load map data
		for(j = 0; j < MAPHEIGHT; j++)
		{
			for(i = 0; i < MAPWIDTH; i++)
			{
				//Read everything into layer 1
				mapdata[i][j][1] = ReadInt(mapfile);
			}
		}

		//3. load interaction objects data
		for(j = 0; j < MAPHEIGHT; j++)
		{
			for(i = 0; i < MAPWIDTH; i++)
			{
				interactionobjectdata[i][j] = ReadInt(mapfile);
			}
		}

		backgroundID = ReadInt(mapfile);
		
		//All 1.5 maps will use cloud eyecandy
		eyecandyID = 1;
	}

	fclose(mapfile);
	clearWarpLocks();

	printf(" done\n");
}




void CMap::saveMap(const char *file)
{
	FILE *mapfile;
	int i, j, k;

	printf("saving map %s ... ", file);

	mapfile = fopen(file, "wb");
	if(mapfile == NULL)
	{
		printf("\n ERROR: couldn't save map\n");
		return;
	}

	//First write the map compatibility version number 
	//(this will allow the map loader to identify if the map needs conversion)
	WriteInt(1, mapfile); //Major
	WriteInt(6, mapfile); //Minor
	WriteInt(1, mapfile); //Micro
	WriteInt(0, mapfile); //Build

	//Calculate warp zones
	bool usedtile[MAPWIDTH][MAPHEIGHT];
	for(j = 0; j < MAPHEIGHT; j++)
		for(i = 0; i < MAPWIDTH; i++)
			usedtile[i][j] = false;

	int numWarps = 0;
	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			if(warpdata[i][j].connection != -1 && !usedtile[i][j])
			{
				int movex = 0, movey = 0;
				int currentx = i, currenty = j;

				if(warpdata[i][j].direction == 0 || warpdata[i][j].direction == 2)
					movex = 1;
				else
					movey = 1;

				while(currentx < MAPWIDTH && currenty < MAPHEIGHT)
				{
					if(warpdata[currentx][currenty].direction != warpdata[i][j].direction ||
						warpdata[currentx][currenty].connection != warpdata[i][j].connection)
					{
						break;
					}

					usedtile[currentx][currenty] = true;
					
					if(numWarps < MAXWARPS)
					{
						warpdata[currentx][currenty].id = numWarps;
					}
					else
					{
						warpdata[currentx][currenty].connection = -1;
						warpdata[currentx][currenty].direction = -1;
					}

					currentx += movex;
					currenty += movey;
				}

				numWarps++;
			}
		}
	}

	//save map
	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			for(k = 0; k < MAPLAYERS; k++)
			{
				WriteInt(mapdata[i][j][k], mapfile);
			}
			
			WriteInt(interactionobjectdata[i][j], mapfile);
			WriteInt(warpdata[i][j].direction, mapfile);
			WriteInt(warpdata[i][j].connection, mapfile);
			WriteInt(warpdata[i][j].id, mapfile);
			WriteInt((int)nospawn[i][j], mapfile);
		}
	}

	//Write background ID
	WriteInt(backgroundID, mapfile);

	//Write eyecandy ID
	WriteInt(eyecandyID, mapfile);

	//Write number of warp zones
	WriteInt(numWarps, mapfile);

	for(j = 0; j < MAPHEIGHT; j++)
		for(i = 0; i < MAPWIDTH; i++)
			usedtile[i][j] = false;

	//scan warp data and write out warp exits
	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			//if we found a warp area and it hasn't been consumed by another warp area
			//then scan for surrounding warp areas and consume them to create a warp exit
			if(warpdata[i][j].connection != -1 && !usedtile[i][j])
			{
				int movex = 0, movey = 0;
				int currentx = i, currenty = j;

				if(warpdata[i][j].direction == 0 || warpdata[i][j].direction == 2)
					movex = 1;
				else
					movey = 1;


				int numblocks = 0;
				while(currentx < MAPWIDTH && currenty < MAPHEIGHT)
				{
					if(warpdata[currentx][currenty].direction != warpdata[i][j].direction ||
						warpdata[currentx][currenty].connection != warpdata[i][j].connection)
					{
						break;
					}
					
					usedtile[currentx][currenty] = true;

					currentx += movex;
					currenty += movey;
					numblocks++;
				}

				//We're on the next block that isn't a warp
				currentx -= movex;
				currenty -= movey;

				WriteInt(warpdata[i][j].direction, mapfile);
				WriteInt(warpdata[i][j].connection, mapfile);
				WriteInt(warpdata[i][j].id, mapfile);
				
				if(warpdata[i][j].direction == 0)
				{
					//Write out warp exit for player
					WriteInt(((currentx * TILESIZE + TILESIZE - i * TILESIZE) >> 1) + i * TILESIZE - HALFPW, mapfile);
					WriteInt(j * TILESIZE - 1 + PHOFFSET, mapfile);

					//Write out position for lock icon
					WriteInt(((currentx * TILESIZE + TILESIZE - i * TILESIZE) >> 1) + i * TILESIZE - 16, mapfile);
					WriteInt(j * TILESIZE, mapfile);
				}
				else if(warpdata[i][j].direction == 2)
				{
					WriteInt(((currentx * TILESIZE + TILESIZE - i * TILESIZE) >> 1) + i * TILESIZE - HALFPW, mapfile);
					WriteInt(j * TILESIZE + 1 + PHOFFSET, mapfile);

					WriteInt(((currentx * TILESIZE + TILESIZE - i * TILESIZE) >> 1) + i * TILESIZE - 16, mapfile);
					WriteInt(j * TILESIZE, mapfile);
				}
				else if(warpdata[i][j].direction == 1)
				{
					WriteInt(i * TILESIZE + TILESIZE - PW - PWOFFSET, mapfile);
					WriteInt(currenty * TILESIZE + TILESIZE - PH - 1, mapfile);

					WriteInt(i * TILESIZE, mapfile);
					WriteInt(((currenty * TILESIZE + TILESIZE - j * TILESIZE) >> 1) + j * TILESIZE - 16, mapfile);
				}
				else if(warpdata[i][j].direction == 3)
				{
					WriteInt(i * TILESIZE - 1 + PWOFFSET, mapfile);
					WriteInt(currenty * TILESIZE + TILESIZE - PH - 1, mapfile);

					WriteInt(i * TILESIZE, mapfile);
					WriteInt(((currenty * TILESIZE + TILESIZE - j * TILESIZE) >> 1) + j * TILESIZE - 16, mapfile);
				}

				WriteInt(i, mapfile);
				WriteInt(j, mapfile);
				WriteInt(numblocks, mapfile);

			}
		}
	}

	calculatespawnareas(false);
	
	if(numspawnareas == 0)
		calculatespawnareas(true);

	//Write spawn areas
	WriteInt(numspawnareas, mapfile);

	for(int m = 0; m < numspawnareas; m++)
	{
		WriteInt(spawnareas[m].left, mapfile);
		WriteInt(spawnareas[m].top, mapfile);
		WriteInt(spawnareas[m].width, mapfile);
		WriteInt(spawnareas[m].height, mapfile);
		WriteInt(spawnareas[m].size, mapfile);
	}


	for(j = 0; j < MAPHEIGHT; j++)
		for(i = 0; i < MAPWIDTH; i++)
			usedtile[i][j] = false;

	//Figure out areas that contain tiles in layers 2 and 3
	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			if(mapdata[i][j][2] == TILESETSIZE && mapdata[i][j][3] == TILESETSIZE)
				usedtile[i][j] = true;
		}
	}

	//Figure out where the overlayer draw rects are
	numdrawareas = 0;
	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			if(!usedtile[i][j])
			{
				bool fDownDone = false;
				bool fRightDone = false;

				int downsize = j + 1;
				int rightsize = i + 1;
				int attempt = 1;

				while(true)
				{
					if(!fRightDone)
					{
						for(int right = i; right < rightsize; right++)
						{
							if(right >= MAPWIDTH || j + attempt >= MAPHEIGHT || usedtile[right][j + attempt])
								fRightDone = true;
						}

						if(!fRightDone)
							downsize++;
					}

					if(!fDownDone)
					{
						for(int down = j; down < downsize; down++)
						{
							if(i + attempt >= MAPWIDTH || down >= MAPHEIGHT || usedtile[i + attempt][down])
								fDownDone = true;
						}

						if(!fDownDone)
							rightsize++;
					}

					if(fDownDone && fRightDone)
					{
						if(numdrawareas < MAXDRAWAREAS)
						{
							drawareas[numdrawareas].x = i * TILESIZE;
							drawareas[numdrawareas].y = j * TILESIZE;
							drawareas[numdrawareas].w = (rightsize - i) * TILESIZE;
							drawareas[numdrawareas].h = (downsize - j) * TILESIZE;

							numdrawareas++;
						}

						for(int down = j; down < downsize; down++)
						{
							for(int right = i; right < rightsize; right++)
							{
								usedtile[right][down] = true;
							}
						}
						
						break;
					}

					attempt++;
				}
			}
		}
	}

	//Write draw areas
	WriteInt(numdrawareas, mapfile);

	for(int m = 0; m < numdrawareas; m++)
	{
		WriteInt(drawareas[m].x, mapfile);
		WriteInt(drawareas[m].y, mapfile);
		WriteInt(drawareas[m].w, mapfile);
		WriteInt(drawareas[m].h, mapfile);
	}

	fclose(mapfile);

	printf("done\n");
}

void CMap::calculatespawnareas(bool usetempblocks)
{
	bool usedtile[MAPWIDTH][MAPHEIGHT];
	int i, j, k;

	for(j = 0; j < MAPHEIGHT; j++)
		for(i = 0; i < MAPWIDTH; i++)
			usedtile[i][j] = false;

	//Figure out areas where spawning is allowed
	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			bool fUsed = false;

			if(j >= 13 || j == 0)
				fUsed = true;

			if(!fUsed)
			{
				if(nospawn[i][j])
					fUsed = true;
			}

			if(!fUsed)
			{
				for(k = 0; k < MAPLAYERS; k++)
				{
					if(tileset[mapdata[i][j][k]] != tile_nonsolid && tileset[mapdata[i][j][k]] != tile_solid_on_top)
					{
						fUsed = true;
						break;
					}
				}
			}
			
			//If there is an interaction block there
			if(!fUsed)
			{
				if(interactionobjectdata[i][j] != INTERACTBLOCKSETSIZE)
					fUsed = true;
			}

			//If there is a death tile directly above
			if(!fUsed)
			{
				if(j > 0)
				{
					TileType type = tile_nonsolid;
					for(k = MAPLAYERS - 1; k >= 0; k--)
					{
						if(tileset[mapdata[i][j - 1][k]] != tile_nonsolid)
						{
							type = tileset[mapdata[i][j - 1][k]];
							break;
						}
					}

					if(type == tile_death_on_bottom || type == tile_death)
					{
						fUsed = true;
					}
				}
			}

			//If there is a death tile anywhere below this tile
			if(!fUsed)
			{
				int m;
				for(m = j; m < MAPHEIGHT; m++)
				{
					TileType type = tile_nonsolid;
					for(k = MAPLAYERS - 1; k >= 0; k--)
					{
						if(tileset[mapdata[i][m][k]] != tile_nonsolid)
						{
							type = tileset[mapdata[i][m][k]];
							break;
						}
					}

					if(type == tile_death_on_top || type == tile_death)
					{
						fUsed = true;
						break;
					}
					
					if(usetempblocks)
					{
						if(type != tile_nonsolid || interactionobjectdata[i][m] != INTERACTBLOCKSETSIZE)
						{
							break;
						}
					}
					else
					{
						if(type != tile_nonsolid || (interactionobjectdata[i][m] != INTERACTBLOCKSETSIZE && interactionobjectdata[i][m] != 0 && interactionobjectdata[i][m] != 2))
						{
							break;
						}
					}
				}

				//If we didn't find a landing spot from here to bottom, then try to wrap around and see
				if(m == MAPHEIGHT)
				{
					for(m = 0; m < j; m++)
					{
						TileType type = tile_nonsolid;
						for(k = MAPLAYERS - 1; k >= 0; k--)
						{
							if(tileset[mapdata[i][m][k]] != tile_nonsolid)
							{
								type = tileset[mapdata[i][m][k]];
								break;
							}
						}

						if(type == tile_death_on_top || type == tile_death)
						{
							fUsed = true;
							break;
						}
						
						if(usetempblocks)
						{
							if(type != tile_nonsolid || interactionobjectdata[i][m] != INTERACTBLOCKSETSIZE)
							{
								break;
							}
						}
						else
						{
							if(type != tile_nonsolid || (interactionobjectdata[i][m] != INTERACTBLOCKSETSIZE && interactionobjectdata[i][m] != 0 && interactionobjectdata[i][m] != 2))
							{
								break;
							}
						}
					}

					if(m == j)
						fUsed = true;
				}
			}

			usedtile[i][j] = fUsed;
		}
	}

	//Figure out where the spawn areas are
	numspawnareas = 0;
	for(j = 0; j < MAPHEIGHT; j++)
	{
		for(i = 0; i < MAPWIDTH; i++)
		{
			if(!usedtile[i][j])
			{
				bool fDownDone = false;
				bool fRightDone = false;

				int downsize = j + 1;
				int rightsize = i + 1;
				int attempt = 1;

				while(true)
				{
					if(!fRightDone)
					{
						for(int right = i; right < rightsize; right++)
						{
							if(right >= MAPWIDTH || j + attempt >= MAPHEIGHT || usedtile[right][j + attempt])
								fRightDone = true;
						}

						if(!fRightDone)
							downsize++;
					}

					if(!fDownDone)
					{
						for(int down = j; down < downsize; down++)
						{
							if(i + attempt >= MAPWIDTH || down >= MAPHEIGHT || usedtile[i + attempt][down])
								fDownDone = true;
						}

						if(!fDownDone)
							rightsize++;
					}

					if(fDownDone && fRightDone)
					{
						if(numspawnareas < MAXSPAWNAREAS)
						{
							spawnareas[numspawnareas].left = i;
							spawnareas[numspawnareas].top = j;
							spawnareas[numspawnareas].width = rightsize - 1 - i;
							spawnareas[numspawnareas].height = downsize - 1 - j;
							spawnareas[numspawnareas].size = (rightsize - i) * (downsize - j);

							numspawnareas++;
						}

						for(int down = j; down < downsize; down++)
						{
							for(int right = i; right < rightsize; right++)
							{
								usedtile[right][down] = true;
							}
						}
						
						break;
					}

					attempt++;
				}
			}
		}
	}
}



void CMap::draw(SDL_Surface *targetsurf, int layer)
{
	int i, j, ts;
	
	//draw left to right full vertical
	bltrect.x = 0;
	for(i = 0; i < MAPWIDTH; i++)
	{
		bltrect.y = -TILESIZE;	//this is okay, see

		for(j = 0; j < MAPHEIGHT; j++)
		{
			bltrect.y += TILESIZE;	// here

			ts = mapdata[i][j][layer];
			if(ts == TILESETSIZE)
				continue;

			tilebltrect.x = (ts%TILESETWIDTH)*TILESIZE;
			tilebltrect.y = (ts/TILESETWIDTH)*TILESIZE;
		
			SDL_BlitSurface(tilesetsurface, &tilebltrect, targetsurf, &bltrect);
		}

		bltrect.x += TILESIZE;
	}

	bltrect.x = 0;
	bltrect.y = 0;
	bltrect.w = 640;
	bltrect.h = 480;
}


void CMap::predrawbackground(gfxSprite &background, gfxSprite &mapspr)
{
	SDL_Rect r;
	r.x = 0;
	r.y = 0;
	r.w = 640;
	r.h = 480;

	SDL_BlitSurface(background.getSurface(), NULL, mapspr.getSurface(), &r);
	
	draw(mapspr.getSurface(), 0);
	draw(mapspr.getSurface(), 1);
	draw(mapspr.getSurface(), 2);
	draw(mapspr.getSurface(), 3);

	SDL_Rect dest;

	dest.w = 32;
	dest.h = 32;

	/*
	for(int j = 0; j < MAPHEIGHT; j++)
	{
		for(int i = 0; i < MAPWIDTH; i++)
		{
			if(testtile[i][j])
			{
				dest.x = i * 32;
				dest.y = j * 32;

				SDL_FillRect(mapspr.getSurface(), &dest, 0x0);		//fill empty area with black
			}
		}
	}
	*/
	/*
	for(int m = 0; m < numspawnareas; m++)
	{
		dest.x = spawnareas[m].left * TILESIZE;
		dest.y = spawnareas[m].top * TILESIZE;

		dest.w = (spawnareas[m].width) * TILESIZE + TILESIZE;
		dest.h = (spawnareas[m].height) * TILESIZE + TILESIZE;

		int color = 0x00 << 24 | rand() % 256 << 16 | rand() % 256 << 8 | rand() % 256;

		SDL_FillRect(mapspr.getSurface(), &dest, color);		//fill empty area with black
	}
	*/
}

void CMap::predrawforeground(gfxSprite &foregroundspr)
{
	SDL_Rect r;
	r.x = 0;
	r.y = 0;
	r.w = 640;
	r.h = 480;

	SDL_FillRect(foregroundspr.getSurface(), NULL, SDL_MapRGB(foregroundspr.getSurface()->format, 255, 0, 255));
	SDL_SetColorKey(foregroundspr.getSurface(), SDL_SRCCOLORKEY, SDL_MapRGB(foregroundspr.getSurface()->format, 255, 0, 255));

	draw(foregroundspr.getSurface(), 2);
	draw(foregroundspr.getSurface(), 3);

	/*
	for(int k = 0; k < numdrawareas; k++)
	{
		SDL_Rect rect;
		rect.x = drawareas[k].x;
		rect.y = drawareas[k].y;
		rect.w = drawareas[k].w;
		rect.h = drawareas[k].h;
		int color = 0x00 << 24 | rand() % 256 << 16 | rand() % 256 << 8 | rand() % 256;

		SDL_FillRect(foregroundspr.getSurface(), &rect, color);
	}
	*/
}

WarpExit * CMap::getRandomWarpExit(int connection, int currentID)
{
	int indices[MAXWARPS];
	int numIndices = 0;

	WarpExit * currentWarp = NULL;

	for(int k = 0; k < numwarpexits; k++)
	{
		if(warpexits[k].connection == connection)
		{
			if(warpexits[k].id == currentID)
				currentWarp = &warpexits[k];
			else
				indices[numIndices++] = k;
		}
	}

	if(numIndices == 0)
		return currentWarp;

	return &warpexits[indices[rand() % numIndices]];
}

void CMap::clearWarpLocks()
{
	for(int k = 0; k < 10; k++)
	{
		warplocktimer[k] = 0;
		warplocked[k] = false;
	}
}

void CMap::updateWarpLocks()
{
	for(int k = 0; k <= maxConnection; k++)
	{
		if(warplocked[k])
		{
			if(++warplocktimer[k] > 200)
			{
				warplocked[k] = false;
				warplocktimer[k] = 0;
			}
		}
	}
}

void CMap::drawWarpLocks()
{
	for(int k = 0; k < numwarpexits; k++)
	{
		if(warplocked[warpexits[k].connection])
		{
			spr_warplock.draw(warpexits[k].lockx, warpexits[k].locky);
		}
	}
}

void CMap::findspawnpoint(int * x, int * y, int width, int height, bool tilealigned)
{
	if(totalspawnsize <= 0)
	{
		*x = 320;
		*y = 224;
		return;
	}

	int spawnarea = rand() % totalspawnsize;

	int currentsize = 0;
	for(int m = 0; m < numspawnareas; m++)
	{
		currentsize += spawnareas[m].size;

		if(spawnarea >= currentsize)
			continue;

		int areawidth = spawnareas[m].width * TILESIZE + TILESIZE;
		int areaheight = spawnareas[m].height * TILESIZE + TILESIZE;

		if(width > areawidth || height > areaheight)
			continue;

		if(tilealigned)
		{
			int xoffset = spawnareas[m].width;
			int yoffset = spawnareas[m].height;

			if(xoffset > 0)
				xoffset = rand() % xoffset;

			if(yoffset > 0)
				yoffset = rand() % yoffset;

			*x = xoffset * TILESIZE + spawnareas[m].left * TILESIZE + (TILESIZE >> 1) - (width >> 1);
			*y = yoffset * TILESIZE + spawnareas[m].top * TILESIZE + (TILESIZE >> 1) - (height >> 1);
		}
		else
		{
			int xoffset = areawidth - width;
			int yoffset = areaheight - height;

			if(xoffset > 0)
				xoffset = rand() % xoffset;

			if(yoffset > 0)
				yoffset = rand() % yoffset;

			*x = xoffset + spawnareas[m].left * TILESIZE;
			*y = yoffset + spawnareas[m].top * TILESIZE;
		}

		return;
	}
}

void CMap::drawfrontlayer()
{
	for(int k = 0; k < numdrawareas; k++)
		spr_frontmap.draw(drawareas[k].x, drawareas[k].y, drawareas[k].x, drawareas[k].y, drawareas[k].w, drawareas[k].h);
}

void CMap::WriteInt(int out, FILE * outFile)
{
#if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
	int t = out;

    ((char*)&out)[0] = ((char*)&t)[3];
    ((char*)&out)[1] = ((char*)&t)[2];
    ((char*)&out)[2] = ((char*)&t)[1];
    ((char*)&out)[3] = ((char*)&t)[0];
#endif

	fwrite(&out, sizeof(int), 1, outFile);
}

int CMap::ReadInt(FILE * inFile)
{
	int in;
	fread(&in, sizeof(int), 1, inFile);

#if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
	int t = in;

    ((char*)&in)[0] = ((char*)&t)[3];
    ((char*)&in)[1] = ((char*)&t)[2];
    ((char*)&in)[2] = ((char*)&t)[1];
    ((char*)&in)[3] = ((char*)&t)[0];
#endif

	return in;
}

void CMap::optimize()
{
	for(int j = 0; j < MAPHEIGHT; j++)
	{
		for(int i = 0; i < MAPWIDTH; i++)
		{
			for(int m = 1; m < MAPLAYERS; m++)
			{
				if(tileset[mapdata[i][j][m]] != tile_nonsolid && tileset[mapdata[i][j][m]] != tile_solid_on_top)
				{
					for(int k = m - 1; k >= 0; k--)
					{
						if(mapdata[i][j][k] == TILESETSIZE)
						{
							mapdata[i][j][k] = mapdata[i][j][k + 1];
							mapdata[i][j][k + 1] = TILESETSIZE;
						}
						else
						{
							break;
						}
					}
				}
			}
		}
	}
}

