
#ifndef _MAP_H
#define _MAP_H

enum TileType{tile_nonsolid = 0, tile_solid = 1, tile_solid_on_top = 2, tile_ice = 3, tile_death = 4, tile_death_on_top = 5, tile_death_on_bottom = 6};

struct Warp
{
	int direction;
	int connection;
	int id;
};

struct WarpExit
{
	int direction;
	int connection;
	int id;
	
	int x; //Player location where player warps out of
	int y; 

	int lockx;  //Location to display lock icon
	int locky;  

	int warpx;  //map grid location for first block in warp
	int warpy;
	int numblocks;  //number of warp blocks for this warp
};

struct SpawnArea
{
	int left;
	int top;
	int width;
	int height;
	int size;
};

class IO_Block;

class CMap
{
	public:
		void loadTileSet(const char *tilesetfile, const char *tilesetbmp);
		void saveTileSet(const char *tilesetfile);
		void clearTileSet();

		void clearMap();
		void loadMap(const char *file);
		void saveMap(const char *file);

		void convertMap();

		void shift(int xshift, int yshift);

		void predrawbackground(gfxSprite &background, gfxSprite &mapspr);
		void predrawforeground(gfxSprite &foregroundspr);

		void drawfrontlayer();

		void optimize();

		CMap() {}
		~CMap();

		//returns the tiletype at the specific position (map coordinates) of the
		//front most visible tile
		TileType map(int x, int y)
		{
			return mapdatatop[x][y];
		}

		IO_Block * block(int x, int y)
		{
			return blockdata[x][y];
		}

		Warp * warp(int x, int y)
		{
			return &warpdata[x][y];
		}

		int blockat(int x, int y)
		{
			return interactionobjectdata[x][y];
		}

		int spawn(int x, int y)
		{
			return !nospawn[x][y];
		}

		bool isconnectionlocked(int connection) {return warplocked[connection];}
		void lockconnection(int connection) {warplocked[connection] = true;}

		WarpExit * getRandomWarpExit(int connection, int currentID);

		void clearWarpLocks();
		void updateWarpLocks();
		void drawWarpLocks();

		void findspawnpoint(int * x, int * y, int width, int height, bool tilealigned);

		int backgroundID;
		int eyecandyID;

		void WriteInt(int out, FILE * outFile);
		int ReadInt(FILE * inFile);

	private:
		
		void calculatespawnareas(bool usetempblocks);
		
		int			mapdata[MAPWIDTH][MAPHEIGHT][MAPLAYERS];		//0 to TILESETSIZE-1: tiles, TILESETSIZE: no tile
		TileType	mapdatatop[MAPWIDTH][MAPHEIGHT];
		int			interactionobjectdata[MAPWIDTH][MAPHEIGHT];
		IO_Block*   blockdata[MAPWIDTH][MAPHEIGHT];
		bool		nospawn[MAPWIDTH][MAPHEIGHT];

		SpawnArea	spawnareas[MAXSPAWNAREAS];
		int			numspawnareas;
		int			totalspawnsize;

		Warp        warpdata[MAPWIDTH][MAPHEIGHT];
		int			numwarpexits; //number of warp exits
		WarpExit	warpexits[MAXWARPS];
		int			warplocktimer[10];
		bool		warplocked[10];
		int			maxConnection;

		TileType	tileset[TILESETSIZE+1];
		SDL_Surface	*tilesetsurface;
		SDL_Rect	tilebltrect;
		SDL_Rect    bltrect;

		SDL_Rect	drawareas[MAXDRAWAREAS];
		int			numdrawareas;

		void draw(SDL_Surface *targetsurf, int layer);

		friend void drawmap();
		friend void drawlayer(int layer);
		friend void drawcopiedlayer(int layer);

		friend bool copyselectedtiles();
		friend void clearselectedmaptiles();
		friend void pasteselectedtiles(int movex, int movey);

		friend int editor_edit();
		friend int editor_tiles();
		friend int editor_blocks();
		friend int editor_warp();
		friend int save_as();
		friend int load();
		friend void loadInteractionMapObjects();
		
		friend class B_BreakableBlock;
		friend class B_DonutBlock;
};

#endif

