#include "global.h"
#include <math.h>

extern int lastgamewinner;
extern void stopinvincibilityfordeadplayer(int playerID);
extern int lookupteamid(int id);

CPlayer::CPlayer(int iGlobalID, int iLocalID, int iTeamID, int iColorID, int keys[6], gfxSprite * nsprites[PGFX_LAST], SScore *nscore, int * respawn, bool cpu)
{
	globalID = iGlobalID;
	localID = iLocalID;
	teamID = iTeamID;
	colorID = iColorID;

	xpressed = 0;
	tpressed = false;

	//cpu stuff
	is_cpu	= cpu;
	cl		= false;
	cr		= false;
	cj		= false;
	cd		= false;
	ct		= false;
	cp		= false;

	//control keys
	kl = keys[0];
	kr = keys[1];
	kj = keys[2];
	kd = keys[3];
	kt = keys[4];
	kp = keys[5];

	for(int k = 0; k < 4; k++)
		pressdirection[k] = false;

	score = nscore;
	score->score = 0;

	spr = PGFX_JUMPING_R;

	for(int i=0;i<PGFX_LAST;i++)
		sprites[i] = nsprites[i];

	sprswitch = 0;

	spawninvincible = false;
	spawninvincibletimer = 0;
	
	ownedEgg = NULL;
	ownerPlayerID = -1;
	ownerColorID = -1;
	jailed = 0;

	spawntext = 20;  //set it to 20 so there is an immediate text spawned upon winning
	
	powerupused = -1;

	respawncounter = respawn;
	setupNewPlayer();
	*respawncounter = 0;
}



void CPlayer::move()
{
	if(hammertimer > 0)
		hammertimer--;

	if(state == player_wait)
	{
		if(++waittimer >= 62)
		{
			waittimer = 0;
			(*respawncounter)--;
		}

		if(*respawncounter <= 0)
		{
			state = player_spawning;

			if(game_values.spawnstyle == 0)
			{
				eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, ix + (HALFPW) - 16, iy + (HALFPH) - 16, 3, 8));
			}
			else if(game_values.spawnstyle == 1)
			{
				eyecandyback.add(new EC_Door(&spr_spawndoor[colorID], sprites[spr], ix + HALFPW - 16, iy + HALFPH - 16, 1));
			}
		}

		return;
	}
	
	if(state == player_spawning)
	{
		if(game_values.spawnstyle == 0)
		{
			state = player_ready;
		}
		else if(game_values.spawnstyle == 1)
		{
			//Wait for door eyecandy to open to let mario out (20 frames for door to appear and 16 frames to open)
			if(++spawntimer > 36)
			{
				spawntimer = 0;
				state = player_ready;
			}
		}
		else if(game_values.spawnstyle == 2)
		{
			spawnradius -= 2.0f;
			spawnangle += 0.05f;

			if(spawnradius < 0.0f)
				state = player_ready;
		}
	}
	else if(iswarping())
	{
		oldy = iy;
		oldx = ix;
		
		if(state == player_entering_warp_left)
		{
			xi(ix - 1);
			increasewarpcounter(PW + PWOFFSET);
		}
		else if(state == player_entering_warp_right)
		{
			xi(ix + 1);
			increasewarpcounter(PW + PWOFFSET);
		}
		else if(state == player_entering_warp_up)
		{
			yi(iy - 1);
			increasewarpcounter(TILESIZE - PHOFFSET);
		}
		else if(state == player_entering_warp_down)
		{
			yi(iy + 1);
			increasewarpcounter(PH + PHOFFSET);
		}
		else if(state == player_exiting_warp_left)
		{
			xi(ix - 1);
			decreasewarpcounter();
		}
		else if(state == player_exiting_warp_right)
		{
			xi(ix + 1);
			decreasewarpcounter();
		}
		else if(state == player_exiting_warp_up)
		{
			yi(iy - 1);
			decreasewarpcounter();
		}
		else if(state == player_exiting_warp_down)
		{
			yi(iy + 1);
			decreasewarpcounter();
		}


		if(state == player_entering_warp_right || state == player_exiting_warp_left)
		{
			warpOffsetX = 1;
			warpWidthOffsetTrailing = -warpcounter - PWOFFSET;
		}
		else if(state == player_entering_warp_left || state == player_exiting_warp_right)
		{
			warpOffsetX = warpcounter + PWOFFSET;
			warpWidthOffsetLeading = warpcounter + PWOFFSET;
			warpWidthOffsetTrailing = -warpcounter - PWOFFSET;
		}
		else if(state == player_entering_warp_up || state == player_exiting_warp_down)
		{
			warpOffsetY = warpcounter + PHOFFSET - 1;
			warpHeightOffsetLeading = warpcounter + PHOFFSET;
			warpHeightOffsetTrailing = -warpcounter - PHOFFSET;
		}
		else if(state == player_entering_warp_down || state == player_exiting_warp_up)
		{
			warpOffsetY = 1;
			warpHeightOffsetTrailing = -warpcounter - 32 + PH + PHOFFSET;
		}
	}
	else
	{
		if(powerupused > -1)
		{
			powerupradius -= 2.0f;
			powerupangle += 0.05f;

			if(powerupradius < 0.0f)
			{
				if(powerupused == 0)
				{
					game_values.gamemode->player1up(*this);
					ifsoundonplay(extraguysound);
				}
				else if(powerupused == 1)
				{
					powerup = 1;
					ifsoundonplay(collectpowerup);
				}
				else if(powerupused == 2)
				{
					makeinvincible();
				}
				else if(powerupused == 3)
				{
					turnslowdownon();
				}
				else if(powerupused == 4)
				{
					eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, ix + (HALFPW) - 16, iy + (HALFPH) - 16, 3, 8));
					ifsoundonplay(transform);
					bobomb = true;
				}
				else if(powerupused == 5)
				{
					ifsoundonplay(thunder);
					game_values.screenshaketimer = 20;
					game_values.screenshakeplayer = this;
				}
				else if(powerupused == 6)
				{
					game_values.bulletbilltimer = 400;
					spawninvincibletimer = 550;  //A little extra while the screen clears
					spawninvincible = true;
				}
				else if(powerupused == 7)
				{
					powerup = 2;
					ifsoundonplay(collectpowerup);
				}

				powerupused = -1;
			}
		}
	}

	//Animate invincibility
	if(invincible)
	{
		animationtimer++;

		if((animationtimer > 3 && invincibletimer < 480) || animationtimer > 6)
		{
			animationtimer = 0;

			if(++animationstate > 3)
				animationstate = 0;
		}

		if(++invincibletimer > 580)
		{
			animationstate = 0;
			animationtimer = 0;
			invincibletimer = 0;
			invincible = false;
		}
	}

	if(state == player_ready)
	{
		if(spawninvincibletimer > 0)
		{
			spawninvincibletimer--;
			spawninvincible = true;
		}
		else
		{
			spawninvincible = false;
		}
	}

	
	//player is warping or spawning so don't pay attention to controls
	if(state != player_ready)
		return;

	//if(ix != (int)fx)
	//	printf("aaah - %d != %f (%p)\n", ix, fy, this);

	int lrn = 0;	//move left-right-no: -1.. left 0 no 1 ... right

	if(is_cpu)
	{
		//Calculate movement every 4th frame (speed up optimization)
		if(game_values.cputurn == globalID)
			cpu_think();

		if(cr) lrn = 1;
		if(cl) lrn = -1;
	}
	else
	{
		if(!(keystates[kr] && keystates[kl]))
		{
			if(keystates[kr])
				lrn = 1;		
			else if(keystates[kl])
				lrn = -1;
		}
	}

	//Used for warping
	if((keystates[kr] && !is_cpu) || cr)
		pressdirection[1] = true;
	else
		pressdirection[1] = false;

	if((keystates[kl] && !is_cpu) || cl)
		pressdirection[3] = true;
	else
		pressdirection[3] = false;

	//Used for bouncing off of note blocks
	if(superjumptimer > 0)
		superjumptimer--;

	if((keystates[kj] && !is_cpu) || cj)
	{	//jump pressed?
		pressdirection[0] = true;

		if(!inair && superjumptimer == 0 && !lockjump)
		{	//only if on the ground and the jump key was released somewhen after it was pressed the last time

			if((keystates[kd] && !is_cpu) || cd)
			{
				fallthrough = true;
			}
			else
			{
				if((game_values.slowdownon != -1 && game_values.slowdownon != teamID) || jailed > 0)
					vely = -VELSLOWJUMP;
				else if(ABS(velx) > VELMOVING && lrn != 0 && tpressed)
					vely = -VELTURBOJUMP;
				else
					vely = -VELJUMP;

				ifsoundonplay(jump);
			}

			lockjump = true;
		}
		else if(superjumptimer > 0 && !lockjump)
		{
			vely = -VELTURBOJUMP;
			ifsoundonplay(springjump);
			superjumptimer = 0;
			lockjump = true;
		}
	}
	else
	{
		pressdirection[0] = false;
		lockjump = false;		//the jump key is not pressed: the player may jump again if he is on the ground

		if(vely < -VELSTOPJUMP)
			vely = -VELSTOPJUMP;
	}

	if((keystates[kd] && !is_cpu) || cd)
	{
		pressdirection[2] = true;

#ifndef _XBOX
		if(!lockfall && !inair)
		{
			lockfall = true;
			fallthrough = true;
		}
#endif
	}
	else
	{
		pressdirection[2] = false;
		lockfall = false;
	}
	
	//TURBO
	if((keystates[kt] && !is_cpu) || ct)
	{	//turbo pressed
		if(!inair)
		{	//only if on the ground
			tpressed = true;
		}
	}
	else
	{
		tpressed = false;	//the turbo key is not pressed
	}

	//POWERUP RELEASE
	if((keystates[kp] && !is_cpu) || cp)
	{	//powerup release pressed
		if(game_values.gamepowerups[globalID] != -1 /*&& !game_values.gamemode->gameover && powerupused < 0*/)
		{
			powerupused = game_values.gamepowerups[globalID];
			game_values.gamepowerups[globalID] = -1;

			powerupradius = 100.0f;
			powerupangle = (float)(rand()%1000 * 0.00628f);

			ifsoundonplay(storedpowerupsound);
		}
	}
	

	//FIREPOWER
	if((keystates[kt] && !is_cpu) || ct)
	{
		if(!lockfire)
		{
			//If we're a bob-omb, explode
			if(bobomb)
			{
				bobomb = false;
				interactionobjectsfront.add(new OMO_Explosion(&spr_explosion, ix + HALFPW - 96, iy + HALFPH - 64, 2, 4, globalID));
				ifsoundonplay(bobombsound);
			}
			else if(powerup > 0 && projectiles[globalID] < 2) //otherwise, shoot a projectile
			{
				if(powerup == 1 || (powerup == 2 && hammertimer == 0))
				{
					projectiles[globalID]++;
					ifsoundonplay(fireball);

					bool side = false;

					if(xpressed == 1)
						side = true;
					else if(xpressed == -1)
						side = false;
					else
					{
						if((spr & 0x1) == 0)
							side = true;
						else
							side = false;
					}

					if(powerup == 1)
					{
						interactionobjectcollisionitems.add(new MO_Fireball(&spr_fireball, ix + 6, iy, 4, side, 5, globalID, colorID));
					}
					else if(powerup == 2)
					{
						interactionobjectsfront.add(new OMO_Hammer(&spr_hammer, ix + 6, iy, 6, side, 5, globalID, colorID));
						hammertimer = 25;
					}
				}
			}

			lockfire = true;
		}
	}
	else
	{
		lockfire = false;
	}

	if(lrn == 1)
	{
		if(onice)
			velx += VELMOVINGADD/4;
		else
			velx += VELMOVINGADD;		//move right

		xpressed = 1;		//player graphic is facing right
		
		float maxVel = 0;
		if((game_values.slowdownon != -1 && game_values.slowdownon != teamID) || jailed > 0)
			maxVel = VELSLOWMOVING;
		else if(tpressed)
			maxVel = VELTURBOMOVING + (game_values.gamemode->tagged == this ? TAGGEDBOOST : 0.0f);
		else
			maxVel = VELMOVING + (game_values.gamemode->tagged == this ? TAGGEDBOOST : 0.0f);
		
		if(velx > maxVel)
			velx = maxVel;
	}
	else if(lrn == -1)
	{
		if(onice)
			velx -= VELMOVINGADD/4;
		else
			velx -= VELMOVINGADD;		//move left

		xpressed = -1;		//player graphic is facing left

		float maxVel = 0;
		if((game_values.slowdownon != -1 && game_values.slowdownon != teamID) || jailed > 0)
			maxVel = -VELSLOWMOVING;
		else if(tpressed)
			maxVel = -VELTURBOMOVING - (game_values.gamemode->tagged == this ? TAGGEDBOOST : 0.0f);
		else
			maxVel = -VELMOVING - (game_values.gamemode->tagged == this ? TAGGEDBOOST : 0.0f);
		
		if(velx < maxVel)
			velx = maxVel;
		
	}
	else
	{
		xpressed = 0;
		if(velx > 0)
		{
			if(inair)
				velx -= VELAIRFRICTION;
			else if(!onice)
				velx -= VELMOVINGFRICTION;

			if(velx < 0)	//too much fricition - where moving in the other direction
				velx = 0;
		}
		else if(velx < 0)
		{
			if(inair)
				velx += VELAIRFRICTION;
			else if(!onice)
				velx += VELMOVINGFRICTION;

			if(velx > 0)	//too much fricition - where moving in the other direction
				velx = 0;
		}
	}
	

	oldy = iy;
	oldx = ix;
	
	collision_detection_map();

	//Deal with terminal burnup velocity
	if(vely >= MAXVELY && !invincible)
	{
		if(++burnupstarttimer >= 20)
		{
			if(burnupstarttimer == 20)
				ifsoundonplay(burnup);

			if(++burnuptimer > 80)
			{
				ifsoundonplay(deathsound);

				if( !game_values.gamemode->playerkilledself(*this))
				{
					deathAwards();
					jumpDeath();
				}
			}

			eyecandyback.add(new EC_SingleAnimation(&spr_burnup, ix + HALFPW - 16, iy + HALFPH - 16, 5, 4));
		}
	}
	else
	{
		burnuptimer = 0;
		burnupstarttimer = 0;
	}

	//Deal with out of arena timer
	if((iy + PH < -1 || iy + PH <= 1 && vely <= 1.4f) && !invincible)
	{
		if(++outofarenatimer > 60)
		{
			outofarenatimer = 0;

			if(--outofarenadisplaytimer < 0)
			{
				ifsoundonplay(deathsound);

				if( !game_values.gamemode->playerkilledself(*this))
				{
					deathAwards();
					jumpDeath();
				}
			}
		}
	}

	//Deal with release from jail timer
	if(jailed > 0)
	{
		if(--jailed <= 0)
		{
			jailed = 0;
			eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, ix + (HALFPW) - 16, iy + (HALFPH) - 16, 3, 8));
			ifsoundonplay(transform);
		}
	}
}

void CPlayer::increasewarpcounter(int iGoal)
{
	if(++warpcounter > iGoal)
	{
		warpcounter = iGoal;
		chooseWarpExit();

		warpOffsetX = 0;
		warpOffsetY = 0;
		warpWidthOffsetLeading = 0;
		warpHeightOffsetLeading = 0;
		warpWidthOffsetTrailing = 0;
		warpHeightOffsetTrailing = 0;
	}
}

void CPlayer::decreasewarpcounter()
{
	if(--warpcounter < 0)
	{
		warpcounter = 0;
		state = player_ready;

		warpOffsetX = 0;
		warpOffsetY = 0;
		warpWidthOffsetLeading = 0;
		warpHeightOffsetLeading = 0;
		warpWidthOffsetTrailing = 0;
		warpHeightOffsetTrailing = 0;
	}
}

void CPlayer::chooseWarpExit()
{
	WarpExit * exit = map.getRandomWarpExit(warpconnection, warpid);
	xi(exit->x);
	yi(exit->y);
	oldx = ix;
	oldy = iy;

	lockjump = false;
	if(exit->direction == 0)
	{
		state = player_exiting_warp_up;
		warpcounter = PH + PHOFFSET;
		velx = 0.0f;
		vely = -4.0f;
	}
	else if(exit->direction == 1)
	{
		state = player_exiting_warp_right;
		warpcounter = PW + PWOFFSET;
		velx = 1.0f;
		vely = 1.0f;
	}
	else if(exit->direction == 2)
	{
		state = player_exiting_warp_down;
		warpcounter = TILESIZE - PHOFFSET;
		velx = 0.0f;
		vely = 1.1f;
		inair = true;
	}
	else if(exit->direction == 3)
	{
		state = player_exiting_warp_left;
		warpcounter = PW + PWOFFSET;
		velx = -1.0f;
		vely = 1.0f;
	}

	if(game_values.spawninvincibility)
	{
		if(!spawninvincible || spawninvincibletimer < 60)
		{
			spawninvincibletimer = 60;
			spawninvincible = true;
		}
	}
}


//Search and return in the list_players[] array the player, which is the nearest. 

CInteractionObject * CPlayer::getNearestGoal(int * distance)
{
	CInteractionObject * target = NULL;
	int		distance_player_pow2;
	int		nearest = 640000; //640*640+480*480
	int		tx, ty;

	for(int i = 0; i < interactionobjectsplayer.list_end; i++)
	{
		//Ignore yoshi if player doens't have egg
		if(interactionobjectsplayer.list[i]->getObjectType() == object_yoshi && !ownedEgg)
			continue;

		//Ignore egg if player already is carrying it
		if(interactionobjectsplayer.list[i]->getObjectType() == object_egg && ownedEgg)
			continue;

		//Calculate normal screen
		tx = interactionobjectsplayer.list[i]->ix - ix;
		ty = interactionobjectsplayer.list[i]->iy - iy;
		distance_player_pow2 = tx*tx + ty*ty;    // a^2 = b^2 + c^2 :)

		if (distance_player_pow2 < nearest)
		{
			target = interactionobjectsplayer.list[i];
			nearest = distance_player_pow2;
		}
	}

	for(int i = 0; i < interactionobjectcollisionitems.list_end; i++)
	{
		//Ignore fireballs and goombas
		if(interactionobjectcollisionitems.list[i]->getObjectType() == object_moving)
		{
			MovingObjectType type = ((IO_MovingObject*)interactionobjectcollisionitems.list[i])->getMovingObjectType();

			if(type == movingobject_fireball || type == movingobject_goomba || type == movingobject_poisonpowerup)
				continue;
		}
		
		//Ignore areas that the player already owns
		if(interactionobjectcollisionitems.list[i]->getObjectType() == object_area &&
			((OMO_Area*)interactionobjectcollisionitems.list[i])->getColorID() == colorID)
			continue;


		//Calculate normal screen
		tx = interactionobjectcollisionitems.list[i]->ix - ix;
		ty = interactionobjectcollisionitems.list[i]->iy - iy;
		distance_player_pow2 = tx*tx + ty*ty;    // a^2 = b^2 + c^2 :)

		if (distance_player_pow2 < nearest)
		{
			target = interactionobjectcollisionitems.list[i];
			nearest = distance_player_pow2;
		}
	}

	for(int i = 0; i < interactionobjectsfront.list_end; i++)
	{
		if(interactionobjectsfront.list[i]->getObjectType() == object_race_goal)
		{
			if(game_values.gamemode->getgamemode() != game_mode_race)
				continue;

			if(((OMO_RaceGoal*)interactionobjectsfront.list[i])->isTagged(teamID) != -1 || 
				(((OMO_RaceGoal*)interactionobjectsfront.list[i])->getGoalID() != ((CGM_Race*)game_values.gamemode)->getNextGoal(teamID)))
				continue;
		}
		else
		{
			continue;
		}

		//Calculate normal screen
		tx = interactionobjectsfront.list[i]->ix - ix;
		ty = interactionobjectsfront.list[i]->iy - iy;
		distance_player_pow2 = tx*tx + ty*ty;    // a^2 = b^2 + c^2 :)

		if (distance_player_pow2 < nearest)
		{
			target = interactionobjectsfront.list[i];
			nearest = distance_player_pow2;
		}
	}
	
	*distance = nearest;
	return target;
}

//detect thwomps, explosions and fireballs so cpu can run away from them
CInteractionObject * CPlayer::getNearestThreat(int * distance)
{
	CInteractionObject * target = NULL;
	int		distance_player_pow2;
	int		nearest = 640000; //640*640+480*480
	int		tx, ty;

	if(!invincible)
	{
		for(int i = 0; i < interactionobjectsfront.list_end; i++)
		{
			if(interactionobjectsfront.list[i]->getObjectType() == object_race_goal)
				continue;

			//Calculate normal screen using more accurate measurement because thwomps and explosions are big
			tx = interactionobjectsfront.list[i]->ix + (interactionobjectsfront.list[i]->collisionWidth >> 1) - ix - HALFPW;
			ty = interactionobjectsfront.list[i]->iy + (interactionobjectsfront.list[i]->collisionHeight >> 1) - iy - HALFPH;
			distance_player_pow2 = tx*tx + ty*ty;    // a^2 = b^2 + c^2 :)

			if (distance_player_pow2 < nearest)
			{
				target = interactionobjectsfront.list[i];
				nearest = distance_player_pow2;
			}
		}

		for(int i = 0; i < interactionobjectcollisionitems.list_end; i++)
		{
			//Only pay attention to fireballs that are from other players
			if(interactionobjectcollisionitems.list[i]->getObjectType() == object_moving)
			{
				MovingObjectType type = ((IO_MovingObject*)interactionobjectcollisionitems.list[i])->getMovingObjectType();

				if( (type == movingobject_fireball &&
					((MO_Fireball*)interactionobjectcollisionitems.list[i])->playerID != globalID) ||
					(type == movingobject_hammer &&
					((OMO_Hammer*)interactionobjectcollisionitems.list[i])->playerID != globalID) ||
					type == movingobject_poisonpowerup)
				{
					//Calculate normal screen
					tx = interactionobjectcollisionitems.list[i]->ix - ix;
					ty = interactionobjectcollisionitems.list[i]->iy - iy;
					distance_player_pow2 = tx*tx + ty*ty;    // a^2 = b^2 + c^2 :)

					if (distance_player_pow2 < nearest)
					{
						target = interactionobjectcollisionitems.list[i];
						nearest = distance_player_pow2;
					}
				}
			}
		}
	}

	*distance = nearest;
	return target;
}



CPlayer *CPlayer::getNearestPlayer(int * distance)
{
	CPlayer *target = NULL;
	int		distance_player_pow2;
	int		nearest = 640000; //640*640+480*480
	int		tx, ty;

	for(int i = 0; i < list_players_cnt; i++)
	{
		if(list_players[i] == this || (list_players[i]->teamID == teamID && !game_values.friendlyfire))
			continue;
		
		//If there is a chicken, only focus on stomping him
		if(game_values.gamemode->chicken && game_values.gamemode->chicken != this && game_values.gamemode->chicken != list_players[i])
			continue;

		//Calculate normal screen
		tx = list_players[i]->ix - ix;
		ty = list_players[i]->iy - iy;
		distance_player_pow2 = tx*tx + ty*ty;    // a^2 = b^2 + c^2 :)

		if (distance_player_pow2 < nearest)
		{
			target = list_players[i];
			nearest = distance_player_pow2;
		}

		/*
		//Calculate across the screen edges - Can't do this yet because cpu_think()
		//doesn't try to run across screen edges
		if(tx < 0)
			tx += 640;
		else
			tx -= 640;
		
		distance_player_pow2 = tx*tx + ty*ty;    // a^2 = b^2 + c^2 :)

		if (distance_player_pow2 < nearest)
		{
			target = list_players[i];
			nearest = distance_player_pow2;
		}
		*/
	}
	*distance = nearest;
	return target;
}

CPlayer * CPlayer::getNearestTeammate(int * distance)
{
	CPlayer * target = NULL;
	int		distance_player_pow2;
	int		nearest = 640000; //640*640+480*480
	int		tx, ty;

	if(game_values.gamemode->gamemode == game_mode_jail)
	{
		for(int i = 0; i < list_players_cnt; i++)
		{
			//Find players in jail on own team
			if(i != localID && list_players[i]->jailed > 0 && list_players[i]->teamID == teamID)
			{
				//Calculate normal screen
				tx = list_players[i]->ix - ix;
				ty = list_players[i]->iy - iy;
				distance_player_pow2 = tx*tx + ty*ty;    // a^2 = b^2 + c^2 :)

				if (distance_player_pow2 < nearest)
				{
					target = list_players[i];
					nearest = distance_player_pow2;
				}
			}
		}
	}

	*distance = nearest;
	return target;
}

CInteractionObject * CPlayer::getNearestStomp(int * distance)
{
	CInteractionObject * target = NULL;
	int		distance_player_pow2;
	int		nearest = 640000; //640*640+480*480
	int		tx, ty;

	for(int i = 0; i < interactionobjectcollisionitems.list_end; i++)
	{
		//Only pay attention to goombas
		if(interactionobjectcollisionitems.list[i]->getObjectType() == object_moving)
		{
			IO_MovingObject * object = ((IO_MovingObject*)interactionobjectcollisionitems.list[i]);

			if(object->getMovingObjectType() == movingobject_goomba && object->GetState() == 1)
			{
				//Calculate normal screen
				tx = interactionobjectcollisionitems.list[i]->ix - ix;
				ty = interactionobjectcollisionitems.list[i]->iy - iy;
				distance_player_pow2 = tx*tx + ty*ty;    // a^2 = b^2 + c^2 :)

				if (distance_player_pow2 < nearest)
				{
					target = interactionobjectcollisionitems.list[i];
					nearest = distance_player_pow2;
				}
			}
		}
	}

	*distance = nearest;
	return target;
}



/*
if is_cpu is true, cpu_think will be used. Fist he search the nearest player, than if "target" is lower or at the same y-axis-level, he runs 
to the target; if the target is a little higher, he stands still and wait. If the target is very high, he jumps to get "maybe" higher 
than the target. If target is very near (on x-axis), he jumps. If target is very near (x-axis) and higher(y-axis) he "runs away".
*/
void CPlayer::cpu_think()
{
	int playerdistance = 0;
	CPlayer *target = getNearestPlayer(&playerdistance);		//search the nearest Player
	
	int goaldistance = 0;
	CInteractionObject *goal = getNearestGoal(&goaldistance);  //search for the nearest goal

	int threatdistance = 0;
	CInteractionObject *threat = getNearestThreat(&threatdistance);  //search for the nearest goal

	int teammatedistance = 0;
	CPlayer *teammate = getNearestTeammate(&teammatedistance);  //search for the nearest goal

	int stompdistance = 0;
	CInteractionObject *stomp = getNearestStomp(&stompdistance);		//search the nearest stomp goal

	int actionType = 0;
	if(threat && threatdistance < 14400)
	{
		actionType = 2;
	}
	else if(stomp && stompdistance < 14400)
	{
		actionType = 4;
	}
	else if(goal && teammate)
	{
		if(goaldistance < teammatedistance)
			actionType = 1;
		else
			actionType = 3;
	}
	else if(teammate)
	{
		actionType = 3;
	}
	else if(goal)
	{
		actionType = 1;
	}
	else
	{
		actionType = 0;
	}
	
	ct = false;
	//return if no players, goals, or threats available
	if((actionType != 0 || target) && (actionType != 1 || goal) && (actionType != 2 || threat) && 
		(actionType != 3 || teammate) && (actionType != 4 || stomp) && fall_danger == 0)
	{

		//Use turbo
		if(bobomb)
		{
			if(playerdistance <= 4096)
				ct = true;
		}
		else
		{
			if(powerup == 0)
				ct = true;
			else if(!(rand() % 20))
				ct = true;
		}

		if(actionType == 0)
		{
			//-----------    what will computer do --------------
			if(target->ix > ix)
			{  //if true target is on the right side
				if((target->iy >= iy && !target->invincible && !target->bobomb && 
					game_values.gamemode->tagged != target && target->teamID != teamID) || invincible || bobomb)
				{					//if true target is lower or at the same level, run right
					cl=false;
					cr=true;
				}
				else
				{
					if(target->ix - ix < 90 &&		//else if target is near but higher, run away (left)
						target->ix - ix > -90)
					{
						cl=true;
						cr=false;
					}
					else
					{							//else target is not very near, stand still, do nothing
						cl=false;
						cr=false;
					}
				}
			}
			else if(target->ix < ix)
			{  //if true target is on the left side,  (rest equal right)
				if((target->iy >= iy && !target->invincible && !target->bobomb &&
					game_values.gamemode->tagged != target && target->teamID != teamID) || invincible || bobomb)
				{
					cl=true;
					cr=false;
				}
				else
				{
					if(target->ix - ix < 90 && 
						target->ix - ix > -90)
					{
						cl=false;
						cr=true;
					}
					else
					{
						cl=false;
						cr=false;
					}
				}
			}
			else 
			{
				cr=false;
				cl=false;
			}

			if (target->iy <= iy &&					//jump if target is higher or at the same level and
				target->ix - ix < 45 &&				//target is very near
				target->ix - ix > -45)
			{				//or if target is high
				cj=true;
				cd=false;
			}
			else if (target->iy > iy &&			//try to down jump if target is below us
				target->ix - ix < 45 &&				
				target->ix - ix > -45 && rand() % 2)
			{				//or if target is high
				cd=true;
				cj=true;
			}
			else if(iy - target->iy > 70)  //If the target is significatly below us, then jump
			{
				cj=true;
				cd=false;
			}
			else
			{
				cj=false;
				cd=false;

				if(!(rand() % 60))
					cj=true;
			}
		}
		else if(actionType == 1)
		{
			if(goal->ix > ix)
			{
				cl=false;
				cr=true;
			}
			else if(goal->ix < ix)
			{
				cl=true;
				cr=false;
			}
			else 
			{
				cr=false;
				cl=false;
			}

			if (goal->iy <= iy && goal->ix - ix < 45 && goal->ix - ix > -45)
			{
				cj=true;
				cd=false;
			}
			else if (goal->iy > iy && goal->ix - ix < 45 && goal->ix - ix > -45)
			{
				cd=true;
				cj=true;
			}
			else if(iy - goal->iy > 70)
			{
				cj=true;
				cd=false;
			}
			else
			{
				cj=false;
				cd=false;

				if(!(rand() % 60))
					cj=true;
			}
		}
		else if(actionType == 2)
		{
			//If threat, always use turbo
			ct = true;

			if(threat->ix > ix)
			{  //if true, threat is on the right side
				cl=true;
				cr=false;
			}
			else
			{  //if true, threat is on the left side,  (rest equal right)
				cl=false;
				cr=true;
			}
			
			if (threat->iy <= iy && threat->ix - ix < 60 && threat->ix - ix > -60)
			{
				cd=true;
				cj=true;
			}
			else if (threat->iy > iy && threat->ix - ix < 60 && threat->ix - ix > -60)
			{
				cd=false;
				cj=true;
			}
			else
			{
				cj=false;
				cd=false;
			}
		}
		else if(actionType == 3)
		{
			if(teammate->ix > ix)
			{
				cl=false;
				cr=true;
			}
			else if(teammate->ix < ix)
			{
				cl=true;
				cr=false;
			}
			else 
			{
				cr=false;
				cl=false;
			}

			if (teammate->iy <= iy && teammate->ix - ix < 45 && teammate->ix - ix > -45)
			{
				cj=true;
				cd=false;
			}
			else if (teammate->iy > iy && teammate->ix - ix < 45 && teammate->ix - ix > -45)
			{
				cd=true;
				cj=true;
			}
			else if(iy - teammate->iy > 70)
			{
				cj=true;
				cd=false;
			}
			else
			{
				cj=false;
				cd=false;

				if(!(rand() % 60))
					cj=true;
			}
		}
		else if(actionType == 4)
		{
			if(stomp->ix > ix)
			{  
				if(stomp->iy >= iy || invincible || bobomb)
				{					//if true target is lower or at the same level, run right
					cl=false;
					cr=true;
				}
				else
				{
					if(stomp->ix - ix < 90 && stomp->ix - ix > -90)
					{
						cl=true;
						cr=false;
					}
					else
					{							//else target is not very near, stand still, do nothing
						cl=false;
						cr=false;
					}
				}
			}
			else if(stomp->ix < ix)
			{  
				if(stomp->iy >= iy || invincible || bobomb)
				{
					cl=true;
					cr=false;
				}
				else
				{
					if(stomp->ix - ix < 90 && stomp->ix - ix > -90)
					{
						cl=false;
						cr=true;
					}
					else
					{
						cl=false;
						cr=false;
					}
				}
			}
			else 
			{
				cr=false;
				cl=false;
			}

			if (stomp->iy <= iy && stomp->ix - ix < 45 && stomp->ix - ix > -45)
			{
				cj=true;
				cd=false;
			}
			else if(stomp->iy > iy && stomp->ix - ix < 45 && stomp->ix - ix > -45)
			{
				cd=true;
				cj=true;
			}
			else if(iy - stomp->iy > 70)
			{
				cj=true;
				cd=false;
			}
			else
			{
				cj=false;
				cd=false;

				if(!(rand() % 60))
					cj=true;
			}
		}
	}
	
	if(inair && vely > 0)
		cj=false;

	//Make sure we don't jump over something that could kill us
	int iDeathY = iy / TILESIZE;
	int iDeathX1 = ix / TILESIZE;
	int iDeathX2 = (ix+PW)/TILESIZE;
	
	if(ix < 0)
		iDeathX1 = (ix + 640) / TILESIZE;

	if(iDeathY < 0)
		iDeathY = 0;

	//int depth = -1;
	while(iDeathY < MAPHEIGHT)
	{
		TileType ttLeftTile = map.map(iDeathX1, iDeathY);
		TileType ttRightTile = map.map(iDeathX2, iDeathY);

		if((ttLeftTile == tile_solid || ttLeftTile == tile_solid_on_top || ttLeftTile == tile_death_on_bottom || ttLeftTile == tile_ice || map.block(iDeathX1, iDeathY)) &&
			(ttRightTile == tile_solid || ttRightTile == tile_solid_on_top || ttRightTile == tile_death_on_bottom || ttRightTile == tile_ice || map.block(iDeathX2, iDeathY)))
		{
			fall_danger = 0;
			break;
		}
		else if(ttLeftTile == tile_death_on_top || ttLeftTile == tile_death ||
			ttRightTile == tile_death_on_top || ttRightTile == tile_death)
		{
			//depth = iDeathY - (iy / TILESIZE);
			if(fall_danger == 0)
				fall_danger = (cr ? -1 : 1);
			break;
		}

		iDeathY++;
	}

	//There is a death tile below us so move to the side that is safest
	if(fall_danger < 0)
	{
		cr = true;
		cl = false;
		cj = true;
		ct = true;
		cd = false;
	}
	else if(fall_danger > 0)
	{
		cr = false;
		cl = true;
		cj = true;
		ct = true;
		cd = false;
	}

	//Make sure we don't jump up into something that can kill us
	iDeathY = iy / TILESIZE;

	if(iDeathY < 0)
		iDeathY = 0;
	
	int heightlimit = 3;
	while(iDeathY >= 0 && heightlimit > 0)
	{
		TileType ttLeftTile = map.map(iDeathX1, iDeathY);
		TileType ttRightTile = map.map(iDeathX2, iDeathY);

		if(ttLeftTile == tile_solid || ttLeftTile == tile_death_on_top || ttLeftTile == tile_ice ||
			ttRightTile == tile_solid || ttRightTile == tile_death_on_top || ttRightTile == tile_ice ||
			map.block(iDeathX1, iDeathY) || map.block(iDeathX2, iDeathY))
		{
			break;
		}
		else if(ttLeftTile == tile_death_on_bottom || ttLeftTile == tile_death ||
			ttRightTile == tile_death_on_bottom || ttRightTile == tile_death)
		{
			cj = false;
			break;
		}

		iDeathY--;
		heightlimit--;
	}

	cp = false;
	if(game_values.gamepowerups[globalID] > -1)
	{
		if(game_values.gamepowerups[globalID] == 0 || game_values.gamepowerups[globalID] == 3 ||
			game_values.gamepowerups[globalID] == 5 || game_values.gamepowerups[globalID] == 6)
		{
			cp = true;
		}
		else if((game_values.gamepowerups[globalID] == 1 && powerup == 0) ||
				(game_values.gamepowerups[globalID] == 7 && powerup == 0) ||
				(game_values.gamepowerups[globalID] == 2 && !invincible) ||
				(game_values.gamepowerups[globalID] == 4 && !bobomb))
		{
			cp = true;
		}
	}
}




void CPlayer::die()
{
	gfxSprite * corpseSprite = sprites[PGFX_DEAD];

	if(diedas == 1)
		corpseSprite = spr_chocobo[colorID][PGFX_DEAD];
	else if(diedas == 2)
		corpseSprite = spr_bobomb[colorID][PGFX_DEAD];
	
	eyecandyback.add(new EC_Corpse(corpseSprite, (float)(ix - PWOFFSET), (float)(iy+PH-32)));

	if(game_values.screencrunch)
		y_shake += CRUNCHAMOUNT;
	
	setupNewPlayer();
}

void CPlayer::jumpDeath()
{
	gfxSprite * corpseSprite = sprites[PGFX_DEADFLYING];

	if(diedas == 1)
		corpseSprite = spr_chocobo[colorID][PGFX_DEADFLYING];
	else if(diedas == 2)
		corpseSprite = spr_bobomb[colorID][PGFX_DEADFLYING];

	eyecandyfront.add(new EC_FallingObject(corpseSprite, ix + HALFPW - 16, iy+PH-32, -VELJUMP*1.1f));
	
	if(game_values.screencrunch)
		y_shake += CRUNCHAMOUNT;

	setupNewPlayer();
}

void CPlayer::setupNewPlayer()
{
	findspawnpoint();
	
	velx = 0.0f;
	
	if(game_values.spawnstyle == 1)
		vely = 0.0f;
	else
		vely = -(VELJUMP / 2);	//make the player jump up on respawn

	oldx = ix;
	oldy = iy;

	inair = true;
	onice = false;
	lockjump = true;
	superjumptimer = 0;
	powerup	= 0;
	bobomb = false;
	hammertimer = 0;

	invincible = false;
	invincibletimer = 0;

	animationstate = 0;
	animationtimer = 0;

	killsinrow = 0;
	killsinrowinair = 0;
	awardangle = 0.0f;

	frictionslidetimer = 0;
	spawnangle = (float)(rand()%1000 * 0.00628f);  //random new spawn angle
	spawnradius = 100.0f;
	spawntimer = 0;
	waittimer = 0;
	*respawncounter = game_values.respawn;
	state = player_wait;

	fallthrough = false;
	diedas = 0;

	burnuptimer = 0;
	burnupstarttimer = 0;

	outofarenatimer = 0;
	outofarenadisplaytimer = 4;

	warpcounter = 0;

	warpOffsetX = 0;
	warpOffsetY = 0;
	warpWidthOffsetLeading = 0;
	warpHeightOffsetLeading = 0;
	warpWidthOffsetTrailing = 0;
	warpHeightOffsetTrailing = 0;

	powerupused = -1;

	if(game_values.gamemode->getgamemode() == game_mode_thwomp || game_values.spawninvincibility)
	{
		spawninvincibletimer = 60;
		spawninvincible = true;
	}

	fall_danger = 0;
	cl = false;
	cr = false;
	cd = false;
	ct = false;
	cj = false;
}

//very slow and stupid brute force method (tm)
void CPlayer::findspawnpoint()
{
	map.findspawnpoint(&ix, &iy, PW, PH, false);
	fx = (float)ix;
	fy = (float)iy;
}



void CPlayer::spawnText(char * szText)
{
	if(++spawntext >= 20)
	{
		eyecandyfront.add(new EC_GravText(&font[2], ix + HALFPW, iy, szText, -VELJUMP));
		spawntext = 0;	//spawn text every 20 frames
	}
}



void CPlayer::spawnGameOverMario()
{
	gfxSprite * corpseSprite = sprites[PGFX_DEADFLYING];

	if(diedas == 1 || game_values.gamemode->chicken == this)
		corpseSprite = spr_chocobo[colorID][PGFX_DEADFLYING];
	else if(diedas == 2 || bobomb)
		corpseSprite = spr_bobomb[colorID][PGFX_DEADFLYING];

	eyecandyfront.add(new EC_FallingObject(corpseSprite, ix, iy+PH-32, -VELJUMP*1.1f));

	if(ownedEgg)
	{
		ownedEgg->owner = NULL;
		ownedEgg = NULL;
	}
}

void CPlayer::bouncejump()
{
	if((keystates[kj] && !is_cpu) || cj)
	{
		lockjump = true;
		vely = -VELJUMP;
	}
	else
	{
		vely = -VELJUMP / 2;	//jump a little off of collision object
	}
}



bool CPlayer::isstomping(CPlayer &o)
{
	//printf("ID: %d  Stomp old: %d <= %d  new: %d >= %d  vely: %.2f\n", globalID, oldy + PH, o.oldy, iy + PH, o.iy, vely);

	if(oldy + PH <= o.oldy && iy + PH >= o.iy)
	{
		//don't reposition if player is warping when he kills the other player
		if(state == player_ready)
		{
			yi(o.iy - PH);		//set new position to top of other player
			collision_detection_checktop();
		}
		
		bool fKillPotential = false;
		if(vely > 1.0f)
			fKillPotential = true;

		bouncejump();

		if(fKillPotential)
		{
			killplayer(*this, o, death_style_squish, kill_style_stomp);
		}
		else
		{
			if(game_values.gamemode->tagged == &o && isready() && o.isready())
			{
				game_values.gamemode->tagged = this;
				o.spawninvincible = true;
				o.spawninvincibletimer = 60;
				eyecandyfront.add(new EC_GravText(&font[2], game_values.gamemode->tagged->ix + (HALFPW), game_values.gamemode->tagged->iy + PH, "Tagged!", -VELJUMP*1.5));
				eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, game_values.gamemode->tagged->ix + (HALFPW) - 16, game_values.gamemode->tagged->iy + (HALFPH) - 16, 3, 8));
				ifsoundonplay(transform);
			}
			else if(game_values.gamemode->tagged == this && isready() && o.isready())
			{
				game_values.gamemode->tagged = &o;
				spawninvincibletimer = 60;
				spawninvincible = true;
				eyecandyfront.add(new EC_GravText(&font[2], game_values.gamemode->tagged->ix + (HALFPW), game_values.gamemode->tagged->iy + PH, "Tagged!", -VELJUMP*1.5));
				eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, game_values.gamemode->tagged->ix + (HALFPW) - 16, game_values.gamemode->tagged->iy + (HALFPH) - 16, 3, 8));
				ifsoundonplay(transform);
			}
		}

		return true;
	}

	return false;
}

void CPlayer::explodeawards()
{
	if(killsinrow < MINAWARDSNEEDED)
		return;
		
	ifsoundonplay(cannon);

	int numawards = (killsinrow > MAXAWARDS ? MAXAWARDS : killsinrow);
	float addangle = TWO_PI / (float)numawards;

	for(int k = 0; k < numawards; k++)
	{
		float angle = (float)k * addangle + awardangle;
		float cosangle = cos(angle);
		float sinangle = sin(angle);

		int awardx = ix + HALFPW - 8 + (int)(30.0f * cosangle);
		int awardy = iy + HALFPH - 8 + (int)(30.0f * sinangle);
		
		float awardvelx = 7.0f * cosangle;
		float awardvely = 7.0f * sinangle;
			
		eyecandyfront.add(new EC_ExplodingAward(&spr_awardsolid, awardx, awardy, awardvelx, awardvely, 30, awards[k]));
	}
}

void CPlayer::addswirlingawards()
{
	if(killsinrow < MINAWARDSNEEDED)
		return;
		
	ifsoundonplay(cannon);

	int numawards = (killsinrow > MAXAWARDS ? MAXAWARDS : killsinrow);
	float addangle = TWO_PI / (float)numawards;

	for(int k = 0; k < numawards; k++)
	{
		float angle = (float)k * addangle + awardangle;
		eyecandyfront.add(new EC_SwirlingAward(&spr_awardkillsinrow[colorID], ix + HALFPW - 8, iy + HALFPH - 8, angle, 30.0f, 0.05f, 60, numawards - 1));
	}
}

void CPlayer::addrocketawards()
{
	if(killsinrow < MINAWARDSNEEDED)
		return;
		
	ifsoundonplay(cannon);

	int numawards = (killsinrow > MAXAWARDS ? MAXAWARDS : killsinrow);
	
	float addangle = QUARTER_PI / 10.0f;
	float startangle = -HALF_PI - ((addangle / 2) * (float)(numawards - 1));
	
	for(int k = 0; k < numawards; k++)
	{
		float angle = (float)k * addangle + startangle;
		float awardvelx = 9.0f * cos(angle);
		float awardvely = 9.0f * sin(angle);
			
		eyecandyfront.add(new EC_RocketAward(&spr_awardkillsinrow[colorID], ix + HALFPW - 8, iy + HALFPH - 8, awardvelx, awardvely, 80, numawards - 1, 10));
	}
}

void CPlayer::deathAwards()
{
	if(game_values.awardstyle == award_style_halo)
		explodeawards();
	else if(game_values.awardstyle == award_style_souls && killsinrow >= MINAWARDSNEEDED)
		eyecandyfront.add(new EC_SoulsAward(&spr_awardsouls, &spr_awardsoulspawn, ix + HALFPW, iy + HALFPH, 60, 9.0f, killsinrow, awards));
}

void killplayer(CPlayer &killer, CPlayer &killed, int deathstyle, int killstyle)
{
	//If this player is already dead, then don't kill him again
	if(killed.state != player_ready)
		return;

	killer.score->score++;

	if(killed.bobomb)
	{
		killed.diedas = 2;
		killer.bobomb = true;
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, killer.ix + (HALFPW) - 16, killer.iy + (HALFPH) - 16, 3, 8));
		ifsoundonplay(transform);
	}

	if(game_values.gamemode->chicken == &killer && killstyle != kill_style_flag)
	{
		ifsoundonplay(chicken);
	}
	else
	{
		if(deathstyle == 0)
			ifsoundonplay(deathsound);
		else
			ifsoundonplay(mip);
	}

	if(killer.teamID != killed.teamID)
		addawardkill(&killer, &killed, killstyle);

	if(game_values.awardstyle != award_style_none)
		killed.deathAwards();
	
	if ( ! game_values.gamemode->playerkilledplayer(killer, killed) )
	{	//true if the player was deleted
		
		//now kill the player (don't call this function earlier because we need the old position, etc.	
		if(deathstyle == 0)
			killed.jumpDeath();
		else
			killed.die();
	}
}

void addawardkill(CPlayer * killer, CPlayer * killed, int killstyle)
{
	killer->killsinrow++;

	if(killer->inair && (killstyle == kill_style_stomp || killstyle == kill_style_goomba || killstyle == kill_style_bulletbill))
		killer->killsinrowinair++;

	//Play announcer
	bool fSoundPlayed = false;
	if(killer->killsinrowinair > 1)
	{
		if(ifsoundonandreadyplay(announcer[9]) > -1)
			fSoundPlayed = true;
	}

	if(killed && killed->killsinrow >= 2 && !fSoundPlayed)
	{
		if(ifsoundonandreadyplay(announcer[10]) > -1)
			fSoundPlayed = true;
	}

	int awardIndex = 0;
	if(killer->killsinrow >= 2)
	{	
		awardIndex = (killer->killsinrow - 2) >= PAWARD_LAST ? PAWARD_LAST -1 : (killer->killsinrow - 2);
		
		if(!fSoundPlayed)
		{
			if(ifsoundonandreadyplay(announcer[awardIndex]) > -1)
				fSoundPlayed = true;
		}
	}

	//Add eyecandy
	if(game_values.awardstyle != award_style_none)
	{
		if(game_values.awardstyle == award_style_halo)
			killer->awards[(killer->killsinrow - 1) % MAXAWARDS] = killstyle;
		else if(game_values.awardstyle == award_style_souls)
		{
			if(killed)
				killer->awards[(killer->killsinrow - 1) % MAXAWARDS] = killed->colorID;
			else
				killer->awards[(killer->killsinrow - 1) % MAXAWARDS] = 4; //soul id for goomba
		}
	
		if(game_values.awardstyle == award_style_swirl)
			killer->addswirlingawards();
		else if(game_values.awardstyle == award_style_fireworks)
			killer->addrocketawards();
		
		if(killer->killsinrowinair > 1)
		{
			float angle = 0.0f; 
			for(int k = 0; k < 15; k++)
			{
				float vel = 7.0f + ((k % 2) * 5.0f);
				float awardvelx = vel * cos(angle);
				float awardvely = vel * sin(angle);
				
				eyecandyfront.add(new EC_BrokenBlock(&spr_bonus[killer->colorID], killer->ix + HALFPW - 8, killer->iy + HALFPH - 8, awardvelx, awardvely));
				angle -= (float)PI / 14;
			}
		}

		//if we have enough kills in a row -> spawn an award	

		if(game_values.awardstyle == award_style_text)
		{
			if(killer->killsinrow >= 2)
			{	
				char	text[128];				//text to show
				sprintf(text, "%d - %s", killer->killsinrow, awards[awardIndex].name);

				//now add the eyecandy
				eyecandyfront.add(new EC_GravText(awards[awardIndex].font, killer->ix + HALFPW, killer->iy+PH, text, -VELJUMP));
			}

			//if we stopped the other players run show another award
			if(killed && killed->killsinrow >= 2)
			{
				int a = (killed->killsinrow - 2) >= PAWARD_LAST ? PAWARD_LAST -1 : (killed->killsinrow - 2);
				char text[128];
				sprintf(text, "%s Stopped!",  awards[a].name);

				eyecandyfront.add(new EC_GravText(awards[a].font, killed->ix + HALFPW, killed->iy+PH, text, -VELJUMP*1.3f));
			}
		}
	}
}

//this is no member of CPlayer, but can only be used with CPlayer
//and it belongs to the context p2p collision detection + response (CPlayer->isstomping, collisionhandler, ...)

//handles a collision between two players (is being called if o1, o2 collide)
void collisionhandler_p2p(CPlayer &o1, CPlayer &o2)
{
	//If teams tag each other
	if(o1.teamID == o2.teamID)
	{
		if(o1.jailed > 0)
		{
			eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, o1.ix + (HALFPW) - 16, o1.iy + (HALFPH) - 16, 3, 8));
			ifsoundonplay(transform);
			o1.jailed = 0;
		}

		if(o2.jailed > 0)
		{
			eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, o2.ix + (HALFPW) - 16, o2.iy + (HALFPH) - 16, 3, 8));
			ifsoundonplay(transform);
			o2.jailed = 0;
		}
		
		//Don't collision detect players on same team if friendly fire is turned off
		if(!game_values.friendlyfire)
			return;
	}

	//--- 1. is player invincible? ---
	if(o1.invincible && !o2.invincible && !o2.spawninvincible)
	{
		killplayer(o1, o2, death_style_jump, kill_style_star);
		return;
	}
	if(!o1.invincible && !o1.spawninvincible && o2.invincible)
	{
		killplayer(o2, o1, death_style_jump, kill_style_star);
		return;
	}
	
	if((o1.invincible && o2.invincible) || o1.spawninvincible || o2.spawninvincible)
		return;

	//--- 2. stompin other player? ---
	if(o1.isstomping(o2))
		return;
	if(o2.isstomping(o1))
		return;

	//--- 3. push back (horizontal) ---
	if(o1.ix < o2.ix)				//o1 is left -> o1 pushback left, o2 pushback right
		_collisionhandler_p2p_pushback(o1, o2);
	else
		_collisionhandler_p2p_pushback(o2, o1);
}

//handles a collision between a player and an object
void collisionhandler_p2o(CPlayer &o1, CInteractionObject &o2)
{
	o2.collide(&o1);
}

//calculates the new positions for both players when they are pushing each other
void _collisionhandler_p2p_pushback(CPlayer &o1, CPlayer &o2)
{
	//o1 is left to o2
	//  |o1||o2|
	//-----------

	if(game_values.gamemode->tagged == &o1 && o1.isready() && o2.isready())
	{
		game_values.gamemode->tagged = &o2;
		o1.spawninvincible = true;
		o1.spawninvincibletimer = 60;
		eyecandyfront.add(new EC_GravText(&font[2], game_values.gamemode->tagged->ix + (HALFPW), game_values.gamemode->tagged->iy + PH, "Tagged!", -VELJUMP*1.5));
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, game_values.gamemode->tagged->ix + (HALFPW) - 16, game_values.gamemode->tagged->iy + (HALFPH) - 16, 3, 8));
		ifsoundonplay(transform);
	}
	else if(game_values.gamemode->tagged == &o2 && o1.isready() && o2.isready())
	{
		game_values.gamemode->tagged = &o1;
		o2.spawninvincible = true;
		o2.spawninvincibletimer = 60;
		eyecandyfront.add(new EC_GravText(&font[2], game_values.gamemode->tagged->ix + (HALFPW), game_values.gamemode->tagged->iy + PH, "Tagged!", -VELJUMP*1.5));
		eyecandyfront.add(new EC_SingleAnimation(&spr_fireballexplosion, game_values.gamemode->tagged->ix + (HALFPW) - 16, game_values.gamemode->tagged->iy + (HALFPH) - 16, 3, 8));
		ifsoundonplay(transform);
	}

	bool overlapcollision = false;
	if(o1.ix + PW < 320 && o2.ix > 320)
		overlapcollision = true;

	if((o1.velx == 0 && !o2.iswarping() && o2.velx != 0) || o1.iswarping())
	{
		if(overlapcollision)
		{//o2 reposition to the right side of o1, o1 stays
			o2.xi(o1.ix - PW + 639);
			o2.collision_detection_checkleft();
		}
		else
		{
			o2.xi(o1.ix + PW + 1);
			o2.collision_detection_checkright();
		}
	}
	else if((o2.velx == 0 && o1.iswarping() && o1.velx != 0) || o2.iswarping())
	{
		if(overlapcollision)
		{//o1 reposition to the left side of o2, o2 stays
			o1.xi(o2.ix + PW - 639);	
			o1.collision_detection_checkright();
		}
		else
		{
			o1.xi(o2.ix - PW - 1);	
			o1.collision_detection_checkleft();
		}
	}
	else if(!o1.iswarping() && !o2.iswarping())
	{	//both objects moving - calculate middle and set both objects
		
		if(overlapcollision)
		{
			int middle = o2.ix - 640 + ((o1.ix + PW) - o2.ix - 640) / 2;		//no ABS needed (o1.x < o2.x -> o1.x+w > o2.x !)
			o1.xi(middle + 1);	//o1 is left
			o2.xi(middle - PW + 639);		//o2 is right

			o1.collision_detection_checkright();
			o2.collision_detection_checkleft();
		}
		else
		{
			int middle = o2.ix + ((o1.ix + PW) - o2.ix) / 2;		//no ABS needed (o1.x < o2.x -> o1.x+w > o2.x !)
			//printf("hlf:%f, o1x:%f, o2x:%f\n", hlf, o1.x, o2.x);
			o1.xi(middle - PW - 1);	//o1 is left
			o2.xi(middle + 1);		//o2 is right

			o1.collision_detection_checkleft();
			o2.collision_detection_checkright();
		}
	}

	float absv1 = 0.0f;
	float absv2 = 0.0f;
	
	if(overlapcollision)
	{
		absv1 = ( o1.velx < 0 ? o1.velx : -1.0f ) * 1.5f;	//o1 is on the left side (only positive velx counts)
		absv2 = ( o2.velx > 0 ? o2.velx : 1.0f ) * 1.5f;	//o2 right (only negative velx counts)
	}
	else
	{
		absv1 = ( o1.velx > 0 ? o1.velx : 1.0f ) * 1.5f;	//o1 is on the left side (only positive velx counts)
		absv2 = ( o2.velx < 0 ? o2.velx : -1.0f ) * 1.5f;	//o2 right (only negative velx counts)
	}

	if(o1.isready())
		o1.velx = CapVelocity(absv2);
	
	if(o2.isready())
		o2.velx = CapVelocity(absv1);
}


void CPlayer::draw()
{
	if(state == player_wait)
	{
		//Draw respawn timer here
		return;
	}

	//Use correct sprite (and animate)
	if(!game_values.pausegame && !game_values.exitinggame)
	{
		if(iswarping())
		{
			if(state == player_entering_warp_left || state == player_entering_warp_right ||
				state == player_exiting_warp_left || state == player_exiting_warp_right ||
				state == player_exiting_warp_up || state == player_exiting_warp_up)
				inair = false;
			else
				inair = true;
		}

		//lockjump is true when we are in the air (even if we fell of an edge)
		if(state == player_spawning)
		{
			if((spr & 0x1) == 0)
				spr = PGFX_JUMPING_R;
			else
				spr = PGFX_JUMPING_L;
		}
		else if(inair)
		{
			frictionslidetimer = 0;

			if(xpressed == 1)
				spr = PGFX_JUMPING_R;
			else if(xpressed == -1)
				spr = PGFX_JUMPING_L;
			else
			{
				if((spr & 0x1) == 0)		// ;) - this is used to determine the direction the player was facing in the last frame
					spr = PGFX_JUMPING_R;	// if the last sprite was facing right spr & 0x1 is 0
				else						// because the last bit of every PGFX_[action]_R is 0
					spr = PGFX_JUMPING_L;	// the last bit of PGFX_[action]_L is 1
			}								// IF YOU CHANGE THE ORDER OF THE PGFX_L/R STUFF THIS WON'T WOR
		}
		else
		{
			if(velx > 0)
			{
				if(xpressed == -1 && state == player_ready)
				{
					spr = PGFX_STOPPING_R;

					if(frictionslidetimer == 0)
						eyecandyfront.add(new EC_SingleAnimation(&spr_frictionsmoke, ix, iy + PH - 12, 4, 4));

					if(++frictionslidetimer >= 4)
						frictionslidetimer = 0;
						
				}
				else
				{
					frictionslidetimer = 0;

					if(xpressed == 0 && onice)
						spr = PGFX_STANDING_R;
					else
					{
						if(sprswitch == 0)
						{
							if(spr == PGFX_STANDING_R)
								spr = PGFX_RUNNING_R;
							else
								spr = PGFX_STANDING_R;

							sprswitch = 4;
						}
						else
							sprswitch--;
					}
				}
			}
			else if(velx < 0)
			{
				if(xpressed == 1 && state == player_ready)
				{
					spr = PGFX_STOPPING_L;

					if(frictionslidetimer == 0)
						eyecandyfront.add(new EC_SingleAnimation(&spr_frictionsmoke, ix + PW - 16, iy + PH - 12, 4, 4));

					if(++frictionslidetimer >= 4)
						frictionslidetimer = 0;
				}
				else
				{
					frictionslidetimer = 0;

					if(xpressed == 0 && onice)
						spr = PGFX_STANDING_L;
					else
					{
						if(sprswitch == 0)
						{
							if(spr == PGFX_STANDING_L)
								spr = PGFX_RUNNING_L;
							else
								spr = PGFX_STANDING_L;

							sprswitch = 4;
						}
						else
							sprswitch--;
					}
					
				}
			}
			else
			{	
				//Last frame of sliding when velocity is 0 but player is still pressing
				//the opposite direction key
				if(xpressed != 0 && oldx != ix && state == player_ready)
				{
					if((xpressed == -1 && oldx > ix) || (xpressed == 1 && oldx < ix))
					{
						if((spr & 0x1) == 0)
							spr = PGFX_STANDING_R;
						else
							spr = PGFX_STANDING_L;
					}
					else
					{
						if((spr & 0x1) == 0)
							spr = PGFX_STOPPING_R;
						else
							spr = PGFX_STOPPING_L;
					}
				}
				else
				{
					//standing
					frictionslidetimer = 0;

					if(xpressed < 0)
						spr = PGFX_STANDING_L;
					else if(xpressed > 0)
						spr = PGFX_STANDING_R;
					else
					{
						if((spr & 0x1) == 0)
							spr = PGFX_STANDING_R;
						else
							spr = PGFX_STANDING_L;
					}
				}
			}
		}
	}

	//Draw player
	if(state == player_spawning)
	{
		//With spawn smoke trails, we don't need to draw the spawning mario
		/*
		sprites[spr]->draw(ix1, iy1, (game_values.gamemode->tagged == this ? 96 : 0), 0, 32, 32);
		sprites[spr]->draw(ix2, iy2, (game_values.gamemode->tagged == this ? 96 : 0), 0, 32, 32);
		sprites[spr]->draw(ix3, iy3, (game_values.gamemode->tagged == this ? 96 : 0), 0, 32, 32);
		sprites[spr]->draw(ix4, iy4, (game_values.gamemode->tagged == this ? 96 : 0), 0, 32, 32);
		*/

		if(game_values.spawnstyle == 2)
		{
			if(++spawntimer > 1)
			{
				spawntimer = 0;

				int ix1 = ix - PWOFFSET + (int)(spawnradius * cos(spawnangle));
				int iy1 = iy - PHOFFSET + (int)(spawnradius * sin(spawnangle));
					
				int ix2 = ix - PWOFFSET + (int)(spawnradius * cos(spawnangle+HALF_PI));
				int iy2 = iy - PHOFFSET + (int)(spawnradius * sin(spawnangle+HALF_PI));

				int ix3 = ix - PWOFFSET + (int)(spawnradius * cos(spawnangle+PI));
				int iy3 = iy - PHOFFSET + (int)(spawnradius * sin(spawnangle+PI));

				int ix4 = ix - PWOFFSET + (int)(spawnradius * cos(spawnangle+THREE_HALF_PI));
				int iy4 = iy - PHOFFSET + (int)(spawnradius * sin(spawnangle+THREE_HALF_PI));

				eyecandyfront.add(new EC_SingleAnimation(&spr_spawnsmoke[colorID], ix1, iy1, 4, 4));
				eyecandyfront.add(new EC_SingleAnimation(&spr_spawnsmoke[colorID], ix2, iy2, 4, 4));
				eyecandyfront.add(new EC_SingleAnimation(&spr_spawnsmoke[colorID], ix3, iy3, 4, 4));
				eyecandyfront.add(new EC_SingleAnimation(&spr_spawnsmoke[colorID], ix4, iy4, 4, 4));
			}
		}
	}
	else
	{
		gfxSprite * drawspr = sprites[spr];
		bool tagged = false;

		if(bobomb) //draw him as bob-omb
		{
			static int bobombsmoketimer = 0;
			drawspr = spr_bobomb[colorID][spr];

			if(++bobombsmoketimer > 2 && (velx != 0.0f || vely != 1.0f) && state == player_ready)
			{
				bobombsmoketimer = 0;
				eyecandyfront.add(new EC_SingleAnimation(&spr_frictionsmoke, ix + HALFPH - 8, iy - PHOFFSET - 8, 4, 4));
			}
		}
		else if(game_values.gamemode->chicken == this) //draw him as chicken
		{
			drawspr = spr_chocobo[colorID][spr];
		}
		else if(game_values.gamemode->tagged == this) //draw him tagged
		{
			tagged = true;
		}

		if(ownerPlayerID > -1)
		{
			if(iswarping())
			{
				//Deal with only drawing partially the owned circle
				int ownedWarpOffsetX = warpOffsetX + (state == player_entering_warp_left || state == player_exiting_warp_right ? 8 : 0);
				int ownedWarpOffsetY = warpOffsetY + (state == player_entering_warp_up || state == player_exiting_warp_down ? 8 : 0);
				int ownedWarpWidthOffsetLeading = warpWidthOffsetLeading + (state == player_entering_warp_left || state == player_exiting_warp_right ? 8 : 0);
				int ownedWarpHeightOffsetLeading = warpHeightOffsetLeading + (state == player_entering_warp_up || state == player_exiting_warp_down ? 8 : 0);
				int ownedWarpWidthOffsetTrailing = warpWidthOffsetTrailing + (state == player_entering_warp_right || state == player_exiting_warp_left ? -8 : 0);
				int ownedWarpHeightOffsetTrailing = warpHeightOffsetTrailing + (state == player_entering_warp_down || state == player_exiting_warp_up ? -8 : 0);

				spr_owned[ownerColorID].draw(ix - PWOFFSET - 8 + ownedWarpOffsetX, iy - PHOFFSET - 8 + ownedWarpOffsetY, ownedWarpWidthOffsetLeading, ownedWarpHeightOffsetLeading, 48 + ownedWarpWidthOffsetTrailing, 48 + ownedWarpHeightOffsetTrailing);

				if(ix - PWOFFSET - 8 < 0)
					spr_owned[ownerColorID].draw(ix - PWOFFSET + 632 + ownedWarpOffsetX, iy - PHOFFSET - 8 + ownedWarpOffsetY, ownedWarpWidthOffsetLeading, ownedWarpHeightOffsetLeading, 48 + ownedWarpWidthOffsetTrailing, 48 + ownedWarpHeightOffsetTrailing);
				else if(ix - PWOFFSET + 40 >= 640)
					spr_owned[ownerColorID].draw(ix - PWOFFSET - 648 + ownedWarpOffsetX, iy - PHOFFSET - 8 + ownedWarpOffsetY, ownedWarpWidthOffsetLeading, ownedWarpHeightOffsetLeading, 48 + ownedWarpWidthOffsetTrailing, 48 + ownedWarpHeightOffsetTrailing);
				
			}
			else
			{
				spr_owned[ownerColorID].draw(ix - PWOFFSET - 8, iy - PHOFFSET - 8);

				if(ix - PWOFFSET - 8 < 0)
					spr_owned[ownerColorID].draw(ix - PWOFFSET + 632, iy - PHOFFSET - 8);
				else if(ix - PWOFFSET + 40 >= 640)
					spr_owned[ownerColorID].draw(ix - PWOFFSET - 648, iy - PHOFFSET - 8);
			}
		}
		
		drawspr->draw(ix - PWOFFSET + warpOffsetX, iy - PHOFFSET + warpOffsetY, (invincible ? animationstate * 32 : (tagged ? 96 : (spawninvincible ? 128 : 0))) + warpWidthOffsetLeading, warpHeightOffsetLeading, 32 + warpWidthOffsetTrailing, 32 + warpHeightOffsetTrailing);

		if(ix < 0)	//if the player is in the -both-sides-of-the-screen- area
			drawspr->draw(640 + ix - PWOFFSET + warpOffsetX, iy - PHOFFSET + warpOffsetY, (invincible ? animationstate * 32 : (tagged ? 96 : (spawninvincible ? 128 : 0))) + warpWidthOffsetLeading, warpHeightOffsetLeading, 32 + warpWidthOffsetTrailing, 32 + warpHeightOffsetTrailing);
	
		
		if(jailed > 0)
		{
			if(iswarping())
			{
				//Deal with only drawing partially the owned circle
				int ownedWarpOffsetX = warpOffsetX + (state == player_entering_warp_left || state == player_exiting_warp_right ? 6 : 0);
				int ownedWarpOffsetY = warpOffsetY + (state == player_entering_warp_up || state == player_exiting_warp_down ? 6 : 0);
				int ownedWarpWidthOffsetLeading = warpWidthOffsetLeading + (state == player_entering_warp_left || state == player_exiting_warp_right ? 6 : 0);
				int ownedWarpHeightOffsetLeading = warpHeightOffsetLeading + (state == player_entering_warp_up || state == player_exiting_warp_down ? 6 : 0);
				int ownedWarpWidthOffsetTrailing = warpWidthOffsetTrailing + (state == player_entering_warp_right || state == player_exiting_warp_left ? -6 : 0);
				int ownedWarpHeightOffsetTrailing = warpHeightOffsetTrailing + (state == player_entering_warp_down || state == player_exiting_warp_up ? -6 : 0);

				spr_jail.draw(ix - PWOFFSET - 6 + ownedWarpOffsetX, iy - PHOFFSET - 6 + ownedWarpOffsetY, ownedWarpWidthOffsetLeading, ownedWarpHeightOffsetLeading, 44 + ownedWarpWidthOffsetTrailing, 44 + ownedWarpHeightOffsetTrailing);

				if(ix - PWOFFSET - 6 < 0)
					spr_jail.draw(ix - PWOFFSET + 634 + ownedWarpOffsetX, iy - PHOFFSET - 6 + ownedWarpOffsetY, ownedWarpWidthOffsetLeading, ownedWarpHeightOffsetLeading, 44 + ownedWarpWidthOffsetTrailing, 44 + ownedWarpHeightOffsetTrailing);
				else if(ix - PWOFFSET + 38 >= 640)
					spr_jail.draw(ix - PWOFFSET - 646 + ownedWarpOffsetX, iy - PHOFFSET - 6 + ownedWarpOffsetY, ownedWarpWidthOffsetLeading, ownedWarpHeightOffsetLeading, 44 + ownedWarpWidthOffsetTrailing, 44 + ownedWarpHeightOffsetTrailing);
				
			}
			else
			{
				spr_jail.draw(ix - PWOFFSET - 6, iy - PHOFFSET - 6);

				if(ix - PWOFFSET - 6 < 0)
					spr_jail.draw(ix - PWOFFSET + 634, iy - PHOFFSET - 6);
				else if(ix - PWOFFSET + 38 >= 640)
					spr_jail.draw(ix - PWOFFSET - 646, iy - PHOFFSET - 6);
			}
		}

		if(game_values.awardstyle == award_style_halo && killsinrow >= MINAWARDSNEEDED && state == player_ready)
		{
			awardangle += 0.02f;

			if(awardangle > TWO_PI)
				awardangle -= TWO_PI;

			int numawards = (killsinrow > MAXAWARDS ? MAXAWARDS : killsinrow);
			float addangle = TWO_PI / (float)numawards;

			int xoffset = ix + HALFPW - 8;
			int yoffset = iy + HALFPH - 8;

			for(int k = 0; k < numawards; k++)
			{
				float angle = (float)k * addangle + awardangle;
				int awardx = xoffset + (int)(30.0f * cos(angle));
				int awardy = yoffset + (int)(30.0f * sin(angle));
				
				spr_award.draw(awardx, awardy, awards[k] * 16, 0, 16, 16);

				if(awardx + 16 >= 640)
					spr_award.draw(awardx - 640, awardy, awards[k] * 16, 0, 16, 16);
				else if(awardx < 0)
					spr_award.draw(awardx + 640, awardy, awards[k] * 16, 0, 16, 16);
			}
		}

		if(powerupused > -1 && state == player_ready)
		{
			int numeyecandy = 8;
			float addangle = TWO_PI / numeyecandy;
			float displayangle = powerupangle;

			for(int k = 0; k < numeyecandy; k++)
			{
				int powerupX = ix + HALFPW - 8 + (int)(powerupradius * cos(displayangle));
				int powerupY = iy + HALFPH - 8 + (int)(powerupradius * sin(displayangle));

				displayangle += addangle;
			
				spr_storedpowerupsmall.draw(powerupX, powerupY, powerupused * 16, 0, 16, 16);
			}
		}

		//Comment in if you want the player that won the last game to wear the crown during battle
		/*
		if(lastgamewinner == globalID)
		{
			int crownoffset = -4;
			if((spr & 0x1) == 0)
				crownoffset = 4;

			spr_crown.draw(ix + HALFPW - 7 + crownoffset, iy - 10);

			if(ix < 0)	//if the player is in the -both-sides-of-the-screen- area
				spr_crown.draw(640-(PW-(ix+PW)%640) + HALFPW - 7 + crownoffset, iy - 10);
		}
		*/
	}
}

void CPlayer::drawarrows()
{
	if(iy + PH < -1 || iy + PH <= 0 && vely <= 1)
	{
		spr_abovearrows.draw(ix - PWOFFSET, 0, colorID * 32, 0, 32, 26);

		if(ix < 0)  //on both sides of screen
			spr_abovearrows.draw(640 + ix - PWOFFSET, 0, colorID * 32, 0, 32, 26);

		/*
		//This code displays the distance the player is above the top of the screen in TILESIZE increments
		int distanceIndex = -((iy + PH) / TILESIZE);
		if(distanceIndex > 9)
			distanceIndex = 9;

		spr_awardkillsinrow[colorID].draw(ix - PWOFFSET + 8, 18, distanceIndex * 16, 0, 16, 16);
		
		if(ix < 0)
			spr_awardkillsinrow[colorID].draw(ix - PWOFFSET + 648, 18, distanceIndex * 16, 0, 16, 16);
		*/

		//This displays the out of arena timer before the player is killed
		spr_awardkillsinrow[colorID].draw(ix - PWOFFSET + 8, 18, outofarenadisplaytimer * 16, 0, 16, 16);
		
		if(ix < 0)
			spr_awardkillsinrow[colorID].draw(ix - PWOFFSET + 648, 18, outofarenadisplaytimer * 16, 0, 16, 16);
	}
}

void CPlayer::collision_detection_map()
{
	xf(fx + velx);
	float fyvely = fy + vely;  //Fixes weird float rounding error.  Must be computed here before casting to int.  Otherwise, this will miss the bottom collision, but then hit the side collision and the player can slide out of 1x1 spaces.

	flipsidesifneeded();

	if(fyvely + PH < 0.0f)
	{				// on top outside of the screen
		yf(fyvely);
		vely = CapVelocity(GRAVITATION + vely);

		inair = true;
		superjumptimer = 0;
		fallthrough = false;
		onice = false;
		return;
	}
	else if(fyvely + PH >= 480.0f)
	{	//on ground outside of the screen?
		yi(-PH);
		oldy = -PH - 1;
		fallthrough = false;
		onice = false;
		return;
	}

	//printf("Before X - ix: %d\tiy: %d\toldx: %d\toldy: %d\tty: %d\tty2: %d\ttxl: %d\ttxr: %d\tfx: %.2f\tfy: %.2f\tvelx: %.2f\tvely: %.2f\n", ix, iy, oldx, oldy, ty, ty2, ix/TILESIZE, (ix+PW)/TILESIZE, fx, fy, velx, vely);

	//Could be optimized with bit shift >> 5
	int ty = (int)fy / TILESIZE;
	int ty2 = ((int)fy + PH) / TILESIZE;
	int tx = -1;
		
	//-----------------------------------------------------------------
	//  x axis (--)
	//-----------------------------------------------------------------
	if(fy + PH >= 0.0f)
	{
		if(velx > 0)
		{		//moving right	
			tx = (ix + PW) / TILESIZE;	//no special case here when on both sides

			IO_Block * topblock = map.block(tx, ty);
			IO_Block * bottomblock = map.block(tx, ty2);

			//first check to see if player hit a warp
			if(pressdirection[1] && map.warp(tx,ty)->direction == 3 && map.warp(tx,ty2)->direction == 3 && 
				map.warp(tx,ty)->connection == map.warp(tx,ty2)->connection && 
				!map.isconnectionlocked(map.warp(tx,ty)->connection) && !map.isconnectionlocked(map.warp(tx,ty2)->connection))
			{
				xi(tx * TILESIZE - PW - 1);
				enterwarp(map.warp(tx,ty2));
			}
			else if((topblock && !topblock->isTransparent()) || 
				(bottomblock && !bottomblock->isTransparent()))
			{
				if(topblock && !topblock->isTransparent()) //collide with top block
				{	
					topblock->collide(this, 1, true);
					flipsidesifneeded();
				}
				
				if(bottomblock && !bottomblock->isTransparent()) //then bottom
				{	
					bottomblock->collide(this, 1, true);
					flipsidesifneeded();
				}
			}
			else if(map.map(tx, ty) == tile_solid || map.map(tx, ty) == tile_death_on_top || 
				map.map(tx, ty) == tile_death_on_bottom || map.map(tx, ty) == tile_ice ||
				map.map(tx, ty2) == tile_solid || map.map(tx, ty2) == tile_death_on_top || 
				map.map(tx, ty2) == tile_death_on_bottom || map.map(tx, ty2) == tile_ice)
			{	//collision on the right side.

				xi(tx * TILESIZE - PW - 1);			//move to the edge of the tile (tile on the right -> mind the player width)
				oldx = ix;
				velx = 0;

				flipsidesifneeded();
			}
		}
		else if(velx < 0)
		{	//moving left
			if(ix < 0)
			{
				tx = (640 + ix) / TILESIZE;	//test against tiles on the right side of the sceen. x is negativ!
				oldx += 640;
			}
			else
				tx = ix / TILESIZE;

			IO_Block * topblock = map.block(tx, ty);
			IO_Block * bottomblock = map.block(tx, ty2);

			//first check to see if player hit a warp
			if(pressdirection[3] && map.warp(tx,ty)->direction == 1 && map.warp(tx,ty2)->direction == 1 &&
				map.warp(tx,ty)->connection == map.warp(tx,ty2)->connection && 
				!map.isconnectionlocked(map.warp(tx,ty)->connection) && !map.isconnectionlocked(map.warp(tx,ty2)->connection))
			{
				xi(tx * TILESIZE + TILESIZE);
				enterwarp(map.warp(tx,ty2));
			}
			else if((topblock && !topblock->isTransparent()) || 
				(bottomblock && !bottomblock->isTransparent()))
			{
				if(topblock && !topblock->isTransparent()) //collide with top block
				{	
					topblock->collide(this, 3, true);
					flipsidesifneeded();
				}
				
				if(bottomblock && !bottomblock->isTransparent()) //then bottom
				{	
					bottomblock->collide(this, 3, true);
					flipsidesifneeded();
				}
			}
			else if(map.map(tx, ty) == tile_solid || map.map(tx, ty) == tile_death_on_top || 
				map.map(tx, ty) == tile_death_on_bottom || map.map(tx, ty) == tile_ice ||
				map.map(tx, ty2) == tile_solid || map.map(tx, ty2) == tile_death_on_top || 
				map.map(tx, ty2) == tile_death_on_bottom || map.map(tx, ty2) == tile_ice)
			{

				xi(tx * TILESIZE + TILESIZE);			//move to the edge of the tile
				oldx = ix;
				velx = 0;

				flipsidesifneeded();
			}
		}
	}

	//printf("Before Y - ix: %d\tiy: %d\toldx: %d\toldy: %d\tty: %d\tty2: %d\ttxl: %d\ttxr: %d\tfx: %.2f\tfy: %.2f\tvelx: %.2f\tvely: %.2f\n", ix, iy, oldx, oldy, (int)(fyvely) / TILESIZE, ((int)(fyvely) + PH) / TILESIZE, txl, txr, fx, fy, velx, vely);

	//-----------------------------------------------------------------
	//  then y axis (|)
	//-----------------------------------------------------------------

	int txl = -1, txr = -1, txc = -1;

	if(ix < 0)
		txl = (640 + ix) / TILESIZE;
	else
		txl = ix / TILESIZE;

	if(ix + HALFPW < 0)
		txc = (640 + ix + HALFPW) / TILESIZE;
	else
		txc = (ix + HALFPW) / TILESIZE;

	txr = (ix + PW) / TILESIZE;

	//What block is the player aligned to (this will be the block that has the action on it)
	int alignedBlockX = 0;
	int alignedBlockIX = 0;
	int unAlignedBlockX = 0;
	int unAlignedBlockIX = 0;

	int overlaptxl = txl * TILESIZE + TILESIZE + 1;
	if(overlaptxl > 639)
		overlaptxl -= 640;

	if(ix + HALFPW < overlaptxl)
	{
		alignedBlockX = txl;
		alignedBlockIX = txl * TILESIZE + TILESIZE + 1;

		unAlignedBlockX = txr;
		unAlignedBlockIX = txr * TILESIZE - PW - 1;
	}
	else
	{
		alignedBlockX = txr;
		alignedBlockIX = txr * TILESIZE;

		unAlignedBlockX = txl;
		unAlignedBlockIX = txl * TILESIZE + TILESIZE + 1;
	}

	if(vely < 0)
	{	//moving up
		fallthrough = false;

		ty = (int)(fyvely) / TILESIZE;

		IO_Block * leftblock = map.block(txl, ty);
		IO_Block * centerblock = map.block(txc, ty);
		IO_Block * rightblock = map.block(txr, ty);
		
		if(pressdirection[0] && map.warp(alignedBlockX, ty)->direction == 2 && map.warp(unAlignedBlockX, ty)->direction == 2 && 
			map.warp(alignedBlockX, ty)->connection == map.warp(unAlignedBlockX, ty)->connection && 
			!map.isconnectionlocked(map.warp(alignedBlockX, ty)->connection) && !map.isconnectionlocked(map.warp(unAlignedBlockX, ty)->connection))
		{
			yi(ty * TILESIZE + TILESIZE + 1);
			enterwarp(map.warp(unAlignedBlockX,ty));
			return;
		}
		
		if(centerblock && !centerblock->isTransparent())
		{
			if(!centerblock->collide(this, 0, true))
				return;
		}

		if(map.map(alignedBlockX, ty) == tile_solid || map.map(alignedBlockX, ty) == tile_death_on_top || map.map(alignedBlockX, ty) == tile_ice ||
			((invincible || spawninvincible) && (map.map(alignedBlockX, ty) == tile_death_on_bottom || map.map(alignedBlockX, ty) == tile_death)))
		{
			yi(ty * TILESIZE + TILESIZE + 1);
			oldy = iy - 1;
			vely = CapVelocity(-vely * BOUNCESTRENGTH);
			return;
		}

		if(leftblock && !leftblock->isTransparent()) //then left
		{	
			bool useBehavior = alignedBlockX == txl || rightblock == NULL || rightblock->isTransparent();
				
			if(!leftblock->collide(this, 0, useBehavior))
				return;
		}
		
		if(rightblock && !rightblock->isTransparent()) //then right
		{	
			bool useBehavior = alignedBlockX == txr || leftblock == NULL || leftblock->isTransparent();
				
			if(!rightblock->collide(this, 0, useBehavior))
				return;
		}
				
		if(map.map(unAlignedBlockX, ty) == tile_solid || map.map(unAlignedBlockX, ty) == tile_death_on_top || map.map(unAlignedBlockX, ty) == tile_ice ||
			((invincible || spawninvincible) && (map.map(unAlignedBlockX, ty) == tile_death_on_bottom || map.map(unAlignedBlockX, ty) == tile_death)))
		{
			xi(unAlignedBlockIX);
			oldx = ix;
			//velx = 0;  //Why did I have this line here... I bet commenting it out breaks something

			yf(fyvely);
			vely = CapVelocity(GRAVITATION + vely);
		}
		else if(map.map(alignedBlockX, ty) == tile_death_on_bottom || map.map(unAlignedBlockX, ty) == tile_death_on_bottom ||
			map.map(alignedBlockX, ty) == tile_death|| map.map(unAlignedBlockX, ty) == tile_death)
		{
			if( !game_values.gamemode->playerkilledself(*this))
			{
				deathAwards();
				jumpDeath();
			}

			ifsoundonplay(deathsound);
		}
		else
		{
			yf(fyvely);
			vely = CapVelocity(GRAVITATION + vely);
		}
		
		inair = true;
		onice = false;
	}		 
	else
	{	//moving down / on ground
		ty = ((int)fyvely + PH) / TILESIZE;
		
		IO_Block * leftblock = map.block(txl, ty);
		IO_Block * rightblock = map.block(txr, ty);

		if(pressdirection[2] && map.warp(txl,ty)->direction == 0 && map.warp(txr,ty)->direction == 0 &&
			map.warp(txl, ty)->connection == map.warp(txr, ty)->connection && 
			!map.isconnectionlocked(map.warp(txl,ty)->connection) && !map.isconnectionlocked(map.warp(txr,ty)->connection))
		{
			yi(ty * TILESIZE - PH - 1);
			enterwarp(map.warp(txr,ty));

			fallthrough = false;
			return;
		}
		
		if((leftblock && !leftblock->isTransparent()) || 
			(rightblock && !rightblock->isTransparent()))
		{
			bool collisionresult = true;
			if(leftblock && !leftblock->isTransparent()) //collide with left block
			{	
				collisionresult &= leftblock->collide(this, 2, alignedBlockX == txl || rightblock == NULL || rightblock->isTransparent());
			}
			
			if(rightblock && !rightblock->isTransparent()) //then right
			{	
				collisionresult &= rightblock->collide(this, 2, alignedBlockX == txr || leftblock == NULL || leftblock->isTransparent());
			}

			if(!collisionresult)
			{
				onice = false;
				return;
			}
		}
		
		if((map.map(txl, ty) == tile_solid_on_top || map.map(txr, ty) == tile_solid_on_top) && oldy + PH <= ty * TILESIZE)
		{	//on ground
			//Deal with player down jumping through solid on top tiles
			if(fallthrough && map.map(txl, ty) != tile_solid && map.map(txl, ty) != tile_ice && map.map(txl, ty) != tile_death_on_bottom && 
				map.map(txr, ty) != tile_solid && map.map(txr, ty) != tile_ice && map.map(txr, ty) != tile_death_on_bottom)
			{
				yi(ty * TILESIZE - PH + 1);
				inair = true;
			}
			else
			{
				//we were above the tile in the previous frame
				yi(ty * TILESIZE - PH - 1);
				vely = 1;
				
				inair			= false;
				killsinrowinair = 0;
			}

			oldy = iy;
			fallthrough = false;
			onice = false;
			return;
		}
		
		if(map.map(txl, ty) == tile_solid || map.map(txl, ty) == tile_death_on_bottom || map.map(txl, ty) == tile_ice ||
			map.map(txr, ty) == tile_solid || map.map(txr, ty) == tile_death_on_bottom || map.map(txr, ty) == tile_ice ||
			((invincible || spawninvincible) && (map.map(txl, ty) == tile_death_on_top || map.map(txr, ty) == tile_death_on_top ||
			map.map(txl, ty) == tile_death || map.map(txr, ty) == tile_death)))
		{	//on ground

			yi(ty * TILESIZE - PH - 1);
			vely	= 1;				//1 so we test against the ground again int the next frame (0 would test against the ground in the next+1 frame)

			if(map.map(alignedBlockX, ty) == tile_ice || (map.map(unAlignedBlockX, ty) == tile_ice && map.map(alignedBlockX, ty) == tile_nonsolid))
				onice = true;
			else 
				onice = false;

			inair = false;
			killsinrowinair = 0;
		}
		else if(map.map(txl, ty) == tile_death_on_top || map.map(txr, ty) == tile_death_on_top ||
			map.map(txl, ty) == tile_death || map.map(txr, ty) == tile_death)
		{
			if( !game_values.gamemode->playerkilledself(*this))
			{
				deathAwards();
				jumpDeath();
			}

			ifsoundonplay(deathsound);
		}
		else
		{
			//falling (in air)
			yf(fyvely);
			vely = CapVelocity(GRAVITATION + vely);

			superjumptimer = 0;
			inair = true;
		}
	}

	fallthrough = false;
	
	if(inair)
		onice = false;
	
	//printf("After Y - ix: %d\tiy: %d\toldx: %d\toldy: %d\tty: %d\tty2: %d\ttxl: %d\ttxr: %d\tfx: %.2f\tfy: %.2f\tvelx: %.2f\tvely: %.2f\n", ix, iy, oldx, oldy, ty, ty, txl, txr, fx, fy, velx, vely);
}

bool CPlayer::collision_detection_checktop()
{
	if(iy < 0)
		return false;

	int ty = iy / TILESIZE;

	if(ty < 0 || ty >= MAPHEIGHT)
		return false;

	int txl = -1;
	if(ix < 0)
		txl = (640 + ix) / TILESIZE;
	else
		txl = ix / TILESIZE;

	if(txl < 0 || txl >= MAPWIDTH)
		return false;

	int txr = (ix + PW) / TILESIZE;

	if(txr < 0 || txr >= MAPWIDTH)
		return false;
	
	if((map.map(txl, ty) != tile_nonsolid && map.map(txl, ty) != tile_solid_on_top) || 
		(map.map(txr, ty) != tile_nonsolid && map.map(txr, ty) != tile_solid_on_top) ||
		(map.block(txl, ty) && !map.block(txl, ty)->isTransparent()) || 
		(map.block(txr, ty) && !map.block(txr, ty)->isTransparent()))
	{
		yi(ty * TILESIZE + TILESIZE + 1);
		return true;
	}

	return false;
}

bool CPlayer::collision_detection_checkleft()
{
	if(fy < 0.0f)
		return false;

	int ty = (int)fy / TILESIZE;

	if(ty < 0 || ty >= MAPHEIGHT)
		return false;

	int ty2 = ((int)fy + PH) / TILESIZE;

	if(ty2 < 0 || ty2 >= MAPHEIGHT)
		return false;

	int tx = -1;

	if(ix < 0)
		tx = (640 + ix) / TILESIZE;
	else
		tx = ix / TILESIZE;

	if(tx < 0 || tx >= MAPWIDTH)
		return false;

	if((map.map(tx, ty) != tile_nonsolid && map.map(tx, ty) != tile_solid_on_top) ||
		(map.map(tx, ty2) != tile_nonsolid && map.map(tx, ty2) != tile_solid_on_top) ||
		(map.block(tx, ty) && !map.block(tx, ty)->isTransparent()) || 
		(map.block(tx, ty2) && !map.block(tx, ty2)->isTransparent()))
	{
		xi(tx * TILESIZE + TILESIZE);
		flipsidesifneeded();
		return true;
	}

	return false;
}

bool CPlayer::collision_detection_checkright()
{
	if(fy < 0.0f)
		return false;

	int ty = (int)fy / TILESIZE;

	if(ty < 0 || ty >= MAPHEIGHT)
		return false;

	int ty2 = ((int)fy + PH) / TILESIZE;

	if(ty2 < 0 || ty2 >= MAPHEIGHT)
		return false;

	int tx = (ix + PW) / TILESIZE;	//no special case here when on both sides

	if(tx < 0 || tx >= MAPWIDTH)
		return false;

	if((map.map(tx, ty) != tile_nonsolid && map.map(tx, ty) != tile_solid_on_top) ||
		(map.map(tx, ty2) != tile_nonsolid && map.map(tx, ty2) != tile_solid_on_top) ||
		(map.block(tx, ty) && !map.block(tx, ty)->isTransparent()) || 
		(map.block(tx, ty2) && !map.block(tx, ty2)->isTransparent()))
	{
		xi(tx * TILESIZE - PW - 1);
		flipsidesifneeded();
		return true;
	}

	return false;
}

void CPlayer::enterwarp(Warp * warp)
{
	if(warp->direction == 0)
	{
		state = player_entering_warp_down;
		vely = 0.0f;
		velx = 0.0f;
	}
	else if(warp->direction == 1)
	{
		state = player_entering_warp_left;
		vely = 0.0f;
		velx = -1.0f;
	}
	else if(warp->direction == 2)
	{
		state = player_entering_warp_up;
		vely = 0.0f;
		velx = 0.0f;
	}
	else if(warp->direction == 3)
	{
		state = player_entering_warp_right;
		vely = 0.0f;
		velx = 1.0f;
	}

	warpconnection = warp->connection;
	warpid = warp->id;

	map.lockconnection(warpconnection);

	ifsoundonplay(sfx_pipe);
}

void CPlayer::flipsidesifneeded()
{
	if(ix < -PW)
	{		//out of -on-both-sides- area
	   xf(fx + 640.0f); //x is negativ!
	   oldx = oldx + 640;
	}
	else if(ix + PW > 639)
	{		//entered -on-both-sides- area (mind: the screen ends at 639 !!!)
	   xf(fx - 640.0f);
	   oldx = oldx - 640;
	}
}

void CPlayer::makeinvincible()
{
	invincible = true;
	invincibletimer = 0;
	animationstate = 0;
	animationtimer = 0;

	//Stop the invincible music if a player is already invincible 
	//(we don't want two invincible music sounds playing at the same time)
	ifsoundonstop(invinciblemusic);
	ifsoundonstop(timewarning);
	
	iInvincibleMusicChannel = ifsoundonplayloop(invinciblemusic, 2);

	if(game_values.music && game_values.sound)
		backgroundmusic[0].stop();
}

void CPlayer::turnslowdownon()
{
	game_values.slowdownon = teamID;
	game_values.slowdowncounter = 0;

	if(iSlowdownMusicChannel > -1)
		Mix_HaltChannel(iSlowdownMusicChannel);
	
	iSlowdownMusicChannel = ifsoundonplayloop(slowdownmusic, 16);
}
