
#ifndef _PLAYER_H
#define _PLAYER_H

enum PlayerState {player_wait, player_spawning, player_ready, player_dead, player_entering_warp_up, player_entering_warp_right, player_entering_warp_down, player_entering_warp_left, player_exiting_warp_up, player_exiting_warp_right, player_exiting_warp_down, player_exiting_warp_left};

class CInteractionObject;
class OMO_Egg;

struct SScore
{
	int		score;
	
	//Where to display score
	int		x;
	int		y;

	int destx;
	int desty;

	int place;
	int order;  //the order in which the team died

	int fromx;
	int fromy;

};


enum killstyle{kill_style_stomp = 0, kill_style_star = 1, kill_style_fireball = 2, kill_style_bobomb = 3, kill_style_bounce = 4, kill_style_flag = 5, kill_style_goomba = 6, kill_style_bulletbill = 7, kill_style_hammer = 8};
enum awardstyle{award_style_none = 0, award_style_fireworks = 1, award_style_swirl = 2, award_style_halo = 3, award_style_souls = 4, award_style_text = 5};
enum deathstyle{death_style_jump = 0, death_style_squish = 1};


//the player class - a lot of optimization can be done here (especially
//the collision detection stuff in collision_detection_map() and think() )
class CPlayer
{
	public:
		CPlayer(int iGlobalID, int iLocalID, int iTeamID, int iTeamColorID, int keys[6], gfxSprite * nsprites[PGFX_LAST], SScore *nscore, int * respawn, bool cpu);

		void draw();
		void drawarrows();
		void move();
		void mapcollisions();
		void cpu_think();

		void die();

		int getKills(){return score->score;}
		CPlayer * getNearestPlayer(int * distance);
		CInteractionObject * getNearestThreat(int * distance);
		CInteractionObject * getNearestGoal(int * distance);
		CPlayer * getNearestTeammate(int * distance);
		CInteractionObject * getNearestStomp(int * distance);

		void spawnText(char * szText);
		void spawnGameOverMario();

		void deathAwards();
		void jumpDeath();
		void setupNewPlayer();

		bool isready() {return state == player_ready;}
		bool isspawning() {return state == player_spawning;}
		bool iswarping() {return state > player_dead;}
		bool isdead() {return state == player_dead;}

		bool getPlayerPressDirection(int direction);
		int getGlobalID() {return globalID;}

		void bouncejump();

	private:
		void xf(float xf){fx = xf; ix = (int)fx;};
 		void xi(int xi){ix = xi; fx = (float)ix;};
		void yf(float yf){fy = yf; iy = (int)fy;};
		void yi(int yi){iy = yi; fy = (float)iy;};
		
		void findspawnpoint();
		void collision_detection_map();
		bool collision_detection_checktop();
		bool collision_detection_checkleft();
		bool collision_detection_checkright();
		void flipsidesifneeded();
		void fliptopifneeded();
		void makeinvincible();
		void turnslowdownon();
		bool isstomping(CPlayer &o);
		void explodeawards();
		void addswirlingawards();
		void addrocketawards();
		void enterwarp(Warp * warp);
		void chooseWarpExit();
		void increasewarpcounter(int iGoal);
		void decreasewarpcounter();
		
		int kl, kr, kj, kd, kt, kp;		//key config
		bool pressdirection[4];

		SScore *score;
		int killsinrow;
		int killsinrowinair;

		float awardangle;
		int awards[MAXAWARDS];

		int globalID;
		int localID;
		int teamID;
		int colorID;

		int ix, iy;			//x, y coordinate (top left of the player rectangle)
		float fx, fy;
		float velx, vely;	//velocity on x, y axis
		int oldx, oldy;
		
		int xpressed;		//0...no -1... left 1...right
		bool tpressed;

		bool inair;			//true... player is in the air, false... player is on the ground
							//inair is set in CPlayer::collision_detection_map()
		bool onice;			//on ice... friction goes way down

		bool lockjump;		//is the player allowed to jump
		bool lockfall;		//on PC verion, the down key makes player fall through solid_on_top tiles
		bool lockfire;		//the player is allowed to shoot a fireball

		int superjumptimer;		//this is true when a player is able to do a super jump off a note block
		int	hammertimer;		//Only allow the player to throw powerful hammers so fast
		
		int frictionslidetimer;  //this controls how much friction smoke is generated when sliding

		bool cl,cr,cj,cd,ct,cp;		//control for computer
		bool is_cpu;
		int fall_danger;

		gfxSprite *sprites[PGFX_LAST];
		int animationstate;
		int animationtimer;

		int spr;
		int sprswitch;
	
		bool invincible;
		int invincibletimer;

		bool spawninvincible;
		int spawninvincibletimer;

		int burnuptimer;
		int burnupstarttimer;

		int outofarenatimer;
		int outofarenadisplaytimer;

		int powerup;
		bool bobomb;

		PlayerState state;
		float spawnradius;
		float spawnangle;
		int spawntimer;

		int waittimer;
		int * respawncounter;

		int warpcounter;
		int warpconnection;
		int warpid;

		int warpOffsetX;
		int warpOffsetY;
		int warpWidthOffsetLeading;
		int warpHeightOffsetLeading;
		int warpWidthOffsetTrailing;
		int warpHeightOffsetTrailing;

		int powerupused;
		float powerupradius;
		float powerupangle;

		//Pointer to the egg object for Yoshi's Eggs mode
		OMO_Egg * ownedEgg;
		//ID of the player that owns this player for Owned mode
		int ownerPlayerID;
		int ownerColorID;
		//If greater than 0, the player is jailed and moves slowly for Jail mode
		int jailed;

		bool fallthrough;

		int diedas;  //What sprite the player died as 0 == mario, 1 == chocobo, 2 == bobomb

		int spawntext;

		friend bool coldec_player2player(CPlayer &o1, CPlayer &o2);
		friend void collisionhandler_p2p(CPlayer &o1, CPlayer &o2);
		friend void _collisionhandler_p2p_pushback(CPlayer &o1, CPlayer &o2);

		friend bool coldec_player2obj(CPlayer &o1, CInteractionObject &o2);
		friend void collisionhandler_p2o(CPlayer &o1, CInteractionObject &o2);

		friend void killplayer(CPlayer &killer, CPlayer &killed, int deathstyle, int killstyle);
		friend void addawardkill(CPlayer * killer, CPlayer * killed, int killstyle);
		friend void removeplayersbuthighestscoring();
		friend void removeplayersbutteam(int teamID);
		friend void removeteam(int teamid);
		friend void cleandeadplayers();
		friend void stopinvincibilityfordeadplayer(int playerID);
		friend int countaliveteams(int * lastteam);
		friend void rungame();

		friend class CGameMode;
		friend class CGM_DM_Fraglimit;
		friend class CGM_DM_Timelimit;
		friend class CGM_Jail;
		friend class CGM_MarioWar;
		friend class CGM_CaptureTheChicken;
		friend class CGM_Tag;
		friend class CGM_Coins;
		friend class CGM_Fireball;
		friend class CGM_Thwomp;
		friend class CGM_Eggs;
		friend class CGM_Domination;
		friend class CGM_Race;
		friend class CGM_Bobomb;
		friend class CGM_Owned;
		
		friend class IO_Block;
		friend class B_NoteBlock;
		friend class B_BreakableBlock;
		friend class B_PowerupBlock;
		friend class B_DonutBlock;
		friend class B_FlipBlock;
		friend class B_BounceBlock;

		friend class PU_Coin;
		friend class PU_StarPowerup;
		friend class PU_FirePowerup;
		friend class PU_HammerPowerup;
		friend class PU_PoisonPowerup;
		friend class PU_ClockPowerup;
		friend class PU_BobombPowerup;
		friend class PU_PowPowerup;
		friend class PU_BulletBillPowerup;

		friend class PU_FireFlowerCard;
		friend class PU_HammerCard;
		friend class PU_BobombCard;

		friend class OMO_Egg;
		friend class OMO_Yoshi;
		friend class OMO_Thwomp;
		friend class OMO_Area;
		friend class OMO_RaceGoal;
		friend class OMO_Explosion;

		friend class MO_Fireball;
		friend class OMO_Hammer;

		friend class MO_Goomba;
		friend class OMO_BulletBill;
};


void collisionhandler_p2p(CPlayer &o1, CPlayer &o2);
void _collisionhandler_p2p__pushback(CPlayer &o1, CPlayer &o2);

#endif

