#include "sfx.h"

#ifdef _WIN32
	#ifndef _XBOX
		#pragma comment(lib, "SDL_mixer.lib")
	#endif
#endif

extern bool fResumeMusic;
extern void DECLSPEC soundfinished(int channel);
extern void DECLSPEC musicfinished();

bool sfx_init()
{
	Mix_OpenAudio(44100, AUDIO_S16, 2, 2048);
	Mix_AllocateChannels(16);

    return true;
}

void sfx_close()
{
	Mix_CloseAudio();
}

void sfx_stopallsounds()
{
	Mix_HaltChannel(-1);
}

void sfx_setmusicvolume(int volume)
{
	Mix_VolumeMusic(volume);
}

void sfx_setsoundvolume(int volume)
{
	Mix_Volume(-1, volume);
}

sfxSound::sfxSound()
{
	paused = false;
	ready = false;
	sfx = NULL;
}

sfxSound::~sfxSound()
{}

bool sfxSound::init(const char *filename)
{
	if(sfx)
		reset();

	printf("load %s...\n", filename);
	sfx = Mix_LoadWAV(filename);
	
	if(sfx == NULL)
	{
		printf(" failed!\n");
		return false;
	}

	Mix_ChannelFinished(&soundfinished);
	channel = -1;

	ready = true;
	
	return true;
}

int sfxSound::play()
{
	int ticks = SDL_GetTicks();

	//Don't play sounds right over the top (doubles volume)
	if(channel < 0 || ticks - starttime > 40)
	{
		channel = Mix_PlayChannel(-1, sfx, 0);
		starttime = ticks;
	}
	return channel;
}

int sfxSound::playloop(int iLoop)
{
	channel = Mix_PlayChannel(-1, sfx, iLoop);
	return channel;
}

void sfxSound::stop()
{
	if(channel != -1)
	{
		Mix_HaltChannel(channel);
		channel = -1;
	}
}

void sfxSound::pause()
{
	paused = !paused;

	if(paused)
		Mix_Pause(channel);
	else
		Mix_Resume(channel);
}

void sfxSound::reset()
{
	Mix_FreeChunk(sfx);
	sfx = NULL;
	ready = false;
}



sfxMusic::sfxMusic()
{
	paused = false;
	ready = false;
	music = NULL;
}

sfxMusic::~sfxMusic()
{}

bool sfxMusic::load(const char *filename)
{
	if(music)
		reset();

	printf("load %s...\n", filename);
	music = Mix_LoadMUS(filename);
	
	if(!music)
	{
	    printf("Error Loading Music: %s\n", Mix_GetError());
		return false;
	}

	Mix_HookMusicFinished(&musicfinished);

	ready = true;

	return true;
}

void sfxMusic::play(bool fPlayonce, bool fResume)
{
	Mix_PlayMusic(music, (fPlayonce ? 0 : -1));
	fResumeMusic = fResume;
}

void sfxMusic::stop()
{
	Mix_HaltMusic();
}

void sfxMusic::pause()
{
	paused = !paused;

	if(paused)
		Mix_PauseMusic();
	else
		Mix_ResumeMusic();
}

void sfxMusic::reset()
{
	Mix_FreeMusic(music);
	music = NULL;
	ready = false;
}

